<?php

/**
 * SHD SM-VIEW Patch Connector Template API
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class patch_api_model_connector_template extends patch_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'connector_template';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_PATCH_GRID';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_patch_connector_template::instance(isys_application::instance()->database);
    }

    /**
     * Create connector template.
     *
     * $p_param structure:
     * array (
     *     'grid_template_id'  => int,
     *     'connector_title'   => string,
     *     'connector_type'    => int,
     *     'connector_type_id' => int,
     *     'title'             => string,
     *     'number'            => int,
     *     'row_title'         => string,
     *     'pos_x'             => int,
     *     'pos_y'             => int,
     *     'label_orientation' => string,
     *     'status'            => string,
     *     'angle'             => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'grid_template_id',
            'connector_title',
            'connector_type',
            'connector_type_id',
            'pos_x',
            'pos_y',
        );
        $optional_params = array(
            'title',
            'number',
            'row_title',
            'label_orientation',
            'status',
            'angle',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update connector template.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'                => int,
     *     'connector_title'   => string,
     *     'connector_type'    => int,
     *     'connector_type_id' => int,
     *     'title'             => string,
     *     'number'            => int,
     *     'row_title'         => string,
     *     'pos_x'             => int,
     *     'pos_y'             => int,
     *     'label_orientation' => string,
     *     'status'            => string,
     *     'angle'             => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'connector_title',
            'connector_type',
            'connector_type_id',
            'title',
            'number',
            'row_title',
            'pos_x',
            'pos_y',
            'label_orientation',
            'status',
            'angle',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete connector template.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
