<?php

/**
 * SHD SM-VIEW PATCH API
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class patch_api_model_formfactor extends patch_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'patch';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_PATCH_FORMFACTOR';

    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_patch_formfactor::instance(isys_application::instance()->database);
    }

    /**
     * Create formfactor.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'obj_id'  => int,
     *     'half_he' => int,
     *     'comment' => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'obj_id',
        );
        $optional_params = array(
            'half_he',
            'comment',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update formfactor.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'formfactor_id' => int,
     *     'half_he'       => int,
     *     'comment'       => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'half_he',
            'comment',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete formfactor.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
