<?php

/**
 * SHD SM-VIEW Patch Grid API
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class patch_api_model_grid extends patch_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'grid';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_PATCH_GRID';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_patch_grid::instance(isys_application::instance()->database);
    }

    /**
     * List assigned connectors.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'obj_id' => int,
     * )
     *
     * @return array
     */
    public function list_connectors($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $data = $p_params['data'];
        if (empty($data['obj_id'])) {
            return $this->return_error($this->module_name, 'list_connectors', 'could not list connectors - obj_id is missing');
        }
        return $this->dao->get_connectors_by_obj_id($data['obj_id']);
    }
    
    /**
     * Create grid.
     *
     * $p_param structure:
     * array (
     *     'obj_id'           => int,
     *     'description'      => string,
     *     'grid_template_id' => int,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'obj_id',
        );
        $optional_params = array(
            'description',
            'grid_template_id',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update grid.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'               => int,
     *     'description'      => string,
     *     'grid_template_id' => int,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'description',
            'grid_template_id',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete grid.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
