<?php

/**
 * SHD SM-VIEW Patch Grid Connector API
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class patch_api_model_grid_connector extends patch_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'grid_connector';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_PATCH_GRID';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_patch_grid_connector::instance(isys_application::instance()->database);
    }
    
    /**
     * Create grid connector.
     *
     * $p_param structure:
     * array (
     *     'grid_list_id'          => int,
     *     'connector_template_id' => int,
     *     'connector_id'          => int,
     *     'connector_status'      => int,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'grid_list_id',
            'connector_template_id',
            'connector_id',
        );
        $optional_parameters = [
            'connector_status',
        ];
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_parameters);
    }

    /**
     * Update grid connector.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'                    => int,
     *     'connector_template_id' => int,
     *     'connector_id'          => int,
     *     'connector_status'      => int,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'connector_template_id',
            'connector_id',
            'connector_status',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete grid connector.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
