<?php

/**
 * SHD SM-VIEW Patch Grid Template API
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class patch_api_model_grid_template extends patch_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'grid_template';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_PATCH_GRID';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_patch_grid_template::instance(isys_application::instance()->database);
    }
    
    /**
     * Create grid template.
     *
     * $p_param structure:
     * array (
     *     'title'             => string,
     *     'column_count'      => int,
     *     'row_count'         => int,
     *     'column_count_back' => int,
     *     'row_count_back'    => int,
     *     'version'           => string,
     *     'is_template'       => boolean,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'title',
            'column_count',
            'row_count',
        );
        $optional_params = array(
            'column_count_back',
            'row_count_back',
            'version',
            'is_template',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update grid template.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'                => int,
     *     'title'             => string,
     *     'column_count'      => int,
     *     'row_count'         => int,
     *     'column_count_back' => int,
     *     'row_count_back'    => int,
     *     'version'           => string,
     *     'is_template'       => boolean,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'title',
            'column_count',
            'row_count',
            'column_count_back',
            'row_count_back',
            'version',
        //            'is_template', // @todo: do not allow to disable this option
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete grid template.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
