<?php

/**
 * SHD SM-VIEW Parking Deck API
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class patch_api_model_parking_deck implements isys_api_model_interface
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'parking_deck';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_PATCH_PARKING_DECK';

    /**
     * Modules DAO.
     *
     * @var isys_module_dao
     */
    protected $dao;

    /**
     * Indicating whether authentication system should be used during api calls.
     *
     * @var boolean
     */
    protected $useAuth = false;

    /**
     * Constructor.
     */
    function __construct()
    {
        global $g_config;

        // Determine status of authentication system
        $this->useAuth = $g_config['use_auth'];
        $this->dao = isys_cmdb_dao_category_g_smview_patch_parking_deck::instance(isys_application::instance()->database);
    }

    /**
     * Get merged connectors of a local and a parked object.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'local_obj_id'  => int,
     *     'parked_obj_id' => int,
     * )
     *
     * @return array
     * @throws Exception
     */
    public function read($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $local_obj_id = ! empty($p_params['local_obj_id']) ? $p_params['local_obj_id'] : null;
        $parked_obj_id = ! empty($p_params['parked_obj_id']) ? $p_params['parked_obj_id'] : null;
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $connectors = $connector_dao->get_data_as_array(null, $local_obj_id);
        $parkedConnectors = $connector_dao->get_data_as_array(null, $parked_obj_id);
        return [
            'local'  => $this->dao->get_connectors($connectors),
            'parked' => $this->dao->get_connectors($parkedConnectors, true),
        ];
    }

    /**
     * Move connection from local connector to (new) parking deck object.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'connector_id'        => int,
     *     'parking_deck_obj_id' => int,
     *     'title'               => string,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function park_connector($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::EDIT, strtolower($this->category_name));
        }

        $connector_id = ! empty($p_params['connector_id']) ? $p_params['connector_id'] : null;
        $parking_deck_obj_id = ! empty($p_params['parking_deck_obj_id']) ? $p_params['parking_deck_obj_id'] : null;
        $title = ! empty($p_params['title']) ? $p_params['title'] : null;
        return $this->dao->park_connector($connector_id, $parking_deck_obj_id, $title);
    }

    /**
     * Move connection from parked connector to local connector.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'parked_connector_id' => int,
     *     'local_connector_id'  => int,
     *     'mode'                => string,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function connect_connector($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::EDIT, strtolower($this->category_name));
        }

        $parked_connector_id = ! empty($p_params['parked_connector_id']) ? $p_params['parked_connector_id'] : null;
        $local_connector_id = ! empty($p_params['local_connector_id']) ? $p_params['local_connector_id'] : null;
        $mode = ! empty($p_params['mode']) ? $p_params['mode'] : null;
        return $this->dao->connect_connector($parked_connector_id, $local_connector_id, $mode);
    }
}
