<?php

/**
 * SHD SM-VIEW Patch Editor API
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class patch_api_model_patch_editor implements isys_api_model_interface
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'patch_editor';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_PATCH_EDITOR';

    /**
     * Modules DAO.
     *
     * @var isys_module_dao
     */
    protected $dao;

    /**
     * Indicating whether authentication system should be used during api calls.
     *
     * @var boolean
     */
    protected $useAuth = false;

    /**
     * Constructor.
     */
    function __construct()
    {
        global $g_config;

        // Determine status of authentication system
        $this->useAuth = $g_config['use_auth'];
        $this->dao = isys_cmdb_dao_category_g_smview_patch_editor::instance(isys_application::instance()->database);
    }

    /**
     * Read objects.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function read($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        if (isset($p_params["id"])) {
            // get object
            return $this->dao->get_object_by_id($p_params["id"])->get_row();
        }
        return $this->dao->get_data_as_array();
    }

    /**
     * Get rack.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'rack_id'   => int,
     *     'device_id' => int,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function get_rack($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $rack_id = ! empty($p_params['rack_id']) ? $p_params['rack_id'] : null;
        $device_id = ! empty($p_params['device_id']) ? $p_params['device_id'] : null;
        return $this->dao->get_rack($rack_id, $device_id);
    }

    /**
     * Get grid.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'obj_id' => int,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_validation
     */
    public function get_grid($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $obj_id = ! empty($p_params['obj_id']) ? $p_params['obj_id'] : null;
        if (empty($obj_id)) {
            $message = "following parameters are missing: obj_id";
            throw new isys_exception_validation($message, ['obj_id']);
        }
        return $this->dao->get_grid($obj_id);
    }

    /**
     * Get connector.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'connector_id' => int,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_validation
     */
    public function get_connector($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $connector_id = ! empty($p_params['connector_id']) ? $p_params['connector_id'] : null;
        if (empty($connector_id)) {
            $message = "following parameters are missing: connector_id";
            throw new isys_exception_validation($message, ['connector_id']);
        }
        return $this->dao->get_connector_by_id($connector_id);
    }
}
