<?php

/**
 * SHD SM-VIEW Patch Picture API
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class patch_api_model_picture implements isys_api_model_interface
{

    /**
     * Modules DAO.
     *
     * @var isys_cmdb_dao_smview_patch_abstract
     */
    protected $dao;

    /**
     * Indicating whether authentication system should be used during api calls.
     *
     * @var boolean
     */
    protected $useAuth = false;

    /**
     * Constructor.
     */
    function __construct()
    {
        global $g_config;

        // Determine status of authentication system
        $this->useAuth = $g_config['use_auth'];
        $this->dao = isys_cmdb_dao_smview_patch_picture::instance(isys_application::instance()->database);
    }

    /**
     * Read objects.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_general
     */
    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_auth::instance()->module(isys_auth::VIEW, strtolower("C__MODULE__SMVIEW_PATCH"));
        }

        try {
            if (isset($p_params["id"])) {
                // get object
                return $this->dao->get_object('id', $p_params["id"]);
            }
            return $this->dao->get_data();
        } catch (Exception $ex) {
            return array(
                'message'      => "picture.read.error",
                'error'        => $ex->getMessage(),
                'current_date' => date('Y-m-d'),
            );
        }
    }

    /**
     * Update picture assignment.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'     => int,
     *     'obj_id' => int,
     * )
     * @return array
     * @throws isys_exception_auth
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function assign($p_params)
    {
        if ($this->useAuth) {
            isys_auth_auth::instance()->module(isys_auth::EDIT, strtolower("C__MODULE__SMVIEW_PATCH"));
        }

        $instance_id = $p_params['id'];
        $data = $p_params['data'];
        $obj_id = ! empty($data['obj_id']) ? $data['obj_id'] : null;
        
        $params = [
            'obj_id' => $obj_id,
        ];
        
        $row = $this->dao->get_object('id', $instance_id);
        if (empty($row['obj_id'])) {
            $params['connection_type_id'] = $instance_id;
            $instance_id = $this->dao->create_object($params);
        } else {
            if (! empty($obj_id)) {
                $this->dao->update_object($instance_id, $params);
            } else {
                $this->dao->delete_object($instance_id);
            }
        }
        
        return array(
            'message'      => "picture.assign.success",
            'id'           => $instance_id,
            'current_date' => date('Y-m-d'),
        );
    }
}
