import * as React from 'react'
import {DynamicFilter} from '../components/DynamicFilter'
const ReactTestRenderer = require('react-test-renderer');

const lang = {
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_FILTER']: 'Filter',
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_RESOLUTION']: 'Resolution',
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_UPDATE']: 'Update',
    ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_RACK_VIEW']: 'Show rack view',
    ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_CAPACITY']: 'Show capacity',
    ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_ONLY_CONNECTED_NODES']: 'Show only connected nodes',
    ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_NODE_IMAGE']: 'Show node images',
    ['LC__CMDB__CATG__SMVIEW_PATCH__MARK_NEIGHBOR_EDGES']: 'Mark neighbor edges',
    ['LC__CMDB__CATG__SMVIEW_PATCH__FILTER_CONNECTION_TYPE']: 'Connection type',
    ['LC__CMDB__CATG__SMVIEW_PATCH__FILTER_CONNECTOR_TYPE']: 'Connector type',
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_PRESET']: 'Preset',
    ['LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES']: 'Object types',
};
const levels = [
    {
        id: 4,
        title: 'EG',
        type_id: 1,
        type_const: 'C__CATG__FLOOR',
        type_title: 'Floor',
        type_color: '#123456',
        objects: []
    },
    {
        id: 5,
        title: 'Serverraum',
        type_id: 2,
        type_const: 'C__CATG__ROOM',
        type_title: 'Room',
        type_color: '#123456',
        objects: [
            {
                id: 5,
                title: 'Serverraum',
                type_id: 1,
                type_const: 'C__CATG__ROOM',
                type_title: 'Room',
                type_color: '#123456',
            },
            {
                id: 6,
                title: 'Labor',
                type_id: 1,
                type_const: 'C__CATG__ROOM',
                type_title: 'Room',
                type_color: '#123456',
            }
        ]
    }
];
const object_types_next_level = [
    {
        id: 10,
        const: 'C__CATG__RACK',
        title: 'Rack',
        color: '#123456',
        objects: [
            {
                id: 111,
                title: 'Schrank01',
                type_id: 10,
                type_const: 'C__CATG__RACK',
                type_title: 'Rack',
                type_color: '#123456',
            },
            {
                id: 222,
                title: 'Schrank02',
                type_id: 10,
                type_const: 'C__CATG__RACK',
                type_title: 'Rack',
                type_color: '#123456',
            }
        ]
    },
    {
        id: 11,
        const: 'C__CATG__DEVICE',
        title: 'Device',
        color: '#123456',
        objects: []
    }
];

it('renders disabled filter correctly', () => {
    const component = ReactTestRenderer.create(
        <DynamicFilter language={lang}
                       levels={levels}
                       object_types_next_level={[]}
                       is_loading={true}
                       onChange={null}
                       onAddFilter={null}
                       onObjectTypeSelected={null}
                       onRemoveLastFilter={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders enabled filter correctly', () => {
    const mockChangeFunction = jest.fn();
    const mockAddFilterFunction = jest.fn();
    const mockRemoveLastFilterFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <DynamicFilter language={lang}
                       levels={levels}
                       object_types_next_level={[]}
                       is_loading={false}
                       onChange={mockChangeFunction}
                       onAddFilter={mockAddFilterFunction}
                       onObjectTypeSelected={null}
                       onRemoveLastFilter={mockRemoveLastFilterFunction} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let selects = instance.findAllByProps({className: "input input-small"});
    let select = selects[1]
    select.props.onChange({
        currentTarget: { value: '6' }
    });
    expect(mockChangeFunction).toHaveBeenCalledWith('6', '2');

    let button = instance.findByProps({className: "btn add"});
    button.props.onClick();
    expect(mockAddFilterFunction).toHaveBeenCalled();

    button = instance.findByProps({className: "btn remove"});
    button.props.onClick({
        currentTarget: { parentElement: { id: 'type_2' } }
    });
    expect(mockRemoveLastFilterFunction).toHaveBeenCalledWith(2);
});

it('renders next level filter correctly', () => {
    const mockObjectTypeSelectedFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <DynamicFilter language={lang}
                       levels={levels}
                       object_types_next_level={object_types_next_level}
                       is_loading={false}
                       onChange={null}
                       onAddFilter={null}
                       onObjectTypeSelected={mockObjectTypeSelectedFunction}
                       onRemoveLastFilter={null} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let selects = instance.findAllByType("select");
    let select = selects[2]
    select.props.onChange({
        currentTarget: { value: '2' }
    });
    expect(mockObjectTypeSelectedFunction).toHaveBeenCalledWith(2);
});
