import * as React from 'react'
import {PresetSelector} from '../components/PresetSelector'
const ReactTestRenderer = require('react-test-renderer');

const lang = {
  ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_FILTER']: 'Filter',
  ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_RESOLUTION']: 'Resolution',
  ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_UPDATE']: 'Update',
  ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_RACK_VIEW']: 'Show rack view',
  ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_CAPACITY']: 'Show capacity',
  ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_ONLY_CONNECTED_NODES']: 'Show only connected nodes',
  ['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_NODE_IMAGE']: 'Show node images',
  ['LC__CMDB__CATG__SMVIEW_PATCH__MARK_NEIGHBOR_EDGES']: 'Mark neighbor edges',
  ['LC__CMDB__CATG__SMVIEW_PATCH__FILTER_CONNECTION_TYPE']: 'Connection type',
  ['LC__CMDB__CATG__SMVIEW_PATCH__FILTER_CONNECTOR_TYPE']: 'Connector type',
  ['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_PRESET']: 'Preset',
  ['LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES']: 'Object types',
};
const presets = [
  {
    id: 1,
    description: 'Rooms',
    show_rack_view: false,
    object_types: {
      ['C__OBJTYPE__ROOM']: 'Room',
    }
  },
  {
    id: 2,
    description: 'Racks',
    show_rack_view: true,
    object_types: {
      ['C__OBJTYPE__ENCLOSURE']: 'Rack',
    }
  },
  {
    id: 3,
    description: 'Devices',
    show_rack_view: false,
    object_types: {
      ['C__OBJTYPE__PATCH_PANEL']: 'Patch Panel',
      ['C__OBJTYPE__SWITCH']: 'Switch',
    }
  }
];

it('renders disabled preset selector correctly', () => {
  const component = ReactTestRenderer.create(
      <PresetSelector language={lang}
                      is_loading={true}
                      presets={presets}
                      onChange={null} />,
  );
  expect(component).toMatchSnapshot();
});

it('renders enabled preset selector correctly', () => {
  const mockFunction = jest.fn();
  const component = ReactTestRenderer.create(
      <PresetSelector language={lang}
                      is_loading={false}
                      presets={presets}
                      selected_preset={3}
                      onChange={mockFunction} />,
  );
  expect(component).toMatchSnapshot();

  const instance = component.root;
  let select = instance.findByProps({className: "input input-small"});
  select.props.onChange({
    currentTarget: { value: '2' }
  });
  expect(mockFunction).toHaveBeenCalledWith(2);
});

it('renders invalid selection on preset selector correctly', () => {
  const component = ReactTestRenderer.create(
      <PresetSelector language={lang}
                      is_loading={false}
                      presets={presets}
                      selected_preset={88}
                      onChange={null} />,
  );
  expect(component).toMatchSnapshot();
});
