import * as React from "react";
import {Select, SelectElement} from "./Select";

export interface ObjectTypeElement {
  id: number,
  title: string,
  const: string,
  color: string,
  objects: SelectElement[],
}

export interface DynamicLevel extends SelectElement {
  objects: SelectElement[]
}

export interface DynamicFilterProps {
  levels: DynamicLevel[],
  object_types_next_level: ObjectTypeElement[],
  language: {[key: string]: string},
  is_loading: boolean,
  onChange: Function,
  onAddFilter: Function,
  onRemoveLastFilter: Function,
  onObjectTypeSelected: Function,
}

export class DynamicFilter extends React.Component<DynamicFilterProps, {}> {
  constructor(props: DynamicFilterProps) {
    super(props);
    this.handleAddFilter = this.handleAddFilter.bind(this)
    this.handleObjectTypeSelection = this.handleObjectTypeSelection.bind(this)
    this.handleRemoveLastFilter = this.handleRemoveLastFilter.bind(this)
  }

  handleAddFilter() {
    this.props.onAddFilter();
  }

  handleRemoveLastFilter(event: React.MouseEvent<HTMLButtonElement>) {
    let id = event.currentTarget.parentElement.id
    let value = parseInt(id.substring(5))
    this.props.onRemoveLastFilter(value);
  }

  handleObjectTypeSelection(event: React.ChangeEvent<HTMLSelectElement>) {
    let value = parseInt(event.currentTarget.value)
    if (value > 0) {
      this.props.onObjectTypeSelected(value)
    }
  }

  render() {
    let lis = [];
    let index = 1
    this.props.levels.forEach(level => {
      const elements = level.objects
      const selected_item = level.id
      let removeButton = null
      if (index == this.props.levels.length) {
        // only last filter can be removed
        removeButton = <button type="button"
                               className="btn remove"
                               onClick={this.handleRemoveLastFilter}>-</button>
      }

      lis.push(<li key={level.type_id} id={'type_' + level.type_id}>
        <label>{level.type_title}:</label>
        <Select opts={elements}
                onChange={this.props.onChange}
                selected_item={selected_item}
                disabled={this.props.is_loading}
                level={level.type_id.toString()} />
        {removeButton}
      </li>)
      index++
    })

    if (this.props.object_types_next_level && this.props.object_types_next_level.length > 0) {
      // render next level selector
      let object_type_elements = [
          <option key={'new_object_type_0'} value={-1}>-</option>
      ]
      this.props.object_types_next_level.forEach(object_type => {
        object_type_elements.push(<option key={'new_object_type_' + object_type.id}
                                          value={object_type.id}>{object_type.title}</option>)
      })

      lis.push(<li key="new_filter">
        <label>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES}:</label>
        <select onChange={this.handleObjectTypeSelection}
                disabled={this.props.is_loading}
                className="input input-small">{object_type_elements}</select>
      </li>)
    } else {
      // render add button
      lis.push(<li key="add_filter">
        <button type="button"
                className="btn add"
                onClick={this.handleAddFilter}>+</button>
      </li>)
    }

    return <div className="filter">
      <h3>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_FILTER']}</h3>
      <ul>
        {lis}
      </ul>
    </div>;
  }
}
