'use strict';

import 'yfiles/view-layout-bridge.js';
import {INode, IRenderContext, NodeStyleBase, Rect, SimpleNode} from 'yfiles/view-component';
import { ImageNodeStyle } from 'yfiles/styles-other';

export class CustomImageNodeStyle extends NodeStyleBase {

    private readonly baseUrl: string;
    private readonly imageStyle: ImageNodeStyle;
    private imageNode: SimpleNode;
    private readonly level: string;

    constructor(baseUrl: string, level?: string) {
        super();
        this.baseUrl = baseUrl;
        this.level = level;
        this.imageStyle = new ImageNodeStyle();

        this.imageNode = new SimpleNode();
        this.imageNode.layout = new Rect(0, 0, 24, 24);
    }

    getImageUrl() {
        if (! this.level) {
            return this.baseUrl;
        }

        let url = this.baseUrl + 'icons/silk/';
        switch (this.level) {
            case 'location': url += 'world.png'; break;
            case 'building': url += 'building.png'; break;
            case 'floor': url += 'layers.png'; break;
            case 'room': url += 'house.png'; break;
            case 'rack': url += 'drive_stack.png'; break;
            case 'device': url += 'computer.png'; break;
            default: url += 'box.png';
        }
        return url;
    }

    static getImageNode(node: INode) {
        let imageNode = new SimpleNode();
        const layout = node.layout.toRect();
        imageNode.layout = new Rect(0, 0, layout.width, layout.height);
        return imageNode;
    }

    createVisual(context: IRenderContext, node: INode) {
        const layout = node.layout.toRect();

        // create the decoration
        this.imageStyle.image = this.getImageUrl();
        this.imageNode = CustomImageNodeStyle.getImageNode(node);
        this.imageNode.layout = this.getImageNodeLayout(layout);
        let decorationRenderer = this.imageStyle.renderer.getVisualCreator(this.imageNode, this.imageStyle);
        return decorationRenderer.createVisual(context);
    }

    getImageNodeLayout(nodeLayout: Rect) {
        const size = this.imageNode.layout.toSize();
        return new Rect(nodeLayout.x, nodeLayout.y, size.width, size.height);
    }

}
