import * as React from "react";

export interface Preset {
  id: number,
  description: string,
  show_rack_view: boolean,
  object_types: {[key: string]: string}
}

export interface PresetSelectorProps {
  onChange: Function,
  language: {[key: string]: string},
  is_loading: boolean,
  presets: Preset[],
  selected_preset?: number,
}

export class PresetSelector extends React.Component<PresetSelectorProps, {}> {
  constructor(props: PresetSelectorProps) {
    super(props);
    this.handleSelection = this.handleSelection.bind(this);
  }

  handleSelection(event: React.ChangeEvent<HTMLSelectElement>) {
    let value = parseInt(event.currentTarget.value)
    this.props.onChange(value);
  }

  render() {
    const options = this.props.presets.map(preset => <option key={preset.id} value={preset.id}>{preset.description}</option>)

    // list object types of selected preset
    let object_types = []
    if (this.props.selected_preset) {
      const presets = this.props.presets.filter(p => p.id == this.props.selected_preset)
      if (presets.length > 0) {
        const preset = presets[0]
        for (let k in preset.object_types) {
          if (preset.object_types.hasOwnProperty(k) ) {
            object_types.push(<span key={k}>{preset.object_types[k]}</span>)
          }
        }
      }
    }

    return <div className="preset">
      <h3>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_RESOLUTION']}</h3>
      <ul>
        <li>
          <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_PRESET']}:</label>
          <select className="input input-small"
                  disabled={this.props.is_loading}
                  onChange={this.handleSelection}
                  value={this.props.selected_preset}>{options}</select>
        </li>
        <li>
          <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES']}:</label>
          {object_types}
        </li>
      </ul>
    </div>;
  }
}
