import * as React from "react";

export interface SelectProps {
    opts: SelectElement[],
    onChange: Function,
    level: string,
    selected_item?: number,
    disabled?: boolean,
}

export interface SelectElement {
    id: number,
    title: string,
    type_id: number,
    type_const: string,
    type_title: string,
    type_color: string,
    connections?: Connection[],
}

export interface Connection {
    sibling: number,
    used: number,
    count: number,
    cable_color: string,
}

export class Select extends React.Component<SelectProps, {}> {
    constructor(props: SelectProps) {
        super(props);
        this.handleChange = this.handleChange.bind(this);
    }

    handleChange (event: React.ChangeEvent<HTMLSelectElement>) {
        this.props.onChange(event.currentTarget.value, this.props.level);
    }

    render() {
        // convert to sortable array
        const sortables = []
        for(let key in this.props.opts) {
            if (this.props.opts.hasOwnProperty(key)) {
                const item = this.props.opts[key]
                sortables.push([item.id, item.title])
            }
        }

        // sort array
        sortables.sort(function (a, b) {
            return (a[1].toString() > b[1].toString()) ? 1 : -1
        })

        let options = [];
        options.push(<option key="-1" value='-1'>-</option>);
        for (const c in sortables) {
            if (sortables.hasOwnProperty(c)) {
                const row = sortables[c]
                const i:number = parseInt(row[0])
                options.push(<option key={i} value={i}>{row[1]}</option>)
            }
        }

        return <select className="input input-small"
                       disabled={this.props.disabled}
                       onChange={this.handleChange}
                       value={this.props.selected_item ? this.props.selected_item : '-1'}>{options}</select>;
    }
}
