import * as React from "react";

export interface SelectProps {
    opts: {[key: number]: string},
    onChange: Function,
    has_empty_option?: boolean,
    disabled?: boolean,
}

export class SimpleSelect extends React.Component<SelectProps, {}> {
    constructor(props: SelectProps) {
        super(props);
        this.handleChange = this.handleChange.bind(this);
    }

    handleChange (event: React.ChangeEvent<HTMLSelectElement>) {
        this.props.onChange(event.currentTarget.value);
    }

    render() {
        // convert to sortable array
        const sortables = []
        for(let key in this.props.opts) {
            if (this.props.opts.hasOwnProperty(key)) {
                const opt = this.props.opts[key]
                sortables.push([key, opt])
            }
        }

        // sort array
        sortables.sort(function (a, b) {
            return (a[1].toString() > b[1].toString()) ? 1 : -1
        })

        let options = [];
        if (this.props.has_empty_option) {
            options.push(<option key="-1" value='-1'>-</option>);
        }

        for (const c in sortables) {
            if (sortables.hasOwnProperty(c)) {
                const row = sortables[c]
                const i:number = parseInt(row[0])
                options.push(<option key={i} value={i}>{row[1]}</option>)
            }
        }
        return <select className="input input-small"
                       disabled={this.props.disabled}
                       onChange={this.handleChange}>{options}</select>;
    }
}