import * as React from 'react'
import {AngleSelector} from '../components/AngleSelector'
const ReactTestRenderer = require('react-test-renderer');

it('renders angle selector correctly', () => {
    const lang = {
        ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT']: 'Front',
        ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK']: 'Back',
    };
    const mockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <AngleSelector angle="F" onAngleChange={mockFunction} language={lang} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let select = instance.findByProps({className: "input angle_selector"});
    select.props.onChange({
        currentTarget: { value: '8' }
    });
    expect(mockFunction).toHaveBeenCalledWith('8');
});

it('renders angle selector vertical correctly', () => {
    const lang = {
        ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT']: 'Front',
        ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK']: 'Back',
        ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT_VERTICAL']: 'Front vertical',
        ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK_VERTICAL']: 'Back vertical',
    };
    const mockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <AngleSelector angle="F" onAngleChange={mockFunction} language={lang} addFrontVerticalOption={true} addBackVerticalOption={true} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let select = instance.findByProps({className: "input angle_selector"});
    select.props.onChange({
        currentTarget: { value: 'FB' }
    });
    expect(mockFunction).toHaveBeenCalledWith('FB');
});