import * as React from 'react'
import {Device} from '../components/Device/Device'
const ReactTestRenderer = require('react-test-renderer');

const lang = {
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT']: 'Front',
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK']: 'Back',
    ['LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED']: 'connected',
    ['LC__CMDB__CATG__SMVIEW_PATCH__NOT_CONNECTED']: 'not connected',
    ['LC__CMDB__CATG__SMVIEW_PATCH__OPEN_OBJECT']: 'Open object',
    ['LC__CMDB__CATG__SMVIEW_PATCH__PRINT_DEVICE_DATA']: 'Print device data',
    ['LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_LIST_VIEW']: 'Toggle list view',
};

const colors = {
    ['color_patch']: '00CC00',
    ['color_link']: 'CCCC00',
    ['color_socket']: '00CCCC'
};

const connector_status_ok = {
    title: 'ok',
    color: '#00FF00'
};

const connector_status_warning = {
    title: 'warning',
    color: '#FFFF00'
};

const status_list = {
    [1]: connector_status_ok,
    [2]: connector_status_warning,
};

const common_device = {
    id: 1,
    title: 'PP01',
    description: 'Patch Panel 01',
    type_title: 'PatchPanel',
    url: '/',
    print_url: '/',
    grid_title: 'Grid01',
    column_count: 4,
    row_count: 1,
    is_template: false
};

const common_device_back = {
    id: 1,
    title: 'PP01',
    description: 'Patch Panel 01',
    type_title: 'PatchPanel',
    url: '/',
    print_url: '/',
    grid_title: 'Grid01',
    column_count: 4,
    row_count: 1,
    column_count_back: 6,
    row_count_back: 2,
    is_template: false
};

it('renders normal device correctly', () => {
    const connectors = [
        {
            id: 1,
            connector_id: 1,
            connector_title: '01',
            connector_type: 1,
            connector_type_id: 1,
            connector_type_title: 'RJ45',
            connector_status: 1,
            pos_x: 1,
            pos_y: 1,
            rotation: 0,
            angle: 'F',
            status: 'ok'
        }
    ];
    const component = ReactTestRenderer.create(
        <Device language={lang}
                angle="F"
                colors={colors}
                connectors={connectors}
                device={common_device}
                image_url=""
                side="left"
                size="M"
                status_list={status_list}
                use_cable_color={false}
                right_align_connector_titles={false}
                selected_port_id={1}
                onAngleChange={null}
                onPortSelectionChange={null}
                onSlotSelected={null}
                onActivatePlannedConnections={null}
                is_planner_mode={false}
                has_planner_activate_permission={true} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders normal device backside correctly', () => {
    const component = ReactTestRenderer.create(
        <Device language={lang}
                angle="B"
                colors={colors}
                connectors={[]}
                device={common_device_back}
                image_url=""
                side="left"
                size="M"
                status_list={status_list}
                use_cable_color={false}
                right_align_connector_titles={false}
                selected_port_id={1}
                onAngleChange={null}
                onPortSelectionChange={null}
                onSlotSelected={null}
                onActivatePlannedConnections={null}
                is_planner_mode={false}
                has_planner_activate_permission={true} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders chassis device correctly', () => {
    const slot_devices1 = [
        {
            id: 1,
            title: "Device1",
            image_url: "URL1",
        }
    ];
    const connectors = [
        {
            id: 1,
            connector_id: 1,
            connector_title: '01',
            connector_type: 1,
            connector_type_id: 1,
            connector_type_title: 'RJ45',
            connector_status: 2,
            pos_x: 1,
            pos_y: 1,
            rotation: 0,
            angle: 'F',
            status: 'ok'
        },
        {
            id: 2,
            connector_id: 2,
            connector_title: '02',
            connector_type: 1,
            connector_type_id: 1,
            connector_type_title: 'RJ45',
            connector_status: 1,
            pos_x: 2,
            pos_y: 1,
            rotation: 0,
            angle: 'F',
            status: 'ok'
        }
    ];
    const slots = [
        {
            id: 1,
            title: "Slot1",
            x_from: 1,
            x_to: 1,
            y_from: 0,
            y_to: 0,
            devices: slot_devices1
        }
    ];
    const chassis = {
        id: 1111,
        title: 'Chassis01',
        width: 4,
        height: 2,
        slots: slots,
    };
    const device = {
        id: 1,
        title: 'CP01',
        type_title: 'ChassisPanel',
        url: '/',
        print_url: '/',
        grid_title: 'Grid01',
        column_count: 4,
        row_count: 1,
        is_template: false,
        is_chassis: true,
        chassis_front: chassis,
    };
    const component = ReactTestRenderer.create(
        <Device language={lang}
                angle="F"
                colors={colors}
                connectors={connectors}
                device={device}
                image_url=""
                side="left"
                size="L"
                status_list={status_list}
                use_cable_color={false}
                right_align_connector_titles={false}
                selected_port_id={1}
                onAngleChange={null}
                onPortSelectionChange={null}
                onSlotSelected={null}
                onActivatePlannedConnections={null}
                is_planner_mode={false}
                has_planner_activate_permission={true} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders cable color correctly', () => {
    const patch = {
        id: 1,
        title: '02',
        obj_id: 1234,
        obj_title: 'PP01',
        obj_url: '/',
        cable_id: 14,
        cable_title: 'Default-Cable14',
        cable_length: '100m',
        cable_color_id: 1,
        cable_color: '#FF0000',
    };
    const connectors = [
        {
            id: 8,
            connector_id: 8,
            connector_title: '08',
            connector_type: 1,
            connector_type_id: 1,
            connector_type_title: 'RJ45',
            connector_status: 1,
            pos_x: 2,
            pos_y: 1,
            rotation: 0,
            angle: 'F',
            status: 'ok',
            patch: patch,
            link_type: 'link',
            url: 'http://connector08',
        }
    ];
    const selectionMockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <Device language={lang}
                angle="F"
                colors={colors}
                connectors={connectors}
                device={common_device}
                image_url="http://img/"
                side="left"
                size="M"
                status_list={status_list}
                use_cable_color={true}
                right_align_connector_titles={false}
                selected_port_id={1}
                onAngleChange={null}
                onPortSelectionChange={selectionMockFunction}
                onSlotSelected={null}
                onActivatePlannedConnections={null}
                is_planner_mode={false}
                has_planner_activate_permission={true} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let img = instance.findByProps({id: "8"});
    img.props.onClick({
        currentTarget: { id: '8' }
    });
    expect(selectionMockFunction).toHaveBeenCalledWith("8", "left");

    let button = instance.findAllByProps({className: "icon-button"});
    button[0].props.onClick();
    // expect(selectionMockFunction).toHaveBeenCalledWith("8", "left");
});

it('renders cable pins correctly', () => {
    const connectors = [
        {
            id: 10,
            connector_id: 10,
            connector_title: '10',
            connector_type: 1,
            connector_type_id: 1,
            connector_type_title: 'RJ45',
            connector_status: 1,
            pos_x: 2,
            pos_y: 1,
            rotation: 0,
            angle: 'F',
            status: 'ok',
            cable_pins: [
                {
                    id: 1,
                    title: "01",
                    obj_title: "Switch",
                    obj_url: "http://switch",
                }
            ],
            link_type: 'link',
        }
    ];
    const component = ReactTestRenderer.create(
        <Device language={lang}
                angle="F"
                colors={colors}
                connectors={connectors}
                device={common_device}
                image_url=""
                side="left"
                size="M"
                status_list={status_list}
                use_cable_color={true}
                right_align_connector_titles={false}
                selected_port_id={1}
                onAngleChange={null}
                onPortSelectionChange={null}
                onSlotSelected={null}
                onActivatePlannedConnections={null}
                is_planner_mode={false}
                has_planner_activate_permission={true} />,
    );
    expect(component).toMatchSnapshot();
});
