import * as React from 'react'
import {DevicePortIcon} from '../components/Device/DevicePortIcon'
const ReactTestRenderer = require('react-test-renderer');

const lang = {
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT']: 'Front',
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK']: 'Back',
    ['LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED']: 'connected',
    ['LC__CMDB__CATG__SMVIEW_PATCH__NOT_CONNECTED']: 'not connected',
    ['LC__CMDB__CATG__SMVIEW_PATCH__OPEN_OBJECT']: 'Open object',
    ['LC__CMDB__CATG__SMVIEW_PATCH__PRINT_DEVICE_DATA']: 'Print device data',
    ['LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_LIST_VIEW']: 'Toggle list view',
};

const colors = {
    ['color_patch']: '00CC00',
    ['color_link']: 'CCCC00',
    ['color_socket']: '00CCCC'
};

const connector_status_ok = {
    title: 'ok',
    color: '#00FF00'
};

const connector_status_warning = {
    title: 'warning',
    color: '#FFFF00'
};

const status_list = {
    [1]: connector_status_ok,
    [2]: connector_status_warning,
};

it('renders selected device port correctly', () => {
    const connector = {
        id: 1,
        connector_id: 1,
        connector_title: '01',
        connector_type: 1,
        connector_type_id: 1,
        connector_type_title: 'RJ45',
        connector_status: 1,
        pos_x: 1,
        pos_y: 1,
        rotation: 0,
        angle: 'F',
        status: 'ok'
    };
    const component = ReactTestRenderer.create(
        <DevicePortIcon index={1} language={lang} colors={colors} connector={connector} status_list={status_list}
                    use_cable_color={false} right_align_connector_titles={false} is_planner_mode={false} is_selected={true} onSelectedIdChange={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders patch status of device port correctly', () => {
    const patch = {
        id: 1,
        title: '02',
        obj_id: 1234,
        obj_title: 'PP01',
        obj_url: '/',
        cable_id: 14,
        cable_title: 'Default-Cable14',
        cable_length: '100m',
        cable_color_id: 1,
        cable_color: '#FF0000'
    };
    const connector = {
        id: 8,
        connector_id: 8,
        connector_title: '08',
        connector_type: 1,
        connector_type_id: 1,
        connector_type_title: 'RJ45',
        connector_status: 1,
        pos_x: 2,
        pos_y: 1,
        rotation: 0,
        angle: 'F',
        status: 'ok',
        patch: patch,
        link_type: 'link',
    };
    const component = ReactTestRenderer.create(
        <DevicePortIcon index={1} language={lang} colors={colors} connector={connector} status_list={status_list}
                    use_cable_color={false} right_align_connector_titles={false} is_planner_mode={false} is_selected={false} onSelectedIdChange={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders cable color of device port correctly', () => {
    const patch = {
        id: 1,
        title: '02',
        obj_id: 1234,
        obj_title: 'PP01',
        obj_url: '/',
        cable_id: 14,
        cable_title: 'Default-Cable14',
        cable_length: '100m',
        cable_color_id: 1,
        cable_color: '#FF0000'
    };
    const link = {
        id: 2,
        title: '14',
        obj_id: 4321,
        obj_title: 'Device01',
        obj_url: '/',
        cable_id: 18,
        cable_title: 'Default-Cable18',
        cable_length: '100m',
        cable_color_id: 2,
        cable_color: '#6688AA'
    };
    const connector = {
        id: 8,
        connector_id: 8,
        connector_title: '08',
        connector_type: 1,
        connector_type_id: 1,
        connector_type_title: 'RJ45',
        connector_status: 1,
        pos_x: 2,
        pos_y: 1,
        rotation: 0,
        angle: 'F',
        status: 'ok',
        patch: patch,
        link_type: 'link',
        link: link,
    };
    const component = ReactTestRenderer.create(
        <DevicePortIcon index={1} language={lang} colors={colors} connector={connector} status_list={status_list}
                    use_cable_color={true} right_align_connector_titles={false} is_planner_mode={false} is_selected={false} onSelectedIdChange={null} />,
    );
    expect(component).toMatchSnapshot();
});
