import * as React from 'react'
import {Port} from '../components/Port/Port'
const ReactTestRenderer = require('react-test-renderer');

const lang = {
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT']: 'Front',
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK']: 'Back',
    ['LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED']: 'connected',
    ['LC__CMDB__CATG__SMVIEW_PATCH__NOT_CONNECTED']: 'not connected',
    ['LC__CMDB__CATG__SMVIEW_PATCH__OPEN_OBJECT']: 'Open object',
    ['LC__CMDB__CATG__SMVIEW_PATCH__PRINT_DEVICE_DATA']: 'Print device data',
    ['LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_LIST_VIEW']: 'Toggle list view',
};

const colors = {
    ['color_patch']: '00CC00',
    ['color_link']: 'CCCC00',
    ['color_socket']: '00CCCC'
};

const cable_types = {
    [1]: 'CAT5',
    [2]: 'CAT6',
    [3]: 'CAT7'
};

const connector_status_ok = {
    title: 'ok',
    color: '#00FF00'
};

const connector_status_warning = {
    title: 'warning',
    color: '#FFFF00'
};

const status_list = {
    [1]: connector_status_ok,
    [2]: connector_status_warning,
};

const types = {
    [1]: "RJ45",
    [2]: "RJ11",
};

const link = {
    id: 1,
    title: "01",
    obj_id: 4,
    obj_title: "Switch",
    obj_url: "",
    cable_id: 8,
    cable_title: "Cable01",
    cable_length: "100m",
    cable_color_id: 1,
    cable_color: "#FF0000",
};

const permissions = {
    has_edit_permission: true,
    has_planner_edit_permission: true,
    has_planner_activate_permission: true,
    has_end_device_edit_permission: true,
}

it('renders port correctly', () => {
    const component = ReactTestRenderer.create(
        <Port language={lang} connector_id={1} connector_title="Connector01" connector_type={1} connector_type_id={1}
              connector_type_title="RJ45" connector_status={1} status_list={status_list} cable_colors={colors} permissions={permissions}
              cable_types={cable_types} connector_types={types} image_url="/" side="left" is_planner_mode={false} sibling_id={8} link={link} patch={{}}
              onCablePropertyChange={null} onCableSave={null} onConnectorStatusChange={null} onConnectorTypeChange={null} onSocketClick={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders no link component without sibling ID', () => {
    const component = ReactTestRenderer.create(
        <Port language={lang} connector_id={1} connector_title="Connector01" connector_type={1} connector_type_id={1}
              connector_type_title="RJ45" connector_status={1} status_list={status_list} cable_colors={colors} permissions={permissions}
              cable_types={cable_types} connector_types={types} image_url="/" side="left" is_planner_mode={false} link={link} patch={{}}
              onCablePropertyChange={null} onCableSave={null} onConnectorStatusChange={null} onConnectorTypeChange={null} onSocketClick={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders empty port correctly', () => {
    const component = ReactTestRenderer.create(
        <Port language={lang} connector_id={-1} connector_title="" connector_type={0} connector_type_id={0}
              connector_type_title="" connector_status={0} status_list={status_list} cable_colors={colors} permissions={permissions}
              cable_types={cable_types} connector_types={types} image_url="/" side="left" is_planner_mode={false} link={{}} patch={{}}
              onCablePropertyChange={null} onCableSave={null} onConnectorStatusChange={null} onConnectorTypeChange={null} onSocketClick={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders read-only port correctly', () => {

    const ro_permissions = {
        has_edit_permission: false,
        has_planner_edit_permission: false,
        has_planner_activate_permission: false,
        has_end_device_edit_permission: false,
    }

    const component = ReactTestRenderer.create(
        <Port language={lang} connector_id={1} connector_title="Connector01" connector_type={1} connector_type_id={1}
              connector_type_title="RJ45" connector_status={1} status_list={status_list} cable_colors={colors} permissions={ro_permissions}
              cable_types={cable_types} connector_types={types} image_url="/" side="left" is_planner_mode={false} sibling_id={8} link={link} patch={{}}
              onCablePropertyChange={null} onCableSave={null} onConnectorStatusChange={null} onConnectorTypeChange={null} onSocketClick={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});
