import * as React from 'react'
import {PortConnectionDetail} from '../components/Port/PortConnectionDetail'
const ReactTestRenderer = require('react-test-renderer');

const lang = {
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT']: 'Front',
    ['LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK']: 'Back',
};

const cable_colors = {
    1: '#00FF00',
    2: '#FF00FF',
};

const cable_types = {
    [1]: 'CAT5',
    [2]: 'CAT6',
    [3]: 'CAT7'
};

const permissions = {
    has_edit_permission: true,
    has_planner_edit_permission: true,
    has_planner_activate_permission: true,
    has_end_device_edit_permission: true,
}

it('renders link connector cable correctly', () => {
    const port_connector = {
        id: 1,
        title: "01",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 8,
        cable_title: "Cable01",
        cable_length: "100m",
        cable_color_id: 1,
        cable_color: "#FF0000",
    };
    const traceMockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="link" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                            cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={null}
                            onCablePropertyChange={null} onCableSave={null} onTraceClick={traceMockFunction} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let button = instance.findByProps({className: "icon-button"});
    button.props.onClick({});
    expect(traceMockFunction).toHaveBeenCalled();
});

it('renders patch connector cable correctly', () => {
    const port_connector = {
        id: 2,
        title: "02",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 14,
        cable_title: "Cable14",
        cable_length: "400m",
        cable_color_id: 2,
        cable_color: "#FF0000"
    };
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="patch" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                            cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={null}
                            onCablePropertyChange={null} onCableSave={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders socket connector cable correctly', () => {
    const port_connector = {
        id: 2,
        title: "02",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 14,
        cable_title: "Cable14",
        cable_length: "400m",
        cable_color_id: 2,
        cable_color: "#FF0000",
        is_socket: true
    };
    const socketMockFunction = jest.fn();
    const cablePropertyMockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="link" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                            cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={socketMockFunction}
                            onCablePropertyChange={cablePropertyMockFunction} onCableSave={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let button = instance.findByProps({className: "icon-button"});
    button.props.onClick({});
    expect(socketMockFunction).toHaveBeenCalled();

    let input = instance.findByProps({value: "Cable14"});
    input.props.onChange({
        target: { value: 'NewCable' }
    });
    expect(cablePropertyMockFunction).toHaveBeenCalledWith('left', 'link', 'cable_title', null, 'NewCable');

    input = instance.findByProps({value: "400m"});
    input.props.onChange({
        target: { value: '1400m' }
    });
    expect(cablePropertyMockFunction).toHaveBeenCalledWith('left', 'link', 'cable_length', null, '1400m');

    let select = instance.findByProps({id: "patch_cable_color"});
    select.props.onChange('#123456');
    expect(cablePropertyMockFunction).toHaveBeenCalledWith('left', 'link', 'cable_color_id', '#123456');

    select = instance.findByProps({id: "patch_cable_type"});
    select.props.onChange('2');
});

it('renders save button connector cable correctly', () => {
    const port_connector = {
        id: 2,
        title: "02",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 14,
        cable_title: "Cable14",
        cable_length: "400m",
        cable_color_id: 2,
        cable_color: "#FF0000",
        has_changed: true
    };
    const cableSaveMockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="link" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                            cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={null}
                            onCablePropertyChange={null} onCableSave={cableSaveMockFunction} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let button = instance.findByProps({className: "icon-button icon-blink"});
    button.props.onClick({});
    expect(cableSaveMockFunction).toHaveBeenCalledWith('left', 'link');
});

it('renders read-only save button connector cable correctly', () => {
    const port_connector = {
        id: 2,
        title: "02",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 14,
        cable_title: "Cable14",
        cable_length: "400m",
        cable_color_id: 2,
        cable_color: "#FF0000",
        has_changed: true
    };
    const cableSaveMockFunction = jest.fn();
    permissions.has_edit_permission = false
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="link" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                              cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={null}
                              onCablePropertyChange={null} onCableSave={cableSaveMockFunction} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders manual end device correctly', () => {
    const port_connector = {
        id: 2,
        title: "02",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 14,
        cable_title: "Cable14",
        cable_length: "400m",
        cable_color_id: 2,
        cable_color: "#FF0000",
        is_socket: true,
        manual_end_device_name: "Host01",
    };
    permissions.has_edit_permission = true
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="patch" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                            cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={null}
                            onCablePropertyChange={null} onCableSave={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders planned manual end device correctly', () => {
    const port_connector = {
        id: 2,
        title: "02",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 14,
        cable_title: "Cable14",
        cable_length: "400m",
        cable_color_id: 2,
        cable_color: "#FF0000",
        is_socket: true,
        is_planner_mode: true,
        planned_manual_end_device_name: "PlannedHost01",
    };
    permissions.has_edit_permission = false
    permissions.has_end_device_edit_permission = true
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="patch" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                              cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={null}
                              onCablePropertyChange={null} onCableSave={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders feed hosts correctly', () => {
    const port_connector = {
        id: 2,
        title: "02",
        obj_id: 4,
        obj_title: "Switch",
        obj_url: "",
        cable_id: 14,
        cable_title: "Cable14",
        cable_length: "400m",
        cable_color_id: 2,
        cable_color: "#FF0000",
        feed_hosts: ["Feed-Host01", "Feed-Host02"],
    };
    const component = ReactTestRenderer.create(
        <PortConnectionDetail type="patch" port_connector={port_connector} cable_colors={cable_colors} permissions={permissions}
                            cable_types={cable_types} image_url="" side="left" language={lang} onSocketClick={null}
                            onCablePropertyChange={null} onCableSave={null} onTraceClick={null} />,
    );
    expect(component).toMatchSnapshot();
});