import * as React from 'react'

interface AngleSelectorProps {
    angle: string
    onAngleChange: Function,
    language: {[key: string]: string},
    addFrontVerticalOption?: boolean,
    addBackVerticalOption?: boolean,
}

export class AngleSelector extends React.Component<AngleSelectorProps, {}> {
  constructor (props: AngleSelectorProps) {
    super(props)
    this.handleChange = this.handleChange.bind(this)
  }

  handleChange (e: React.FormEvent<HTMLSelectElement>) {
    this.props.onAngleChange(e.currentTarget.value)
  }

  render () {
    const angles = [
      { id: 'F', label: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT },
      { id: 'B', label: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK }
    ]

    if (this.props.addFrontVerticalOption) {
      angles.push({ id: 'FV', label: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT_VERTICAL })
    }

    if (this.props.addBackVerticalOption) {
      angles.push({ id: 'BV', label: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK_VERTICAL })
    }

    return <select onChange={this.handleChange}
      defaultValue={this.props.angle}
      className="input angle_selector">
      {
        angles.map((a) => (
          <option key={a.id}
            value={a.id}>
            {a.label}
          </option>
        ))
      }
    </select>
  }
}
