import {DeviceConnectorProps} from "./DeviceProps";
import * as React from "react";

interface DeviceListItemProps {
    onSelectedIdChange: Function,
    is_selected?: boolean,
    connector?: DeviceConnectorProps,
}

export class DeviceListItem extends React.Component<DeviceListItemProps, {}> {
    constructor (props: DeviceListItemProps) {
        super(props)
    }

    render () {
        const connector = this.props.connector
        if (!connector) {
            return null
        }

        const selectedClass = this.props.is_selected ? 'selected' : ''
        let link_href = null
        if (connector.link) {
            const name = connector.link.chassis_title ? connector.link.chassis_title + '_' + connector.link.slot_title : connector.link.obj_title
            link_href = <a href={connector.link.obj_url} target="_blank">
                {name} - {connector.link.title}
            </a>
        }

        let patch_href = null
        if (connector.patch) {
            const name = connector.patch.chassis_title ? connector.patch.chassis_title + '_' + connector.patch.slot_title : connector.patch.obj_title
            patch_href = <a href={connector.patch.obj_url} target="_blank">
                {name} - {connector.patch.title}
            </a>
        }

        if (connector.cable_pins && connector.cable_pins.length > 0) {
            // display connections to cable PINs
            let divs = []
            connector.cable_pins.forEach(pin => {
                const additional_info = pin.route_name ? <a href={pin.route_url} target="_blank">[{pin.route_name}]</a> : null
                let pin_title = pin.scope ? pin.scope + ' - ' : ''
                pin_title += pin.title
                divs.push(
                    <div key={pin.id}>
                        <a href={pin.obj_url} target="_blank">
                            {pin.obj_title} - {pin_title}
                        </a> {additional_info}
                    </div>
                )
            })
            link_href = divs
        }

        return <tr className={selectedClass}>
            <td>
                <a id={connector.connector_id.toString()}
                   href="#"
                   title={connector.connector_title}
                   onClick={(event) => this.props.onSelectedIdChange(event.currentTarget.id)}>
                    {connector.connector_title}
                </a>
            </td>
            <td>
                {link_href}
            </td>
            <td>
                {patch_href}
            </td>
        </tr>
    }
}
