import {ConnectorStatus} from "../Port/Port";
import {DeviceConnectorProps} from "./DeviceProps";
import * as React from "react";

interface DevicePortIconProps {
    index: number,
    onSelectedIdChange: Function,
    language: {[key: string]: string},
    colors: {[name: string]: string},
    status_list: {[id: number]: ConnectorStatus},
    use_cable_color: boolean,
    right_align_connector_titles: boolean,
    is_planner_mode: boolean,
    is_selected?: boolean,
    connector?: DeviceConnectorProps,
}

export class DevicePortIcon extends React.Component<DevicePortIconProps, {}> {
    constructor (props: DevicePortIconProps) {
        super(props)
        this.handleSelectedIdChange = this.handleSelectedIdChange.bind(this)
    }

    handleSelectedIdChange (event: React.MouseEvent<HTMLImageElement>) {
        this.props.onSelectedIdChange(event.currentTarget.id)
    }

    render () {
        const connector = this.props.connector
        if (!connector) {
            return <td>&nbsp;</td>
        }

        const selectedClass = this.props.is_selected ? 'selected' : ''
        const clsName = connector.rotation > 0 ? 'rotate-' + connector.rotation : ''
        let patchStatusTitle = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__NOT_CONNECTED
        let patchStyle = { backgroundColor: '#AAA' }
        let isPatchConnected
        if (this.props.is_planner_mode) {
            isPatchConnected = connector.virtual_connection && connector.virtual_connection.obj_id
        } else {
            isPatchConnected = connector.patch && connector.patch.obj_id
        }
        if (isPatchConnected) {
            patchStatusTitle = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED
            const color = this.props.colors.color_patch
            patchStyle = { backgroundColor: '#' + color }
        }

        if (! this.props.is_planner_mode && this.props.use_cable_color && connector.patch) {
            const is_hex = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(connector.patch.cable_color)
            if (connector.patch.cable_color && is_hex) {
                patchStyle = { backgroundColor: connector.patch.cable_color }
            }
        }

        if (connector.patch && connector.patch.is_socket) {
            const color = this.props.colors.color_patch_direct
            patchStyle = { backgroundColor: '#' + color }
        }

        let linkStatusTitle = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__NOT_CONNECTED
        let linkStyle = { backgroundColor: '#AAA' }
        if (connector.link_type) {
            linkStatusTitle = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED
            const color = connector.link_type == 'link' ? this.props.colors.color_link : this.props.colors.color_socket
            linkStyle = { backgroundColor: '#' + color }
        }
        if (connector.cable_pins && connector.cable_pins.length > 0) {
            linkStatusTitle = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED
            const color = this.props.colors.color_cable_pin
            linkStyle = { backgroundColor: '#' + color }
        }

        if (! this.props.is_planner_mode && this.props.use_cable_color && connector.link) {
            const is_hex = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(connector.link.cable_color)
            if (connector.link.cable_color && is_hex) {
                linkStyle = { backgroundColor: connector.link.cable_color }
            }
        }

        let img
        let style
        const color = this.props.status_list[connector.connector_status]
        if (color) {
            const is_hex = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(color.color)
            if (color && is_hex) {
                style = { backgroundColor: color.color }
            }
        }
        if (connector.url) {
            img = <img id={connector.connector_id.toString()}
                       src={connector.url}
                       alt={connector.connector_title}
                       className={clsName}
                       style={style}
                       title={connector.connector_title}
                       onClick={this.handleSelectedIdChange} />
        } else {
            img = <span id={connector.connector_id.toString()}
                        className="no-image"
                        style={style}
                        title={connector.connector_title}
                        onClick={this.handleSelectedIdChange}>
                    {connector.connector_title}
                  </span>
        }

        let span = <span className="port-index" title={connector.connector_title} style={{ textAlign: 'center' }}>{connector.connector_title}</span>
        if (this.props.right_align_connector_titles) {
            span = <span className="port-index" title={connector.connector_title} style={{ textOverflow: 'clip', direction: 'rtl' }}>{connector.connector_title}</span>
        }
        return <td className={selectedClass}>
            {span}
            <div className="status" style={patchStyle} title={patchStatusTitle}>&nbsp;</div>
            {img}
            <div className="status" style={linkStyle} title={linkStatusTitle}>&nbsp;</div>
        </td>
    }
}
