import * as React from 'react'
import { ChassisProps } from './DeviceProps'

interface SlotTabProps {
    chassis: ChassisProps,
    onTabSelected: Function,
    is_image_view: boolean,
    selected_id?: number,
}

export class SlotTabs extends React.Component<SlotTabProps, {}> {
  constructor (props: SlotTabProps) {
    super(props)
    this.handleTabSelected = this.handleTabSelected.bind(this)
  }

  handleTabSelected (event: React.MouseEvent<HTMLElement>) {
    const id = event.currentTarget.id
    this.props.onTabSelected(id)
  }

  render () {
    // calculate the same size for each column
    const width = this.props.chassis.width ? parseInt(this.props.chassis.width.toString()) + 1 : 1
    const colgroup = []
    for (let x:number = 0; x < width; x++) {
      const style = { width: (100 / width) + '%' }
      colgroup.push(<col key={'slot-' + x} style={style} width={(100 / width) + '%'} />)
    }

    // set at least one row
    let chassisHeight = this.props.chassis.height ? this.props.chassis.height : 1
    chassisHeight = chassisHeight > 0 ? chassisHeight : 1
    const trs = []
    const spannedCells:string[] = []
      for (let y:number = 0; y < chassisHeight; y++) {
          // add spanned cells for chassis cell
          const ident = '0-' + y
          spannedCells.push(ident)
      }

    for (let y:number = 0; y < chassisHeight; y++) {
      const tds = []
      for (let x:number = 0; x < width; x++) {
          if (x == 0 && y == 0) {
              // add chassis cell first
              tds.push(
                  <td key={'chassis'} rowSpan={chassisHeight} colSpan={1} >-<br />
                      <span className={this.props.selected_id == this.props.chassis.id ? 'selected' : ''}
                            id={this.props.chassis.id.toString()}
                            key={-1}
                            onClick={this.handleTabSelected}>{this.props.chassis.title}</span>
                  </td>)
          }

        const slots = this.props.chassis.slots.filter(s => s.x_from == x && s.y_from == y)
        if (slots.length == 0) {
          const id = x + '-' + y
          if (spannedCells.indexOf(id) > -1) {
            // do not render spanned cells
            continue
          }

          // render cell without an assigned slot
          tds.push(<td key={y + '-' + x}>&nbsp;</td>)
          continue
        }

        const elements = []
        const slot = slots[0]
        const rowSpan = (slot.y_to - slot.y_from) + 1
        const colSpan = (slot.x_to - slot.x_from) + 1
        for (const key in slot.devices) {
          if (!slot.devices.hasOwnProperty(key)) {
            continue
          }

          const device = slot.devices[key]
          let className = ''
          if (this.props.selected_id && this.props.selected_id == device.id) {
            className = 'selected'
          }

          if (device.image_url && this.props.is_image_view) {
            elements.push(<img src={device.image_url}
              alt={device.title}
              id={device.id.toString()}
              key={device.id}
              title={device.title}
              className={className}
              onClick={this.handleTabSelected}/>)
          } else {
            elements.push(<span className={className}
              id={device.id.toString()}
              key={device.id}
              onClick={this.handleTabSelected}>{device.title}</span>)
          }
        }

        // add spanned cells (to be not rendered)
        for (let sx:number = slot.x_from; sx <= slot.x_to; sx++) {
          for (let sy:number = slot.y_from; sy <= slot.y_to; sy++) {
            const ident = sx + '-' + sy
            spannedCells.push(ident)
          }
        }

        tds.push(<td key={y + '-' + x} rowSpan={rowSpan} colSpan={colSpan} >{slot.title}<br />{elements}</td>)
      }
      trs.push(<tr key={y}>{tds}</tr>)
    }

    return <table className="slot-tab">
      <colgroup>
        {colgroup}
      </colgroup>
      <tbody>
        {trs}
      </tbody>
    </table>
  }
}
