import * as React from 'react'
import {PortConnectorProps, VirtualConnection} from './Port'
import {Select} from "../Select";
import {Permissions} from "../../EditorCompose";

export interface ConnectionDetailProps {
    type: string,
    port_connector: PortConnectorProps,
    virtual_port_connector?: VirtualConnection,
    cable_colors: {[id: number]: string},
    cable_types: {[id: number]: string},
    image_url: string,
    side: string,
    is_planner_mode?: boolean,
    permissions: Permissions,
    onTraceClick: Function,
    onSocketClick: Function,
    onCablePropertyChange: Function,
    onCableSave: Function,
    disabled?: boolean,
    language: {[key: string]: string},
}

export class AbstractConnectionDetail extends React.Component<ConnectionDetailProps, {}> {
    constructor (props: ConnectionDetailProps) {
        super(props)
        this.handleCableLengthChange = this.handleCableLengthChange.bind(this)
        this.handleCableTitleChange = this.handleCableTitleChange.bind(this)
        this.handleCableColorChange = this.handleCableColorChange.bind(this)
        this.handleCableTypeChange = this.handleCableTypeChange.bind(this)
        this.handleEndDeviceChange = this.handleEndDeviceChange.bind(this)
        this.handleCableSave = this.handleCableSave.bind(this)
    }

    getCableSetup (port_connector) {
        const { language } = this.props
        let disabled = this.props.disabled
        if (! disabled) {
            disabled = ! this.props.permissions.has_edit_permission
        }
        return <>
            <tr key="connector_cable_title">
                <td>{language.LC__CMDB__CATG__SMVIEW_PATCH__CABLE}:</td>
                <td className="cable-options">
                    <input type="text" title={language.LC__CMDB__CATG__SMVIEW_PATCH__CABLE} disabled={disabled}
                           onChange={this.handleCableTitleChange} value={port_connector.cable_title} />
                    <input type="text" title={language.LC__CMDB__CATG__SMVIEW_PATCH__CABLE_LENGTH} disabled={disabled}
                           onChange={this.handleCableLengthChange} value={port_connector.cable_length} />
                </td>
            </tr>
            <tr key="connector_cable_length">
                <td>&nbsp;</td>
                <td>
                    <Select id="patch_cable_type"
                            options={this.props.cable_types}
                            title={language.LC__CMDB__CATG__SMVIEW_PATCH__CABLE_TYPE}
                            selected_option={port_connector.cable_type_id}
                            disabled={disabled}
                            onChange={this.handleCableTypeChange} />
                    <Select id="patch_cable_color"
                            options={this.props.cable_colors}
                            title={language.LC__CMDB__CATG__SMVIEW_PATCH__CABLE_COLOR}
                            selected_option={port_connector.cable_color_id}
                            disabled={disabled}
                            onChange={this.handleCableColorChange} />
                    {this.getPreviewSpan(port_connector.cable_color_id)}
                </td>
            </tr>
        </>
    }

    getPreviewSpan (color_id: number) {
        const color = this.props.cable_colors[color_id]
        const is_hex = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(color)
        let span
        if (color && is_hex) {
            const style = { backgroundColor: color }
            span = <span className="color-preview" style={style}> </span>
        }
        return span
    }

    handleCableLengthChange (event: React.ChangeEvent<HTMLInputElement>) {
        this.props.onCablePropertyChange(this.props.side, this.props.type, 'cable_length', null, event.target.value)
    }

    handleCableTitleChange (event: React.ChangeEvent<HTMLInputElement>) {
        this.props.onCablePropertyChange(this.props.side, this.props.type, 'cable_title', null, event.target.value)
    }

    handleCableColorChange (color: string) {
        this.props.onCablePropertyChange(this.props.side, this.props.type, 'cable_color_id', color)
    }

    handleCableTypeChange (type: string) {
        this.props.onCablePropertyChange(this.props.side, this.props.type, 'cable_type_id', type)
    }

    handleEndDeviceChange (event: React.ChangeEvent<HTMLInputElement>) {
        const propertyName = this.props.is_planner_mode ? 'planned_manual_end_device_name' : 'manual_end_device_name'
        this.props.onCablePropertyChange(this.props.side, this.props.type, propertyName, null, event.target.value)
    }

    handleCableSave () {
        this.props.onCableSave(this.props.side, this.props.type)
    }
}
