import * as React from 'react'
import {AbstractConnectionDetail, ConnectionDetailProps} from "./AbstractConnectionDetail";

export class PortConnectionDetail extends AbstractConnectionDetail {
  constructor (props: ConnectionDetailProps) {
    super(props)
  }

  render () {
    const { language, port_connector } = this.props
    let title = language.LC__CMDB__CATG__SMVIEW_PATCH__PATCH_CONNECTION
    if (this.props.type == 'link') {
      title = language.LC__CMDB__CATG__SMVIEW_PATCH__LINK_CONNECTION
    }

    let save_button
    if (port_connector.has_changed && (this.props.permissions.has_edit_permission || (this.props.is_planner_mode && this.props.permissions.has_end_device_edit_permission))) {
      save_button = <button type="button" className="icon-button icon-blink" onClick={this.handleCableSave}>
          <img src={this.props.image_url + 'disk.png'} alt="->"
               title={language.LC__UNIVERSAL__BUTTON_SAVE}/>
      </button>
    }

    let btn_image = this.props.image_url + 'link_go.png'
    let object_link = null
    let trace_button

    if (port_connector.id && port_connector.id > 0) {
      object_link = <a href={port_connector.obj_url} target="_blank">
        {port_connector.obj_title} - {port_connector.title}
      </a>

      if (port_connector.is_socket) {
        // add button to connect to an end-device
        btn_image = this.props.image_url + 'computer_link.png'
        if (this.props.type == 'patch') {
          trace_button = <button type="button" className="icon-button" onClick={() => {this.props.onSocketClick('patch', port_connector.id)}}>
            <img src={btn_image} alt="->"
              title={language.LC__CMDB__CATG__SMVIEW_PATCH__OPEN_END_DEVICE_CONNECTION}/>
          </button>
        } else {
          trace_button = <button type="button" className="icon-button" onClick={() => {this.props.onSocketClick('link', port_connector.id)}}>
            <img src={btn_image} alt="->"
              title={language.LC__CMDB__CATG__SMVIEW_PATCH__OPEN_SOCKET_CONNECTION}/>
          </button>
        }
      } else {
        // add button to trace between objects
        trace_button = <button type="button" className="icon-button" onClick={() => {this.props.onTraceClick(port_connector.obj_id, port_connector.id)}}>
          <img src={btn_image} alt="->" title={language.LC__CMDB__CATG__SMVIEW_PATCH__TRACE} />
        </button>
      }
    } else {
      // add button to connect to an end-device
      btn_image = this.props.image_url + 'computer_link.png'
      if (this.props.type == 'patch') {
        trace_button = <button type="button" className="icon-button" disabled={! this.props.permissions.has_edit_permission} onClick={() => {this.props.onSocketClick('patch', port_connector.id)}}>
          <img src={btn_image} alt="->" title={language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_END_DEVICE} />
        </button>
      } else {
        trace_button = <button type="button" className="icon-button" disabled={! this.props.permissions.has_edit_permission} onClick={() => {this.props.onSocketClick('link', port_connector.id)}}>
          <img src={btn_image} alt="->" title={language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_SOCKET} />
        </button>
      }
    }

    let feed_hosts = null
    if (port_connector.feed_hosts && port_connector.feed_hosts.length > 0) {
      feed_hosts = <div className="additional-info">({port_connector.feed_hosts.join(', ')})</div>
    }

    let cable_setup
    if (port_connector.cable_id && port_connector.cable_id > 0) {
      cable_setup = this.getCableSetup(port_connector)
    }

      let end_device
      if (port_connector.is_socket) {
          const end_device_title = this.props.is_planner_mode ? language.LC__CMDB__CATG__SMVIEW_PATCH__PLANNED_END_DEVICE_TITLE : language.LC__CMDB__CATG__SMVIEW_PATCH__END_DEVICE_TITLE
          const end_device_value = this.props.is_planner_mode ? port_connector.planned_manual_end_device_name : port_connector.manual_end_device_name
          end_device = <tr key="end_device">
              <td>{end_device_title}</td>
              <td>
                  <input type="text"
                         className="input"
                         value={end_device_value ?? ''}
                         onChange={this.handleEndDeviceChange}
                         disabled={! this.props.permissions.has_end_device_edit_permission} />
              </td>
          </tr>
      }

    return <table>
      <tbody>
        <tr key="connector_title">
          <td colSpan={2} className="section-header">
              {title} {save_button}
          </td>
        </tr>
        <tr key="connector_connection">
          <td>{language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED_WITH}:</td>
          <td>
            {object_link} {feed_hosts} {trace_button}
          </td>
        </tr>
        {cable_setup}
        {end_device}
      </tbody>
    </table>
  }
}
