import * as React from 'react'
import {AbstractConnectionDetail, ConnectionDetailProps} from "./AbstractConnectionDetail";

export class PortVirtualConnectionDetail extends AbstractConnectionDetail {
    constructor (props: ConnectionDetailProps) {
        super(props)
    }

    render () {
        const { language, port_connector, virtual_port_connector } = this.props
        let save_button
        if (virtual_port_connector && virtual_port_connector.has_changed && this.props.permissions.has_planner_edit_permission) {
            save_button = <button type="button" className="icon-button icon-blink" onClick={this.handleCableSave}>
                <img src={this.props.image_url + 'disk.png'} alt="->"
                     title={language.LC__UNIVERSAL__BUTTON_SAVE}/>
            </button>
        }

        let object_link = null
        let trace_button
        let cable_setup
        if (virtual_port_connector && virtual_port_connector.id && virtual_port_connector.id > 0) {
            object_link = <a href={virtual_port_connector.obj_url} target="_blank">
                {virtual_port_connector.obj_title} - {virtual_port_connector.title}
            </a>

            // add button to trace between objects
            trace_button = <button type="button" className="icon-button" onClick={() => {this.props.onTraceClick(virtual_port_connector.obj_id, virtual_port_connector.id)}}>
                <img src={this.props.image_url + 'link_go.png'} alt="->" title={language.LC__CMDB__CATG__SMVIEW_PATCH__TRACE} />
            </button>

            cable_setup = this.getCableSetup(virtual_port_connector)
        }

        let current_connection
        if (port_connector && port_connector.id && port_connector.id > 0) {
            current_connection = <div>({port_connector.obj_title} - {port_connector.title})</div>
        }

        return <table>
            <tbody>
                <tr key="connector_title">
                    <td colSpan={2} className="section-header">
                        {language.LC__CMDB__CATG__SMVIEW_PATCH__VIRTUAL_PATCH_CONNECTION} {save_button}
                    </td>
                </tr>
                <tr key="connector_connection">
                    <td>{language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED_WITH}:</td>
                    <td>
                        {object_link} {trace_button}
                        {current_connection}
                    </td>
                </tr>
                {cable_setup}
            </tbody>
        </table>
    }
}
