import * as React from 'react'

interface RackSelectorProps {
    side: string,
    racks: {[key: number]: string},
    selected_id: number,
    onSelectionChange: Function,
}

export class RackSelector extends React.Component<RackSelectorProps, {}> {
  constructor (props: RackSelectorProps) {
    super(props)
    this.handleChange = this.handleChange.bind(this)
  }

  handleChange (e: React.FormEvent<HTMLSelectElement>) {
    this.props.onSelectionChange(this.props.side, e.currentTarget.value)
  }

  render () {
    // convert to sortable array
    const sortables = []
    for (const key in this.props.racks) {
      if (this.props.racks.hasOwnProperty(key)) {
        const rack = this.props.racks[key]
        sortables.push([key, rack])
      }
    }

    // sort array
    sortables.sort(function (a, b) {
      return (a[1].toString() > b[1].toString()) ? 1 : -1
    })

    // add options
    const options = []
    options.push(<option key={-1} value='-1'>-</option>)
    for (const c in sortables) {
      if (sortables.hasOwnProperty(c)) {
        const row = sortables[c]
        const i:number = parseInt(row[0])
        options.push(<option key={i} value={i}>{row[1]}</option>)
      }
    }
    return <select className='input'
      value={this.props.selected_id}
      onChange={this.handleChange}>{options}</select>
  }
}
