import * as React from "react";
import {ConnectorStatus} from "./Port/Port";

interface SelectProps {
    id: string,
    options: {[key: number]: string},
    status_options?: {[key: number]: ConnectorStatus},
    selected_option?: number,
    onChange?: Function,
    disabled?: boolean,
    title?: string,
}

export class Select extends React.Component<SelectProps, {}> {
    constructor (props: SelectProps) {
        super(props)
        this.handleChange = this.handleChange.bind(this)
    }

    handleChange (event: React.ChangeEvent<HTMLSelectElement>) {
        if (this.props.onChange) {
            this.props.onChange(event.currentTarget.value)
        }
    }

    static getOption (key: string, color: string, title?: string) {
        let style = { backgroundColor: 'none' }
        const is_hex = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(color)
        if (color && is_hex) {
            style = { backgroundColor: color }
        }
        title = title || color
        return <option key={key} value={key} style={style}>{title}</option>
    }

    render () {
        const options = []
        options.push(<option key={-1} value='-1'>-</option>)

        // sort options by name
        const opts: {[key: string]: string} = this.props.options
        const sortable = Object.entries(opts).sort((a: [string, string], b: [string, string]) => b[1].toLowerCase() < a[1].toLowerCase() ? 1 : -1)

        sortable.forEach((i) => {
            options.push(Select.getOption(i[0], i[1]))
        })

        for (const key in this.props.status_options) {
            if (this.props.status_options.hasOwnProperty(key)) {
                const row = this.props.status_options[key]
                options.push(Select.getOption(key, row.color, row.title))
            }
        }

        const value = this.props.selected_option ? this.props.selected_option.toString() : -1
        return <select id={this.props.id}
                       className='input input-mini'
                       value={value}
                       onChange={this.handleChange}
                       title={this.props.title}
                       disabled={this.props.disabled}>{options}</select>
    }
}
