import * as React from 'react'
import { Port } from '../components/Port'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_CONNECTOR_TYPE: 'No connector of type',
  LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_MATCHING_NAME: 'named',
  LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_HAS_NEW_CONNECTOR: 'New unassigned connector named',
  LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGN: 'Auto assign'
}

const portProps = {
  title: '01',
  id: 1,
  number: 1,
  angle: 'F',
  label_orientation: 'S',
  pos_x: 1,
  pos_y: 1,
  rotation: 0,
  status: 'ok',
  url: '',
  connector_id: 1,
  connector_title: '01',
  connector_type_title: 'RJ-45'
}

it('renders read only port correctly', () => {
  const component = ReactTestRenderer.create(
    <Port {...portProps}
          language={lang}
          is_editmode={false}
          is_display_new={false}
          onMergeClick={null}
          right_align_connector_titles={false} />
  )
  expect(component).toMatchSnapshot()
})

it('renders port correctly', () => {
  const component = ReactTestRenderer.create(
    <Port {...portProps}
          language={lang}
          is_editmode={true}
          is_display_new={false}
          onMergeClick={null}
          right_align_connector_titles={false} />
  )
  expect(component).toMatchSnapshot()
})

it('renders rotated port correctly', () => {
  let p = JSON.parse(JSON.stringify(portProps))
  p.title = null
  p.rotation = 180
  const component = ReactTestRenderer.create(
      <Port {...p}
            language={lang}
            is_editmode={true}
            is_display_new={false}
            onMergeClick={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders port with merge option correctly', () => {
  const mergeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
    <Port {...portProps}
          language={lang}
          is_editmode={true}
          is_display_new={true}
          is_new={true}
          onMergeClick={mergeMockFunction}
          right_align_connector_titles={false} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const input = instance.findByProps({ value: 1 })
  input.props.onClick({
    currentTarget: { value: 1 }
  })
  expect(mergeMockFunction).toHaveBeenCalledWith(1)
})

it('renders port with invalid merge option correctly', () => {
  const mergeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <Port {...portProps}
            language={lang}
            is_editmode={true}
            is_display_new={false}
            is_new={true}
            onMergeClick={mergeMockFunction}
            right_align_connector_titles={false} />
  )
  expect(component).toMatchSnapshot()
})

it('renders port with merge hint correctly', () => {
  const component = ReactTestRenderer.create(
    <Port {...portProps}
          language={lang}
          is_editmode={true}
          is_display_new={false}
          merged={false}
          onMergeClick={null}
          connector_type_title="RJ-45"
          connector_title="01"
          right_align_connector_titles={false} />
  )
  expect(component).toMatchSnapshot()
})

it('handle port mouse over correctly', () => {
  const mouseOverMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <Port {...portProps}
            language={lang}
            is_editmode={true}
            is_display_new={false}
            is_new={false}
            onMergeClick={null}
            onMouseOver={mouseOverMockFunction}
            right_align_connector_titles={false} />
  )

  const instance = component.root
  const input = instance.findByProps({ className: 'port-container' })
  input.props.onMouseOver({
    currentTarget: {
      id: 1
    }
  })
  expect(mouseOverMockFunction).toHaveBeenCalledWith({id: 1})
})

it('handle port drag start correctly', () => {
  const component = ReactTestRenderer.create(
      <Port {...portProps}
            language={lang}
            is_editmode={true}
            is_display_new={false}
            is_new={false}
            onMergeClick={null}
            right_align_connector_titles={false} />
  )

  const instance = component.root
  const input = instance.findByProps({ id: 'conn-1' })
  input.props.onDragStart({
    currentTarget: {
      id: 'conn-1',
      alt: '01',
      parentElement: {
        parentElement: {
          id: 'frame-1-1',
          className: 'frame'
        }
      }
    },
    dataTransfer: {
      setData: jest.fn()
    }
  })

  input.props.onDragStart({
    currentTarget: {
      id: 'conn-1',
      alt: '01',
      parentElement: {
        parentElement: {
          id: 'frame-1-1',
          className: 'no-frame'
        }
      }
    },
    dataTransfer: {
      setData: jest.fn()
    }
  })
})
