import * as React from 'react'
import { PortFrame } from '../components/PortFrame'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_CONNECTOR_TYPE: 'No connector of type',
  LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_MATCHING_NAME: 'named',
  LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_HAS_NEW_CONNECTOR: 'New unassigned connector named',
  LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGN: 'Auto assign',
  LC__CMDB__CATG__SMVIEW_PATCH__SPIN: 'Spin',
  LC__CMDB__CATG__SMVIEW_PATCH__DELETE: 'Delete',
}

const portProps = {
  title: '01',
  id: 1,
  number: 1,
  angle: 'F',
  label_orientation: 'S',
  pos_x: 1,
  pos_y: 1,
  rotation: 0,
  status: 'ok',
  url: '',
  connector_id: 1,
  connector_title: '01',
  connector_type_title: 'RJ-45',
  is_new: false
}

it('renders read only port frame correctly', () => {
  const component = ReactTestRenderer.create(
      <PortFrame x={1}
                 y={1}
                 port={portProps}
                 language={lang}
                 is_editable={false}
                 is_display_new={false}
                 image_url="http://img.url"
                 visibleFrameButtons=""
                 right_align_connector_titles={false}
                 onShowFrameButtons={null}
                 onRotateClick={null}
                 onDeleteClick={null}
                 onMergeClick={null}
                 onDrop={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders port frame correctly', () => {
  const component = ReactTestRenderer.create(
      <PortFrame x={1}
                 y={1}
                 port={portProps}
                 language={lang}
                 is_editable={true}
                 is_display_new={false}
                 image_url="http://img.url"
                 visibleFrameButtons=""
                 right_align_connector_titles={false}
                 onShowFrameButtons={null}
                 onRotateClick={null}
                 onDeleteClick={null}
                 onMergeClick={null}
                 onDrop={null} />
  )
  expect(component).toMatchSnapshot()
})

it('handle merge correctly', () => {
  const mergeMockFunction = jest.fn()
  let pp = JSON.parse(JSON.stringify(portProps))
  pp.is_new = true
  const component = ReactTestRenderer.create(
      <PortFrame x={1}
                 y={1}
                 port={pp}
                 language={lang}
                 is_editable={true}
                 is_display_new={true}
                 image_url="http://img.url"
                 visibleFrameButtons=""
                 right_align_connector_titles={false}
                 onShowFrameButtons={null}
                 onRotateClick={null}
                 onDeleteClick={null}
                 onMergeClick={mergeMockFunction}
                 onDrop={null} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const input = instance.findByProps({ value: 1 })
  input.props.onClick({
    currentTarget: { value: 1 }
  })
  expect(mergeMockFunction).toHaveBeenCalledWith(1)
})

it('handle drop correctly', () => {
  const dropMockFunction = jest.fn()
  const defaultMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <PortFrame x={1}
                 y={1}
                 port={portProps}
                 language={lang}
                 is_editable={true}
                 is_display_new={true}
                 image_url="http://img.url"
                 visibleFrameButtons=""
                 right_align_connector_titles={false}
                 onShowFrameButtons={null}
                 onRotateClick={null}
                 onDeleteClick={null}
                 onMergeClick={null}
                 onDrop={dropMockFunction} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const input = instance.findByProps({ id: 'frame-1-1' })
  input.props.onDrop({
    preventDefault: defaultMockFunction,
    currentTarget: { id: 'frame-1-1' },
    dataTransfer: 'someData'
  })
  expect(dropMockFunction).toHaveBeenCalledWith('1', '1', 'someData')
})

it('handle mouse over parent correctly', () => {
  const deleteMockFunction = jest.fn()
  const rotationMockFunction = jest.fn()
  const showFrameButtonsMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <PortFrame x={1}
                 y={1}
                 port={portProps}
                 language={lang}
                 is_editable={true}
                 is_display_new={true}
                 image_url="http://img.url"
                 visibleFrameButtons="frame-1-1"
                 right_align_connector_titles={false}
                 onShowFrameButtons={showFrameButtonsMockFunction}
                 onRotateClick={rotationMockFunction}
                 onDeleteClick={deleteMockFunction}
                 onMergeClick={null}
                 onDrop={null} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  let input = instance.findByProps({ id: 'frame-1-1' })
  input.props.onMouseOver({
    target: { className: 'frame' },
  })
  expect(showFrameButtonsMockFunction).toHaveBeenCalled()

  input = instance.findByProps({ title: 'Spin' })
  input.props.onClick({
    currentTarget: {
      parentElement: {
        id: 'frame-1-1'
      }
    },
  })
  expect(rotationMockFunction).toHaveBeenCalledWith('1', '1')

  input = instance.findByProps({ title: 'Delete' })
  input.props.onClick({
    currentTarget: {
      parentElement: {
        id: 'frame-1-1'
      }
    },
  })
  expect(deleteMockFunction).toHaveBeenCalledWith('1', '1')
})
