import * as React from 'react'
import { SetupTable } from '../components/SetupTable'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_PATCH__USE_TEMPLATE: 'Use template',
  LC__CMDB__CATG__SMVIEW_PATCH__EDIT_TEMPLATE: 'Edit template',
  LC__CMDB__CATG__SMVIEW_PATCH__CREATE_TEMPLATE_COPY: 'Create template copy',
  LC__CMDB__CATG__SMVIEW_PATCH__ROW_COUNT: 'Row count',
  LC__CMDB__CATG__SMVIEW_PATCH__COLUMN_COUNT: 'Column count',
  LC__CMDB__CATG__SMVIEW_PATCH__SIZE: 'Size',
  'LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_L-R': 'L-R',
  'LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_T-B': 'T-B',
  LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_NO: '-',
  LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT: 'Auto assignment',
  LC__CMDB__CATG__SMVIEW_PATCH__GENERATE_ASSIGNMENT: 'Assign connectors',
  LC__CMDB__CATG__SMVIEW_PATCH__FRAME_PROPERTIES: 'Frame properties',
  LC__CMDB__CATG__SMVIEW_PATCH__DESCRIPTION: 'Description',
  LC__CMDB__CATG__SMVIEW_PATCH__IS_TEMPLATE: 'Is template',
}

const usage = [{
  title: 'Patch01',
  type_title: 'Patch Panel',
  url: 'http://grid.url',
}]

const frame = {
  id: 1,
  title: 'Grid01',
  column_count: 4,
  row_count: 2,
  is_template: false,
  modify_template: false,
  create_copy: false,
  used_by: usage,
}

const templates = {
  1: 'GridTemplate01',
  2: 'GridTemplate02',
}

it('renders read only setup table correctly', () => {
  const component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={frame}
                  templates={templates}
                  is_editable={false}
                  is_edit_template_mode={false}
                  is_initial_mode={false}
                  onFramePropertyChange={null}
                  onTemplateChange={null}
                  onAutoGeneration={null}
                  onEditTemplate={null}
                  onCopyTemplate={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders setup table correctly', () => {
  const component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={frame}
                  templates={templates}
                  is_editable={true}
                  is_edit_template_mode={false}
                  is_initial_mode={false}
                  onFramePropertyChange={null}
                  onTemplateChange={null}
                  onAutoGeneration={null}
                  onEditTemplate={null}
                  onCopyTemplate={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders setup table used by correctly', () => {
  let f = JSON.parse(JSON.stringify(frame))
  f.used_by = null
  let component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={f}
                  templates={templates}
                  is_editable={true}
                  is_edit_template_mode={false}
                  is_initial_mode={false}
                  onFramePropertyChange={null}
                  onTemplateChange={null}
                  onAutoGeneration={null}
                  onEditTemplate={null}
                  onCopyTemplate={null} />
  )
  expect(component).toMatchSnapshot()

  f.used_by = [
    {
      title: 'Patch01',
      type_title: 'Patch Panel',
      url: 'http://grid.url',
    },
    {
      title: 'Patch02',
      type_title: 'Patch Panel',
      url: 'http://grid.url',
    },
    {
      title: 'Patch03',
      type_title: 'Patch Panel',
      url: 'http://grid.url',
    },
    {
      title: 'Patch04',
      type_title: 'Patch Panel',
      url: 'http://grid.url',
    },
    {
      title: 'Patch05',
      type_title: 'Patch Panel',
      url: 'http://grid.url',
    }
  ]

  component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={f}
                  templates={templates}
                  is_editable={true}
                  is_edit_template_mode={false}
                  is_initial_mode={false}
                  onFramePropertyChange={null}
                  onTemplateChange={null}
                  onAutoGeneration={null}
                  onEditTemplate={null}
                  onCopyTemplate={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders setup table edit mode correctly', () => {
  const component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={frame}
                  templates={templates}
                  is_editable={true}
                  is_edit_template_mode={true}
                  is_initial_mode={false}
                  onFramePropertyChange={null}
                  onTemplateChange={null}
                  onAutoGeneration={null}
                  onEditTemplate={null}
                  onCopyTemplate={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders setup table initial mode correctly', () => {
  const autoGenerationMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={frame}
                  templates={templates}
                  is_editable={true}
                  is_edit_template_mode={false}
                  is_initial_mode={true}
                  onFramePropertyChange={null}
                  onTemplateChange={null}
                  onAutoGeneration={autoGenerationMockFunction}
                  onEditTemplate={null}
                  onCopyTemplate={null} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const input = instance.findByProps({ id: 'setup_generate_frame' })
  input.props.onClick()
  expect(autoGenerationMockFunction).toHaveBeenCalledWith('l_r')

  const select = instance.findByProps({ id: 'auto_create_direction' })
  select.props.onChange({
    currentTarget: {
      value: 't_b'
    },
  })

  input.props.onClick()
  expect(autoGenerationMockFunction).toHaveBeenCalledWith('t_b')
})

it('handle normal setup table correctly', () => {
  const framePropertyChangeMockFunction = jest.fn()
  const templateChangeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={frame}
                  templates={templates}
                  is_editable={true}
                  is_edit_template_mode={false}
                  is_initial_mode={false}
                  onFramePropertyChange={framePropertyChangeMockFunction}
                  onTemplateChange={templateChangeMockFunction}
                  onAutoGeneration={null}
                  onEditTemplate={null}
                  onCopyTemplate={null} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  let input = instance.findByProps({ value: 'Grid01' })
  input.props.onChange({
    currentTarget: { value: 'Grid88' },
  })
  expect(framePropertyChangeMockFunction).toHaveBeenCalledWith('title', 'Grid88')

  input = instance.findByProps({ type: 'checkbox' })
  input.props.onChange({
    currentTarget: {
      checked: true
    },
  })
  expect(framePropertyChangeMockFunction).toHaveBeenCalledWith('is_template', true)

  input = instance.findAllByProps({ className: 'input' })[2]
  input.props.onChange({
    currentTarget: {
      value: 2
    },
  })
  expect(templateChangeMockFunction).toHaveBeenCalledWith(2)
})

it('handle setup table edit template correctly', () => {
  const copyTemplateMockFunction = jest.fn()
  const editTemplateMockFunction = jest.fn()
  let f = JSON.parse(JSON.stringify(frame))
  f.is_template = true
  const component = ReactTestRenderer.create(
      <SetupTable language={lang}
                  frame={f}
                  templates={templates}
                  is_editable={true}
                  is_edit_template_mode={false}
                  is_initial_mode={false}
                  onFramePropertyChange={null}
                  onTemplateChange={null}
                  onAutoGeneration={null}
                  onEditTemplate={editTemplateMockFunction}
                  onCopyTemplate={copyTemplateMockFunction} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  let input = instance.findByProps({ id: 'setup_edit_template' })
  input.props.onClick()
  expect(editTemplateMockFunction).toHaveBeenCalled()

  input = instance.findByProps({ id: 'setup_copy_template' })
  input.props.onClick()
  expect(copyTemplateMockFunction).toHaveBeenCalled()
})
