import * as React from 'react'
import { SizeSelector } from '../components/SizeSelector'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_CONNECTOR_TYPE: 'No connector of type',
  LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_MATCHING_NAME: 'named',
  LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_HAS_NEW_CONNECTOR: 'New unassigned connector named',
  LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGN: 'Auto assign'
}

const sizes = {
  's': {
    'label': 'S',
    'cssClass': 'size-s',
  },
  'm': {
    'label': 'M',
    'cssClass': 'size-m',
  },
  'l': {
    'label': 'L',
    'cssClass': 'size-l',
  }
}

it('renders read only size selector correctly', () => {
  const component = ReactTestRenderer.create(
      <SizeSelector sizes={sizes} current_size="m" language={lang} col_span={1} is_editable={false} onChange={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders size selector correctly', () => {
  const changeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SizeSelector sizes={sizes} current_size="m" language={lang} col_span={1} is_editable={true} onChange={changeMockFunction} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const input = instance.findByProps({ id: 's' })
  input.props.onClick({
    currentTarget: { id: 's' }
  })
  expect(changeMockFunction).toHaveBeenCalledWith('s')
})
