import * as React from 'react'
import { SizeWidget } from '../components/SizeWidget'
const ReactTestRenderer = require('react-test-renderer')

it('renders size widget correctly', () => {
  const changeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SizeWidget widget_id="columns" angle={'F'} value={2} onWidgetSizeChange={changeMockFunction} />
  )
  expect(component).toMatchSnapshot()

  // const instance = component.root
  // let input = instance.findByProps({ name: 'minus' })
  // input.props.onClick({
  //   currentTarget: {
  //     name: 'minus',
  //     parentElement: {
  //       id: 'columns'
  //     }
  //   }
  // })
  // expect(changeMockFunction).toHaveBeenCalledWith('columns', 1)

  // input = instance.findByProps({ name: 'plus' })
  // input.props.onClick({
  //   currentTarget: {
  //     name: 'plus',
  //     parentElement: {
  //       id: 'rows'
  //     }
  //   }
  // })
  // expect(changeMockFunction).toHaveBeenCalledWith('columns', 2)
  //
  // input = instance.findByProps({ className: 'small' })
  // input.props.onChange({
  //   currentTarget: {
  //     value: 4,
  //     parentElement: {
  //       id: 'columns'
  //     }
  //   }
  // })
  // expect(changeMockFunction).toHaveBeenCalledWith('columns', 4)
})

it('handle plus button correctly', () => {
  const changeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SizeWidget widget_id="columns" angle={'F'} value={2} onWidgetSizeChange={changeMockFunction} />
  )

  const instance = component.root
  let input = instance.findByProps({ name: 'plus' })
  input.props.onClick({
    currentTarget: {
      name: 'plus',
      parentElement: {
        id: 'columns'
      }
    }
  })
  expect(changeMockFunction).toHaveBeenCalledWith('columns', 3)
})

it('handle minus button correctly', () => {
  const changeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SizeWidget widget_id="columns" angle={'F'} value={2} onWidgetSizeChange={changeMockFunction} />
  )

  const instance = component.root
  let input = instance.findByProps({ name: 'minus' })
  input.props.onClick({
    currentTarget: {
      name: 'minus',
      parentElement: {
        id: 'columns'
      }
    }
  })
  expect(changeMockFunction).toHaveBeenCalledWith('columns', 1)
})

it('handle on change correctly', () => {
  const changeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SizeWidget widget_id="columns" angle={'F'} value={2} onWidgetSizeChange={changeMockFunction} />
  )

  const instance = component.root
  let input = instance.findByProps({ className: 'small' })
  input.props.onChange({
    currentTarget: {
      value: 4,
      parentElement: {
        id: 'columns'
      }
    }
  })
  expect(changeMockFunction).toHaveBeenCalledWith('columns', 4)
})

it('handle on change back correctly', () => {
    const changeMockFunction = jest.fn()
    const component = ReactTestRenderer.create(
        <SizeWidget widget_id="columns" angle={'B'} value={2} onWidgetSizeChange={changeMockFunction} />
    )

    const instance = component.root
    let input = instance.findByProps({ className: 'small' })
    input.props.onChange({
        currentTarget: {
            value: 4,
            parentElement: {
                id: 'columns'
            }
        }
    })
    expect(changeMockFunction).toHaveBeenCalledWith('columns_back', 4)
})

it('handle on blur correctly', () => {
  const changeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SizeWidget widget_id="columns" angle={'F'} value={2} onWidgetSizeChange={changeMockFunction} />
  )

  const instance = component.root
  let input = instance.findByProps({ className: 'small' })
  input.props.onBlur({
    currentTarget: {
      value: 8,
      parentElement: {
        id: 'rows'
      }
    }
  })
  expect(changeMockFunction).toHaveBeenCalledWith('rows', 8)
})

it('handle invalid value', () => {
  const changeMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
      <SizeWidget widget_id="columns" angle={'F'} value={2} onWidgetSizeChange={changeMockFunction} />
  )

  const instance = component.root
  let input = instance.findByProps({ className: 'small' })
  input.props.onChange({
    currentTarget: {
      value: 444,
      parentElement: {
        id: 'columns'
      }
    }
  })
  expect(changeMockFunction).toHaveBeenCalledTimes(0)

  input.props.onChange({
    currentTarget: {
      value: 0,
      parentElement: {
        id: 'columns'
      }
    }
  })
  expect(changeMockFunction).toHaveBeenCalledTimes(0)
})
