import * as React from 'react'
import { Connector, PortProps } from '../GridEditorCompose'
import { Port } from './Port'

interface AvailablePortTableProps {
  all_connectors: Connector[],
  assigned_ports: PortProps[],
  language: {[key: string]: string},
  is_editable: boolean,
  right_align_connector_titles: boolean,
}

export class AvailablePortTable extends React.Component<AvailablePortTableProps, {}> {
  constructor (props: AvailablePortTableProps) {
    super(props)
    this.getUnassignedConnectors = this.getUnassignedConnectors.bind(this)
  }

  getUnassignedConnectors () {
    const all_connectors = this.props.all_connectors
    const unassigned_connectors = []
    all_connectors.forEach(connector => {
      const ports = this.props.assigned_ports.filter(c => c.connector_id == connector.connector_id)
      if (ports.length == 0) {
        unassigned_connectors.push(connector)
      }
    })
    return unassigned_connectors
  }

  render () {
    if (!this.props.is_editable) {
      return null
    }

    let ports = this.getUnassignedConnectors()
    ports.sort(function (a: Connector, b: Connector) {
      return (a.connector_title.toString() > b.connector_title.toString()) ? 1 : -1
    })

    const tds = ports.map(port => <td key={'available_port_' + port.connector_title}>
      <Port {...port}
            is_editmode={this.props.is_editable}
            is_display_new={false}
            right_align_connector_titles={this.props.right_align_connector_titles}
            language={this.props.language} />
    </td>)

    return <div className="layout-container">
      <div className="gradient">
        <span>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PORTS}</span>
      </div>
      <div>
        <table id="ports" className="ports">
          <tbody>
            <tr>
                {tds}
            </tr>
          </tbody>
        </table>
      </div>
    </div>
  }
}
