import * as React from 'react'
import { PortProps } from '../GridEditorCompose'

interface PortComponentProps extends PortProps {
  language: {[key: string]: string},
  is_editmode: boolean,
  is_display_new: boolean,
  right_align_connector_titles: boolean,
  onMergeClick: Function,
  onMouseOver?: Function,
}

export class Port extends React.Component<PortComponentProps, {}> {
  constructor (props: PortComponentProps) {
    super(props)
    this.handleMergeClick = this.handleMergeClick.bind(this)
    this.handleMouseOver = this.handleMouseOver.bind(this)
  }

  handleMergeClick (e: React.FormEvent<HTMLButtonElement>) {
    this.props.onMergeClick(e.currentTarget.value)
  }

  handleMouseOver (e: React.MouseEvent<HTMLDivElement>) {
    if (this.props.onMouseOver) {
      this.props.onMouseOver(e.currentTarget)
    }
  }

  handleDragStart (e: React.DragEvent<HTMLImageElement>) {
    const connector_img = e.currentTarget.id
    const connector_id = connector_img.substring(5) // remove "conn-"

    const parent = e.currentTarget.parentElement.parentElement
    if (parent.className == 'frame') {
      const frame_name = parent.id
      const frame_id = frame_name.substring(6) // remove "frame-"
      const frame_coords = frame_id.split('-')
      e.dataTransfer.setData('current_x', frame_coords[0])
      e.dataTransfer.setData('current_y', frame_coords[1])
      e.dataTransfer.setData('id', e.currentTarget.alt)
    }
    e.dataTransfer.setData('connector_id', connector_id)
  }

  render () {
    const title = this.props.title ? this.props.title : this.props.connector_title
    if (this.props.merged === false) {
      // render a red box with a hint
      let message = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_CONNECTOR_TYPE
      message += ' "' + this.props.connector_type_title + '" '
      message += this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_MATCHING_NAME
      message += ' "' + this.props.connector_title + '"'
      return <div className="port-container">
        <input value={title} disabled={true} />
        <div className="port-not-merged">{message}</div>
      </div>
    }

    if (this.props.is_new) {
      if (this.props.is_display_new) {
        // render a green box with a hint and a merge button
        let messageLong = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_HAS_NEW_CONNECTOR
        messageLong += ' "' + this.props.connector_title + '"'
        return <div className="port-container">
          <input value={title} disabled={true} />
          <div className="port-is_new">
            <button type="button" className="button" value={this.props.id} onClick={this.handleMergeClick} title={messageLong}>
              {'"' + this.props.connector_title + '" ' + this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGN}
            </button>
          </div>
        </div>
      } else {
        // do not render new, unassigned connectors on view mode
        return <div>&nbsp;</div>
      }
    }

    const cssClass = this.props.rotation > 0 ? 'rotate-' + this.props.rotation : ''
    let input = <input value={title} disabled={true} style={{ textAlign: 'center' }} />
    if (this.props.right_align_connector_titles) {
      input = <input value={title} disabled={true} style={{ textOverflow: 'clip', direction: 'rtl' }} />
    }
    return <div className="port-container"
      onMouseOver={this.handleMouseOver}>
      {input}
      <img id={'conn-' + this.props.connector_id}
        alt={this.props.id ? this.props.id.toString() : null}
        src={this.props.url}
        title={title}
        className={cssClass}
        draggable={this.props.is_editmode}
        onDragStart={this.handleDragStart} />
    </div>
  }
}
