import * as React from 'react'
import { Port } from './Port'
import { PortProps } from '../GridEditorCompose'

interface PortFrameProps {
  x: number,
  y: number,
  is_editable: boolean,
  is_display_new: boolean,
  language: {[key: string]: string},
  image_url: string,
  port?: PortProps,
  visibleFrameButtons?: string,
  right_align_connector_titles: boolean,
  onShowFrameButtons: Function,
  onRotateClick: Function,
  onDeleteClick: Function,
  onMergeClick: Function,
  onDrop: Function,
}

export class PortFrame extends React.Component<PortFrameProps, {}> {
  constructor (props: PortFrameProps) {
    super(props)
    this.handleDrop = this.handleDrop.bind(this)
    this.handleMouseOverParent = this.handleMouseOverParent.bind(this)
    this.handleMouseOver = this.handleMouseOver.bind(this)
    this.handleRotationClick = this.handleRotationClick.bind(this)
    this.handleDeleteClick = this.handleDeleteClick.bind(this)
    this.handleAutoAssign = this.handleAutoAssign.bind(this)
  }

  handleDragOver (e: React.DragEvent<HTMLDivElement>) {
    e.preventDefault()
  }

  handleDrop (e: React.DragEvent<HTMLDivElement>) {
    if (!this.props.is_editable) {
      return
    }

    e.preventDefault()
    const pos = e.currentTarget.id.substring(6) // remove "frame-"
    const coords = pos.split('-')
    this.props.onDrop(coords[0], coords[1], e.dataTransfer)
  }

  handleMouseOverParent (e: React.MouseEvent<HTMLDivElement>) {
    const element = e.target as HTMLElement
    if (element.className == 'frame') {
      this.props.onShowFrameButtons()
    }
  }

  handleMouseOver (currentTarget: HTMLDivElement) {
    let id = currentTarget.id
    if (currentTarget.className != 'frame') {
      id = currentTarget.parentElement.id
      if (currentTarget.parentElement.className != 'frame') {
        id = currentTarget.parentElement.parentElement.id
      }
    }
    this.props.onShowFrameButtons(id)
  }

  handleRotationClick (e: React.MouseEvent<HTMLButtonElement>) {
    // get coordinates
    const pos = e.currentTarget.parentElement.id.substring(6) // remove "frame-"
    const coords = pos.split('-')
    this.props.onRotateClick(coords[0], coords[1])
  }

  handleDeleteClick (e: React.MouseEvent<HTMLButtonElement>) {
    // get coordinates
    const pos = e.currentTarget.parentElement.id.substring(6) // remove "frame-"
    const coords = pos.split('-')
    this.props.onDeleteClick(coords[0], coords[1])
  }

  handleAutoAssign (connector_id: number) {
    this.props.onMergeClick(connector_id)
  }

  render () {
    const frame_id = 'frame-' + this.props.x + '-' + this.props.y
    // if (! this.props.is_editable) {
    //   return <div id={frame_id} className="frame">&nbsp;</div>
    // }

    let port = null
    if (this.props.port) {
      // render port
      port = <Port {...this.props.port}
        language={this.props.language}
        is_display_new={this.props.is_display_new}
        is_editmode={this.props.is_editable}
        right_align_connector_titles={this.props.right_align_connector_titles}
        onMergeClick={this.handleAutoAssign}
        onMouseOver={this.handleMouseOver} />
    }

    const buttons = []
    if (this.props.is_editable && this.props.visibleFrameButtons && this.props.visibleFrameButtons == frame_id) {
      buttons.push(<button key="rotate_port"
        type="button"
        title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__SPIN}
        className="btn spin"
        onClick={this.handleRotationClick}>
        <img src={this.props.image_url + 'arrow_rotate_clockwise-half.png'}
          alt="ROT"
          className="vam"/>
      </button>)

      buttons.push(<button key="delete_port"
        type="button"
        title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__DELETE}
        className="btn delete"
        onClick={this.handleDeleteClick}>
        <img src={this.props.image_url + 'cross.png'}
          alt="DEL"
          className="vam"/>
      </button>)
    }

    return <div id={frame_id}
      className="frame"
      onDragOver={this.handleDragOver}
      onDrop={this.handleDrop}
      onMouseOver={this.handleMouseOverParent}>
      {buttons}
      {port}
    </div>
  }
}
