import * as React from 'react'
import { Size } from '../GridEditorCompose'

interface SizeSelectorProps {
  sizes: {[key: string]: Size},
  current_size: string,
  col_span: number,
  onChange: Function,
  language: {[key: string]: string},
  is_editable: boolean,
}

export class SizeSelector extends React.Component<SizeSelectorProps, {}> {
  constructor (props: SizeSelectorProps) {
    super(props)
    this.handleChange = this.handleChange.bind(this)
  }

  handleChange (e: React.MouseEvent<HTMLAnchorElement>) {
    const size = e.currentTarget.id
    this.props.onChange(size)
  }

  render () {
    if (!this.props.is_editable) {
      return null
    }

    const divs = []
    for (const key in this.props.sizes) {
      const size = this.props.sizes[key]
      const currentSize = key == this.props.current_size ? 'size-span-selected' : ''
      divs.push(<div key={'size_' + key}
        className={'size-span ' + currentSize}>
        <a href="#"
          id={key}
          onClick={this.handleChange}>
          {size.label}
        </a>
      </div>)
    }

    return <thead>
      <tr>
        <td colSpan={this.props.col_span}>
          <span>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__SIZE + ': '}</span>
          <div className="size-span-container">
            {divs}
          </div>
        </td>
      </tr>
    </thead>
  }
}
