import * as React from 'react'

interface SizeWidgetProps {
  widget_id: string,
  angle: string,
  value: number,
  onWidgetSizeChange: Function,
}

export class SizeWidget extends React.Component<SizeWidgetProps, {}> {
  constructor (props: SizeWidgetProps) {
    super(props)
    this.handleBlur = this.handleBlur.bind(this)
    this.handleChange = this.handleChange.bind(this)
    this.handleSizeButtonClick = this.handleSizeButtonClick.bind(this)
    this.saveValue = this.saveValue.bind(this)
  }

  handleBlur (e: React.FocusEvent<HTMLInputElement>) {
    const element_id = e.currentTarget.parentElement.id
    const value = parseInt(e.currentTarget.value)
    this.saveValue(element_id, value)
  }

  handleChange (e: React.ChangeEvent<HTMLInputElement>) {
    const element_id = e.currentTarget.parentElement.id
    const value = parseInt(e.currentTarget.value)
    this.saveValue(element_id, value)
  }

  handleSizeButtonClick (e: React.MouseEvent<HTMLButtonElement>) {
    const element_id = e.currentTarget.parentElement.id
    const size = this.props.value
    const value = e.currentTarget.name == 'plus' ? size + 1 : size - 1
    this.saveValue(element_id, value)
  }

  saveValue (element_id: string, value: number) {
    if (value < 1 || value > 100) {
      return
    }
    element_id += this.props.angle == 'B' ? '_back' : ''
    this.props.onWidgetSizeChange(element_id, value)
  }

  render () {
    return <div id={this.props.widget_id.toString()}>
      <button name="minus"
        type="button"
        className="arrow-button gradient"
        onClick={this.handleSizeButtonClick}>
        <div className={'arrow down'}/>
      </button>
      <input value={this.props.value}
        className="small"
        onBlur={this.handleBlur}
        onChange={this.handleChange} />
      <button name="plus"
        type="button"
        className="arrow-button gradient"
        onClick={this.handleSizeButtonClick}>
        <div className={'arrow up'}/>
      </button>
    </div>
  }
}
