import * as React from 'react'
import { MatchTable } from '../components/MatchTable'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ADD_ALL_CONNECTORS: 'Add all connectors',
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_PIN: 'Pin',
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTOR: 'Connector',
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_MATCHES: 'matches',
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGN_CONNECTOR: 'Assign connector',
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGN_OTHER_CONNECTOR: 'Assign other connector',
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGNED_TO: 'Assigned to',
  LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ADD_CONNECTOR: 'Add connector'
}

const types = {
  1: 'RJ45',
  2: 'RJ11'
}

const connectors = {
  3: '01 (RJ45)',
  4: '02 (RJ45)'
}

const rows = [
  {
    pin: {
      id: 1,
      description: '01'
    },
    connector: {
      obj_title: 'Switch01',
      connector_id: 11,
      connector_title: '01',
      connector_type_title: 'RJ45',
      type: 'matching'
    }
  },
  {
    pin: {
      id: 2,
      description: '02'
    },
    connector: {
      obj_title: 'Switch02',
      connector_id: 12,
      connector_title: '02',
      connector_type_title: 'RJ45',
      type: 'assigned'
    }
  },
  {
    pin: {
      id: 3,
      description: '03'
    }
  }
]

it('renders match table correctly', () => {
  const addMockFunction = jest.fn()
  const addAllMockFunction = jest.fn()
  const assignMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
    <MatchTable
      language={lang}
      data={rows}
      connector_types={types}
      connectors={connectors}
      onAddConnector={addMockFunction}
      onAddAllConnectors={addAllMockFunction}
      onAssignConnector={assignMockFunction} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  let button = instance.findByProps({ name: '1' })
  button.props.onClick()
  expect(assignMockFunction).toHaveBeenCalledWith(1, 11)

  button = instance.findByProps({ name: '3' })
  button.props.onClick()
  expect(addMockFunction).toHaveBeenCalledWith(3, null)
})
