import * as React from 'react'

interface ConnectionTableProps {
    language: {[key: string]: string},
    data: ConnectionRowProps[]
}

interface Pin {
  obj_title: string,
  description: string,
  scope: string,
  url: string,
}

export interface ConnectionRowProps {
    connector_id: number,
    connector_title: string,
    connector_type_title?: string,
    pins: Pin[],
}

export class ConnectionTable extends React.Component<ConnectionTableProps, {}> {
  constructor (props: ConnectionTableProps) {
    super(props)
  }

  render () {
    const rows = []
    if (this.props.data.length > 0) {
      for (const key in this.props.data) {
        if (!this.props.data.hasOwnProperty(key)) {
          continue
        }

        const row = this.props.data[key]
        rows.push(<ConnectionRow key={row.connector_type_title + row.connector_title} {...row} />)
      }
    }

    return <table>
      <thead>
        <tr>
          <th className='gradient'>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTOR}</th>
          <th className='gradient'>&nbsp;</th>
          <th className='gradient'>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTED_PIN}</th>
        </tr>
      </thead>
      <tbody>
        {rows}
      </tbody>
    </table>
  }
}

class ConnectionRow extends React.Component<ConnectionRowProps, {}> {
  constructor (props: ConnectionRowProps) {
    super(props)
  }

  render () {
    const pins = []
    this.props.pins.forEach(pin => {
      pins.push(<a key={pin.description} href={pin.url} target="_blank">
        {pin.obj_title} - {pin.scope} - {pin.description}
      </a>)
    })

    let title = this.props.connector_title
    if (this.props.connector_type_title) {
      title += ` (${this.props.connector_type_title})`
    }
    return <tr>
      <td>{title}</td>
      <td>{'->'}</td>
      <td>{pins}</td>
    </tr>
  }
}
