import * as React from 'react'
import { Select } from './Select'
import {useState} from "react";

type TableProps = {
    language: {[key: string]: string},
    onAssignConnector: Function,
    onAddConnector: Function,
    onAddAllConnectors: Function,
    data: DataProps[],
    connector_types: {[key: number]: string},
    connectors: {[key: number]: string},
}

export type DataProps = {
    pin: PinProps,
    connector?: ConnectorProps,
}

type PinProps = {
    id: number,
    classification?: string,
    description: string,
}

type ConnectorProps = {
    obj_title: string,
    connector_id: number,
    connector_title: string,
    connector_type_title?: string,
    type: string,
}

export function MatchTable(props: TableProps) {
    const [selectedConnectorType, setSelectedConnectorType] = useState(null)

    const rows = []
    let can_add_connectors = false
    if (props.data.length > 0) {
        for (const key in props.data) {
            if (!props.data.hasOwnProperty(key)) {
                continue
            }

            const row = props.data[key]
            rows.push(<Row key={row.pin.id} {...row}
                           onAssignConnector={props.onAssignConnector}
                           onAddConnector={props.onAddConnector}
                           language={props.language}
                           connector_types={props.connector_types}
                           connectors={props.connectors} />)

            if (!row.connector) {
                can_add_connectors = true
            }
        }
    }

    let button = null
    let select = null
    if (can_add_connectors) {
        button = <button type='button'
                         className='btn'
                         disabled={! selectedConnectorType}
                         onClick={() => { props.onAddAllConnectors(selectedConnectorType) }}>
            {props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ADD_ALL_CONNECTORS}
        </button>
        select = <Select opts={props.connector_types} onChange={(type: number) => setSelectedConnectorType(type != -1 ? type : null)} />
    }

    return <table>
        <thead>
        <tr>
            <th className='gradient'>{props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_PIN}</th>
            <th className='gradient'>&nbsp;</th>
            <th className='gradient'>{props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTOR}</th>
            <th className='gradient'>{button} {select}</th>
        </tr>
        </thead>
        <tbody>
        {rows}
        </tbody>
    </table>
}

type RowProps = DataProps & {
  language: {[key: string]: string},
  onAssignConnector: Function,
  onAddConnector: Function,
  connector_types: {[key: number]: string},
  connectors: {[key: number]: string},
}

function Row(props: RowProps) {
    const [selectedConnectorType, setSelectedConnectorType] = useState(null)
    const [selectedConnectorId, setSelectedConnectorId] = useState(null)

    let text = ''
    const buttons = []
    if (props.connector && props.connector.type == 'matching') {
        text = props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_MATCHES
        text += `: ${props.connector.connector_title}`
        if (props.connector.connector_type_title) {
            text += ` (${props.connector.connector_type_title})`
        }

        buttons.push(<button type='button'
                              className='btn'
                              key={props.pin.id + '_assign'}
                              name={props.pin.id.toString()}
                              onClick={() => { props.onAssignConnector(props.pin.id, props.connector.connector_id) }}>
            {props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGN_CONNECTOR}
        </button>)
        buttons.push(<br key="br1" />)
    }
    if (props.connector && props.connector.type == 'assigned') {
        text = props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGNED_TO
        text += `: ${props.connector.obj_title} - ${props.connector.connector_title}`
        if (props.connector.connector_type_title) {
            text += ` (${props.connector.connector_type_title})`
        }
    }

    if (!props.connector) {
        buttons.push(<button type='button'
                              className='btn'
                              key={props.pin.id + '_add'}
                              name={props.pin.id.toString()}
                              disabled={!selectedConnectorType}
                              onClick={() => { props.onAddConnector(props.pin.id, selectedConnectorType) }}>
            {props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ADD_CONNECTOR}
        </button>)
        buttons.push(<Select key={props.pin.id + '_add_select'}
                              opts={props.connector_types}
                              onChange={(type: number) => setSelectedConnectorType(type != -1 ? type : null)} />)
        buttons.push(<br key="br2" />)
    }

    let pin = props.pin.classification ? props.pin.classification + ' - ' : ''
    pin += props.pin.description
    return <tr>
        <td>{pin}</td>
        <td>{'->'}</td>
        <td>{text}</td>
        <td>
            {buttons}
            <button type='button'
                    className='btn'
                    key={props.pin.id + '_assign_other'}
                    disabled={!selectedConnectorId}
                    onClick={() => { props.onAssignConnector(props.pin.id, selectedConnectorId) }}>
                {props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGN_OTHER_CONNECTOR}
            </button>
            <Select key={props.pin.id + '_assign_select'}
                    opts={props.connectors}
                    onChange={(id: number) => setSelectedConnectorId(id != -1 ? id : null)} />
        </td>
    </tr>
}
