import * as React from 'react'

export interface SelectProps {
    opts: {[key: number]: string},
    onChange: Function,
    disabled?: boolean
}

export class Select extends React.Component<SelectProps, {}> {
  constructor (props: SelectProps) {
    super(props)
    this.handleChange = this.handleChange.bind(this)
  }

  handleChange (event: React.ChangeEvent<HTMLSelectElement>) {
    this.props.onChange(event.currentTarget.value)
  }

  render () {
    const options = []
    options.push(<option key="-1" value='-1'>-</option>)

    const list = []
    // restructure items
    for (const k in this.props.opts) {
      if (!this.props.opts.hasOwnProperty(k)) {
        continue
      }

      list.push({
        key: k,
        value: this.props.opts[k]
      })
    }

    if (list.length > 0) {
      // sort items
      const sorted_list = list.sort(function (a, b) {
        return a.value.toLowerCase() > b.value.toLowerCase() ? 1 : -1
      })

      for (const index in sorted_list) {
        if (!sorted_list.hasOwnProperty(index)) {
          continue
        }
        const opt = sorted_list[index]
        const value = opt.value.replace(/ &gt; /g, ' > ')
        options.push(<option key={opt.key} value={opt.key}>{value}</option>)
      }
    }
    const disabled = this.props.disabled ? this.props.disabled : false
    return <select className='input' onChange={this.handleChange} disabled={disabled}>{options}</select>
  }
}
