import * as React from 'react'
import { CommonObject, Connector } from './ParkingDeckCompose'

interface SelectableTableProps {
  language: {[key: string]: string},
  side: string,
  title: string,
  object: CommonObject,
  isEditable: boolean,
  rows: Connector[],
  onSelectionChange: Function,
  selected_connector?: number,
  onTitleChange?: Function,
}

export class SelectableTable extends React.Component<SelectableTableProps, {}> {
  constructor (props: SelectableTableProps) {
    super(props)
    this.handleTitleChange = this.handleTitleChange.bind(this)
    this.handleSelectionChange = this.handleSelectionChange.bind(this)
  }

  handleTitleChange (event: React.ChangeEvent<HTMLInputElement>) {
    const text = event.currentTarget.value
    this.props.onTitleChange(text)
  }

  handleSelectionChange (connector_id: number) {
    this.props.onSelectionChange(this.props.side, connector_id)
  }

  render () {
    let parked_object
    if (!this.props.isEditable) {
      parked_object = this.props.object.obj_title
    } else {
      parked_object = <input type="text"
        className="input input-small"
        defaultValue={this.props.object.obj_title}
        onChange={this.handleTitleChange}
        placeholder={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ALTERNATIVE_TITLE} />
    }

    const rows = []
    if (this.props.rows.length > 0) {
      this.props.rows.forEach(row => {
        const isSelected = this.props.selected_connector && this.props.selected_connector == row.connector_id
        rows.push(<SelectableTableRow
          key={row.connector_id}
          {...row}
          language={this.props.language}
          onSelectionChange={this.handleSelectionChange}
          is_selected={isSelected} />
        )
      })
    }

    return <table>
      <thead>
        <tr>
          <th className='gradient'>{this.props.title}: {parked_object}</th>
        </tr>
      </thead>
      <tbody>
        {rows}
      </tbody>
    </table>
  }
}

interface SelectableTableRowProps extends Connector {
  language: {[key: string]: string},
  onSelectionChange: Function,
  is_selected: boolean,
}

class SelectableTableRow extends React.Component<SelectableTableRowProps, {}> {
  constructor (props: SelectableTableRowProps) {
    super(props)
  }

  renderCell (c: Connector) {
    let title = null
    let connection = null
    if (c.connector_title) {
      const type = c.connector_type == 1 ? 'IN' : 'OUT'
      title = c.connector_title + ' (' + type + ')'
    }

    if (c.connection.object_title) {
      const connection_title = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED_WITH + ': ' +
          c.connection.object_title + ' - ' + c.connection.connector_title
      connection = <a href={c.connection.connector_url} target="_blank">{connection_title}</a>
    }

    const className = c.connector_type == 1 ? 'connector-in' : 'connector-out'
    return <td className={className} onClick={() => { this.props.onSelectionChange(this.props.connector_id) }}>
      <span>{title}</span>
      <br />
      {connection}
    </td>
  }

  render () {
    const className = this.props.is_selected ? 'selected' : ''
    return <tr className={className}>
      {this.renderCell(this.props)}
    </tr>
  }
}
