const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const { resolve } = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');

module.exports = {
  entry: {
    analyzer:[
      './src/analyzer/index.tsx',
      './../sass/analyzer.scss'
    ],
    editor:[
      './src/editor/index.tsx',
      './../sass/patch_editor.scss'
    ],
    grid_editor:[
      './src/grid_editor/index.tsx',
      './../sass/grid_editor.scss'
    ],
    mapper:[
      './src/mapper/index.tsx',
      './../sass/pin_mapper.scss'
    ],
    parking_deck:[
      './src/parking_deck/index.tsx',
      './../sass/parking_deck.scss'
    ]
  },
  output: {
    filename: '[name]/[name].[contenthash].js',
    path: resolve(__dirname, './../dist/'),
    publicPath: 'src/classes/modules/smview_patch/assets/dist/',
  },
  resolve: {
    cacheWithContext: false,
    extensions: ['.ts', '.tsx', '.js', '.json'],
    symlinks: false,
  },
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        include: resolve(__dirname, 'src'),
        use: 'ts-loader',
        exclude: /node_modules/,
      },
      // sass to css file
      {
        test: /\.(scss)$/,
        include: resolve(__dirname, './../sass'),
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader'],
        sideEffects: true
      }
    ]
  },
  optimization: {
     minimize: false,
    // separate 3rd-party libs because they won't change so often
    moduleIds: 'deterministic',
    chunkIds: 'named',
    splitChunks: {
      chunks: 'async',
      maxAsyncRequests: 6,
      maxInitialRequests: 4,
      cacheGroups: {
        yfiles: {
          test: /[\\/]yfiles[\\/]/,
          name: 'yfiles',
          chunks: 'all',
          priority: 10,
          filename: 'yfiles.[contenthash].js'
        },
        vendors: {
          test: /[\\/]node_modules[\\/]/,
          name: 'vendors',
          chunks: 'all',
          filename: 'vendors.[contenthash].js'
        }
      }
    }
  },
  plugins: [
    new CleanWebpackPlugin(),
    new HtmlWebpackPlugin({
      path: resolve(__dirname, './../dist/'),
      inject: false,
      chunks: ['analyzer'],
      filename: resolve(__dirname, './../dist/analyzer/index.html'),
      templateContent: ({htmlWebpackPlugin}) => `${htmlWebpackPlugin.tags.headTags} ${htmlWebpackPlugin.tags.bodyTags}`
    }),
    new HtmlWebpackPlugin({
      path: resolve(__dirname, './../dist/'),
      inject: false,
      chunks: ['editor'],
      filename: resolve(__dirname, './../dist/editor/index.html'),
      templateContent: ({htmlWebpackPlugin}) => `${htmlWebpackPlugin.tags.headTags} ${htmlWebpackPlugin.tags.bodyTags}`
    }),
    new HtmlWebpackPlugin({
      path: resolve(__dirname, './../dist/'),
      inject: false,
      chunks: ['grid_editor'],
      filename: resolve(__dirname, './../dist/grid_editor/index.html'),
      templateContent: ({htmlWebpackPlugin}) => `${htmlWebpackPlugin.tags.headTags} ${htmlWebpackPlugin.tags.bodyTags}`
    }),
    new HtmlWebpackPlugin({
      path: resolve(__dirname, './../dist/'),
      inject: false,
      chunks: ['mapper'],
      filename: resolve(__dirname, './../dist/mapper/index.html'),
      templateContent: ({htmlWebpackPlugin}) => `${htmlWebpackPlugin.tags.headTags} ${htmlWebpackPlugin.tags.bodyTags}`
    }),
    new HtmlWebpackPlugin({
      path: resolve(__dirname, './../dist/'),
      inject: false,
      chunks: ['parking_deck'],
      filename: resolve(__dirname, './../dist/parking_deck/index.html'),
      templateContent: ({htmlWebpackPlugin}) => `${htmlWebpackPlugin.tags.headTags} ${htmlWebpackPlugin.tags.bodyTags}`
    }),
    new MiniCssExtractPlugin({
      filename: '[name]/style.css'
    }),
    new CopyWebpackPlugin([
      {from: '../../../smview_maps/assets/css/yfiles.css', to: '.'},
      {from: '../../../smview_maps/assets/js/lib/license.js', to: '.'}
    ])
  ]
};
