<?php

/**
 * i-doit
 *
 * Auth: Authentication class of SM-VIEW Patch.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_auth_smview_patch extends isys_auth implements isys_auth_interface
{

    /**
     * Container for singleton instance.
     *
     * @var isys_auth_smview_patch
     */
    private static $m_instance  = null;

    /**
     * Retrieve singleton instance of authorization class.
     *
     * @return isys_auth_smview_patch
     */
    public static function instance(): isys_auth_smview_patch
    {
        global $g_comp_database;

        if (self::$m_dao === null) {
            self::$m_dao = new isys_auth_dao($g_comp_database);
        }

        if (self::$m_instance === null) {
            self::$m_instance = new self;
        }
        return self::$m_instance;
    }

    /**
     * Method for returning the available auth-methods. This will be used for the GUI.
     *
     * @return array
     * @throws Exception
     */
    public function get_auth_methods(): array
    {
        $lang = isys_application::instance()->container->get('language');
        return [
            'planner' => [
                'title'  => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__PLANNER_PERMISSION'),
                'type'   => 'boolean',
                'rights' => [
                    isys_auth::VIEW,
                    isys_auth::EDIT,
                    isys_auth::EXECUTE,
                ]
            ],
        ];
    }

    /**
     * Get ID of related module.
     *
     * @return int
     */
    public function get_module_id()
    {
        return defined_or_default('C__MODULE__SMVIEW_PATCH');
    }

    /**
     * Get title of related module.
     *
     * @return string
     */
    public function get_module_title(): string
    {
        return 'LC__CMDB__CATG__SMVIEW_PATCH__TREE_TITLE';
    }
}
