<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Patch Grid.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_patch_editor extends isys_cmdb_dao_category_global
{
    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_patch_editor';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * Method for returning the properties.
     *
     * @return array
     */
    protected function properties()
    {
        /*
         *
         * This is only a fake object, but it is required to add a global category!
         *
         */
        return [];
    }

    /**
     * Retrieves the number of saved category-entries to the given object.
     *
     * @param  int $objectId
     * @return int
     * @throws isys_exception_database
     */
    public function get_count($objectId = null)
    {
        if (empty($objectId)) {
            return 0;
        }

        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $location_dao = isys_cmdb_dao_category_g_location::instance($this->m_db);
        $row = $global_dao->get_object($objectId)->get_row();
        if ($row['isys_obj_type__const'] == 'C__OBJTYPE__ENCLOSURE') {
            // check if rack has devices
            return $location_dao->get_data(
                null,
                null,
                "AND isys_catg_location_list__parentid = $objectId",
                null,
                defined_or_default('C__RECORD_STATUS__NORMAL'))->count();
        } else {
            // check if device is in a rack
            $location_row = $location_dao->get_data(null, $objectId)->get_row();
            if ($location_row) {
                $parent_id = $location_row['isys_catg_location_list__parentid'];
                $item = $global_dao->get_object($parent_id)->get_row();

                // check parent object type
                return $item['isys_obj_type__const'] == 'C__OBJTYPE__ENCLOSURE' ? 1 : 0;
            }
        }
        return 0;
    }

    /**
     * Get rack with devices.
     *
     * @param int|null $rack_obj_id
     * @param int|null $device_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_rack($rack_obj_id = null, $device_id = null)
    {
        if (empty($rack_obj_id)) {
            $location_dao = isys_cmdb_dao_category_g_location::instance($this->m_db);
            $location = $location_dao->get_data(null, $device_id)->get_row();

            if (empty($location)) {
                return [];
            }

            $parent_id = $location['isys_catg_location_list__parentid'];
            $parent_location = $location_dao->get_data(null, $parent_id)->get_row();
            $is_rack = $parent_location['isys_obj_type__const'] == 'C__OBJTYPE__ENCLOSURE';
            $rack_obj_id = $is_rack ? $location['isys_catg_location_list__parentid'] : $parent_location['isys_catg_location_list__parentid'];
        }

        $dao = isys_cmdb_dao_category_g_global::instance($this->m_db);
        $global = $dao->get_data(null, $rack_obj_id)->get_row();

        $form_factor_dao = isys_cmdb_dao_category_g_formfactor::instance($this->m_db);
        $form_factor = $form_factor_dao->get_data(null, $rack_obj_id)->get_row();

        $he_dao = isys_cmdb_dao_category_g_smview_patch_formfactor::instance($this->m_db);
        $half_he = $he_dao->get_data(null, $rack_obj_id)->get_row();
        $has_half_he = $half_he && $half_he['isys_catg_smview_patch_formfactor_list__half_he'] == '1';

        $chassis_dao = isys_cmdb_dao_category_s_enclosure::instance($this->m_db);
        $chassis = $chassis_dao->get_data(null, $rack_obj_id)->get_row();

        return array(
            'id'                     => $rack_obj_id,
            'title'                  => $global['isys_obj__title'],
            'units'                  => $form_factor['isys_catg_formfactor_list__rackunits'],
            'has_half_he'            => $has_half_he,
            'sorting'                => $chassis['isys_cats_enclosure_list__slot_sorting'],
            'devices_front'          => $this->_get_assigned_devices($rack_obj_id, 'F', 'H'),
            'devices_front_vertical' => $this->_get_assigned_devices($rack_obj_id, 'F', 'V'),
            'devices_back'           => $this->_get_assigned_devices($rack_obj_id, 'B', 'H'),
            'devices_back_vertical'  => $this->_get_assigned_devices($rack_obj_id, 'B', 'V'),
            'vertical_slots_front'   => $chassis['isys_cats_enclosure_list__vertical_slots_front'],
            'vertical_slots_back'    => $chassis['isys_cats_enclosure_list__vertical_slots_rear'],
            'print_url'              => isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                'request'         => 'print_rack',
                'instance_id'     => $rack_obj_id,
            ]),
        );
    }

    /**
     * Get assigned rack devices.
     *
     * @param  int $rack_obj_id
     * @param  string $angle
     * @param  string $layout
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _get_assigned_devices($rack_obj_id, $angle = 'F', $layout = 'H')
    {
        $insertion = $angle == 'F' ? 1 : 0;
        $option = $layout == 'H' ? 3 : 4;
        $form_factor_dao = isys_cmdb_dao_category_g_formfactor::instance($this->m_db);
        $chassis_dao = isys_cmdb_dao_category_s_chassis_view::instance($this->m_db);
        $formfactor_dao = isys_cmdb_dao_smview_patch_formfactor::instance($this->m_db);
        $dao = isys_cmdb_dao_category_g_virtual_object::instance($this->m_db);
        $data = $dao->get_data_as_array(
            null,
            $rack_obj_id,
            ' AND (isys_catg_location_list__insertion=2 OR isys_catg_location_list__insertion=' . $insertion . ')' .
                      ' AND isys_catg_location_list__option=' . $option,
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL')
        );
        $lang = isys_application::instance()->container->get('language');

        $devices = [];
        foreach ($data as $d) {
            $form_factor = $form_factor_dao->get_data(null, $d['isys_obj__id'])->get_row();
            $slots = [];
            $is_chassis = false;
            $chassis_width = 0;
            $chassis_height = 0;
            if ($this->_is_chassis($d)) {
                $is_chassis = true;

                // display enclosed devices only from front side
                $chassis_angle = 'F';

                $chassis = $chassis_dao->get_data(null, $d['isys_obj__id'])->get_row();
                $chassis_width = $chassis_angle == 'F' ? $chassis['isys_cats_chassis_view_list__front_width'] : $chassis['isys_cats_chassis_view_list__rear_width'];
                $chassis_height = $chassis_angle == 'F' ? $chassis['isys_cats_chassis_view_list__front_height'] : $chassis['isys_cats_chassis_view_list__rear_height'];

                $slots = $this->_get_slot_data($d['isys_obj__id'], $chassis_angle);
            }

            $he_dao = isys_cmdb_dao_category_g_smview_patch_formfactor::instance($this->m_db);
            $half_he = $he_dao->get_data(null, $d['isys_obj__id'])->get_row();
            $has_half_he = $half_he && $half_he['isys_catg_smview_patch_formfactor_list__half_he'] == '1';

            $status_color = null;
            if ($d['isys_obj__isys_cmdb_status__id'] != C__CMDB_STATUS__IN_OPERATION) {
                $sql = "SELECT * FROM isys_cmdb_status WHERE isys_cmdb_status__id = " . $d['isys_obj__isys_cmdb_status__id'];
                $c = $he_dao->retrieve($sql)->get_row();
                $status_color = '#' . $c['isys_cmdb_status__color'];
            }

            $form_factor_row = $formfactor_dao->get_object('obj_id', $d['isys_obj__id']);
            $devices[] = array(
                'id'             => $d['isys_obj__id'],
                'title'          => $d['isys_obj__title'],
                'status_color'   => $status_color,
                'type_id'        => $d['isys_obj_type__id'],
                'type_title'     => $lang->get($d['isys_obj_type__title']),
                'type_color'     => $d['isys_obj_type__color'],
                'position'       => $d['isys_catg_location_list__pos'],
                'he'             => $form_factor['isys_catg_formfactor_list__rackunits'],
                'has_half_he'    => $has_half_he,
                'image_url'      => isys_cmdb_dao_smview_patch_picture::get_device_image($d['isys_obj__id'], $angle == 'F' ? 'front' : 'back'),
                'is_chassis'     => $is_chassis,
                'chassis_width'  => $chassis_width,
                'chassis_height' => $chassis_height,
                'slots'          => $slots,
                'comment'        => $form_factor_row ? $form_factor_row['comment'] : '',
            );
        }
        return $devices;
    }

    /**
     * Get slot data of a chassis.
     *
     * @param  int $rack_segment_id
     * @param  string $angle
     * @return array
     * @throws isys_exception_database
     */
    private function _get_slot_data($rack_segment_id, $angle = 'F')
    {
        $chassis_dao = isys_cmdb_dao_category_s_chassis::instance($this->m_db);
        $slot_dao = isys_cmdb_dao_category_s_chassis_slot::instance($this->m_db);
        $object_dao = isys_cmdb_dao_category_g_global::instance($this->m_db);

        $slots = [];
        $insertion = $angle == 'F' ? 1 : 0;
        $res = $slot_dao->get_data(
            null,
            $rack_segment_id,
            'AND (isys_cats_chassis_slot_list__insertion=2 OR isys_cats_chassis_slot_list__insertion=' . $insertion . ')',
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL')
        );
        while ($row = $res->get_row()) {
            // get assigned devices of this slot
            $l_assigned_slots = array_keys($slot_dao->get_assigned_chassis_items_by_cat_id($row['isys_cats_chassis_slot_list__id']));
            $l_slot_res = $chassis_dao->get_data(null, $rack_segment_id, '', null, defined_or_default('C__RECORD_STATUS__NORMAL'));
            $assigned_slot_devices = [];
            while ($l_slot_row = $l_slot_res->get_row()) {
                if ($l_slot_row['isys_connection__isys_obj__id'] && in_array($l_slot_row['isys_cats_chassis_list__id'], $l_assigned_slots)) {
                    $obj = $object_dao->get_data(null, $l_slot_row['isys_connection__isys_obj__id'])->get_row();
                    if (! empty($obj)) {
                        $assigned_slot_devices[] = array(
                            'id'        => $obj['isys_obj__id'],
                            'title'     => $obj['isys_obj__title'],
                            'image_url' => isys_cmdb_dao_smview_patch_picture::get_device_image($obj['isys_obj__id'], $angle == 'F' ? 'front' : 'back'),
                        );
                    }
                }
            }

            $slots[] = array(
                'id'      => $row['isys_cats_chassis_slot_list__isys_obj__id'],
                'title'   => $row['isys_cats_chassis_slot_list__title'],
                'x_from'  => $row['isys_cats_chassis_slot_list__x_from'],
                'x_to'    => $row['isys_cats_chassis_slot_list__x_to'],
                'y_from'  => $row['isys_cats_chassis_slot_list__y_from'],
                'y_to'    => $row['isys_cats_chassis_slot_list__y_to'],
                'devices' => $assigned_slot_devices,
            );
        }
        return $slots;
    }

    /**
     * Get grid by object ID.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    public function get_grid($obj_id)
    {
        if (empty($obj_id)) {
            return [];
        }

        $lang = isys_application::instance()->container->get('language');
        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $global_row = $global_dao->get_object($obj_id)->get_row();
        $device = [
            'id'          => $obj_id,
            'title'       => $global_row['isys_obj__title'],
            'description' => $this->_get_description_first_line($global_row['isys_obj__description']),
            'type_title'  => $lang->get($global_row['isys_obj_type__title']),
        ];

        if ($this->_is_chassis($global_row)) {
            // add chassis data
            $device['is_chassis'] = true;
            $device = $this->_add_chassis_data($device, $global_row['isys_obj__id']);
        }

        $grid_list_dao = new isys_cmdb_dao_category_g_smview_patch_grid($this->m_db);
        $item = $grid_list_dao->get_data(null, $obj_id)->get_row();
        $available_connectors = $this->get_available_connectors($obj_id);
        if (empty($item)) {
            $device = array_merge($device, [
                'url'         => isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT => $global_row['isys_obj__id'],
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__SMVIEW_PATCH_GRID'),
                ]),
                'grid_title'  => null,
            ]);

            $colors = isys_module_smview_patch::get_editor_colors();
            $color_from_cable = isys_settings::get('smview_patch.editor.color_from_cable', false);
            $colors['color_from_cable'] = $color_from_cable ? '1' : '0';
            return [
                'device'             => $device,
                'connectors'         => [],
                'port_status_colors' => $colors,
                'has_connectors'     => count($available_connectors) > 0
            ];
        }

        $list_id = $item['isys_catg_smview_patch_grid_list__id'];
        $grid_template_id = $item['isys_catg_smview_patch_grid_list__grid_template_id'];

        $dao = isys_cmdb_dao_category_g_smview_patch_grid::instance($this->m_db);
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        $grid_template = $grid_template_dao->get_object('id', $grid_template_id);
        $assigned_connectors = $this->get_assigned_connector_templates($list_id, $grid_template_id, $available_connectors);

        $device = array_merge($device, [
            'url'          => isys_helper_link::create_url([C__CMDB__GET__OBJECT => $obj_id]),
            'print_url'    => isys_helper_link::create_url([
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                'request'         => 'print_device',
                'instance_id'     => $obj_id,
            ]),
            'grid_title'        => $grid_template['title'],
            'column_count'      => $grid_template['column_count'],
            'row_count'         => $grid_template['row_count'],
            'column_count_back' => $grid_template['column_count_back'],
            'row_count_back'    => $grid_template['row_count_back'],
            'is_template'       => $grid_template['is_template'],
            'used_by'           => $dao->get_objects_using_grid_template($grid_template_id),
            'is_chassis'        => false,
        ]);

        // check if device is in a chassis
        $location_dao = isys_cmdb_dao_category_g_location::instance($this->m_db);
        $location_row = $location_dao->get_data(null, $obj_id)->get_row();
        if ($location_row) {
            // get parent id
            $parent_id = $location_row['isys_catg_location_list__parentid'];
            $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
            $item = $global_dao->get_object($parent_id)->get_row();

            // get parent object type
            if ($this->_is_chassis($item)) {
                $device['is_chassis'] = false;
                // set title and type of chassis
                $device['title'] = $item['isys_obj__title'];
                $device['type_title'] = $lang->get($item['isys_obj_type__title']);
                $device = $this->_add_chassis_data($device, $item['isys_obj__id']);
            }
        }

        return [
            'device'     => $device,
            'connectors' => $assigned_connectors,
        ];
    }

    /**
     * Get first line of an object description.
     *
     * @param  string $rawDescription
     * @return string|null
     */
    private function _get_description_first_line($rawDescription)
    {
        if (! $rawDescription) {
            return null;
        }
        $lines = explode("\n", $rawDescription);
        return strip_tags($lines[0]);
    }

    /**
     * Check if an object has specific category 'C__CATS__CHASSIS' assigned.
     *
     * @param  array $item
     * @return boolean
     * @throws isys_exception_database
     */
    private function _is_chassis($item)
    {
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $obj_type_id = $item['isys_obj_type__id'];
        if (empty($obj_type_id)) {
            return false;
        }

        $sql = "SELECT
                    s.isysgui_cats__id,
                    s.isysgui_cats__title,
                    s.isysgui_cats__const,
                    t.isys_obj_type__title
                FROM isys_obj_type t
                JOIN isysgui_cats s ON t.isys_obj_type__isysgui_cats__id = s.isysgui_cats__id
                WHERE t.isys_obj_type__id = $obj_type_id
                AND s.isysgui_cats__const = 'C__CATS__CHASSIS'";
        $row = $dao->retrieve($sql)->get_row();
        return ! empty($row);
    }

    /**
     * Add chassis data to device.
     *
     * @param  array $device
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    private function _add_chassis_data($device, $obj_id)
    {
        // add chassis data
        $chassis_dao = isys_cmdb_dao_category_s_chassis_view::instance($this->m_db);
        $chassis = $chassis_dao->get_data(null, $obj_id)->get_row();

        $device['chassis_front'] = [
            'id'     => $obj_id,
            'title'  => $chassis['isys_obj__title'],
            'width'  => $chassis['isys_cats_chassis_view_list__front_width'],
            'height' => $chassis['isys_cats_chassis_view_list__front_height'],
            'slots'  => $this->_get_slot_data($obj_id, 'F'),
        ];
        $device['chassis_back'] = [
            'id'     => $obj_id,
            'title'  => $chassis['isys_obj__title'],
            'width'  => $chassis['isys_cats_chassis_view_list__rear_width'],
            'height' => $chassis['isys_cats_chassis_view_list__rear_height'],
            'slots'  => $this->_get_slot_data($obj_id, 'B'),
        ];
        return $device;
    }

    /**
     * Get configured connector templates.
     *
     * @param  int $list_id
     * @param  int $grid_template_id
     * @param  array $available_connectors
     * @return array
     * @throws isys_exception_database
     */
    public function get_assigned_connector_templates($list_id, $grid_template_id, $available_connectors)
    {
        $grid_connector_dao = new isys_cmdb_dao_smview_patch_grid_connector($this->m_db);
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template($this->m_db);

        // get all connector templates
        $connector_templates = [];
        $tpls = $connector_template_dao->filter_objects(array('grid_template_id' => $grid_template_id));
        foreach ($tpls as $tpl) {
            $connector_templates[$tpl['id']] = $tpl;
        }

        // get all assigned connector instances
        $connections = $grid_connector_dao->filter_objects(array('grid_list_id' => $list_id));
        $connectors = [];
        foreach ($connections as $c) {
            $connector_template_id = $c['connector_template_id'];
            if (! isset($connector_templates[$connector_template_id])) {
                continue;
            }
            $connector_template = $connector_templates[$connector_template_id];
            $connector_id = $c['connector_id'];
            if (! isset($available_connectors[$connector_id])) {
                // do not add unassigned connectors
                continue;
            }

            $conn_instance = $available_connectors[$connector_id];
            $connector = array_merge($connector_template, $conn_instance);
            $connector['title'] = $connector['title'] ?: $connector['connector_title'];
            $connector['status'] = $c['connector_status'];

            // get corresponding virtual connector if is set
            $connector['virtual_connection'] = $this->_get_virtual_connection($connector_id);
            $connectors[] = $connector;
        }
        return $connectors;
    }

    /**
     * Get virtual connection.
     *
     * @param  int $connector_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_virtual_connection($connector_id)
    {
        $virtual_connection_dao = new isys_cmdb_dao_smview_patch_virtual_connection($this->m_db);
        $sibling = null;
        $virtual_connectorA = $virtual_connection_dao->get_object('from_connector_id', $connector_id);
        if ($virtual_connectorA) {
            $sibling = $virtual_connectorA['to_connector_id'];
        }
        $virtual_connectorB = $virtual_connection_dao->get_object('to_connector_id', $connector_id);
        if ($virtual_connectorB) {
            $sibling = $virtual_connectorB['from_connector_id'];
        }
        if (empty($sibling)) {
            return [];
        }

        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $picture_dao = new isys_cmdb_dao_smview_patch_picture($this->m_db);
        $c = $connector_dao->get_data($sibling)->get_row();
        return [
            'id'          => $sibling,
            'title'       => $c['isys_catg_connector_list__title'],
            'obj_id'      => $c['isys_obj__id'],
            'obj_title'   => $c['isys_obj__title'],
            'obj_url'     => $picture_dao->get_file_url($c['isys_obj__id']),
            'cable_title' => ! empty($virtual_connectorA) ? $virtual_connectorA['cable_name'] : $virtual_connectorB['cable_name'],
            'status'      => $virtual_connectorA['status'],
        ];
    }

    /**
     * Get available connectors of this object.
     *
     * @param int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_available_connectors($obj_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $l_data = $connector_dao->get_data(
            null,
            $obj_id,
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__OUTPUT,
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL')
        );

        $connectors = [];
        while ($row = $l_data->get_row()) {
            $id = $row['isys_catg_connector_list__id'];
            $connectors[$id] = $this->get_connector_by_id($id);
        }
        return $connectors;
    }

    /**
     * Get available connectors of this object.
     *
     * @param  int $connector_id
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    public function get_connector_by_id($connector_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $picture_dao = new isys_cmdb_dao_smview_patch_picture($this->m_db);
        $row = $connector_dao->get_data($connector_id)->get_row();
        $is_patch_connector = $row['isys_catg_connector_list__type'] == C__CONNECTOR__OUTPUT;
        if (! $is_patch_connector && $row['isys_catg_connector_list__isys_catg_connector_list__id']) {
            // use only patch connectors (outgoing ports)
            $row = $connector_dao->get_data($row['isys_catg_connector_list__isys_catg_connector_list__id'])->get_row();
        }

        $connector_type_id = $row['isys_catg_connector_list__isys_connection_type__id'];
        $pic = $picture_dao->get_object('id', $connector_type_id);
        $id = $row['isys_catg_connector_list__id'];

        $conn_dao = isys_cmdb_dao_smview_patch_grid_connector::instance($this->m_db);
        $conn_row = $conn_dao->get_object('connector_id', $id);
        $status = ! empty($conn_row) ? $conn_row['connector_status'] : 6;

        $connector = [
            'connector_id'         => $id,
            'connector_title'      => $row['isys_catg_connector_list__title'],
            'connector_type'       => $row['isys_catg_connector_list__type'],
            'connector_type_id'    => $connector_type_id,
            'connector_type_title' => ! empty($pic) ? $pic['title'] : null,
            'connector_status'     => $status,
            'sibling_id'           => $row['isys_catg_connector_list__isys_catg_connector_list__id'],
            'url'                  => $picture_dao->get_file_url($pic['obj_id']),
            'link_type'            => null,
            'patch'                => [],
            'link'                 => [],
            'virtual_connection'   => [],
        ];

        $is_patch_connector = $row['isys_catg_connector_list__type'] == C__CONNECTOR__OUTPUT;
        $key = $is_patch_connector ? 'patch' : 'link';
        $pin_connections = $this->_get_connected_cable_pins($connector_id);
        if (! empty($pin_connections)) {
            $connector['cable_pins'] = $pin_connections;
        } else {
            // get some additional data for the connected item (output)
            if ($row['con_connector'] > 0) {
                $l_conn_row = $this->_get_connector($row['con_connector']);
                $connected_object = $this->_get_connected_object_information($row, $l_conn_row);
                $connected_object = $this->_add_cable_information($connected_object, $row);
                $connector[$key] = $connected_object;
            }
        }

        $key = $is_patch_connector ? 'link' : 'patch';
        $pin_connections = $this->_get_connected_cable_pins($row['isys_catg_connector_list__isys_catg_connector_list__id']);
        if (! empty($pin_connections)) {
            $connector['cable_pins'] = $pin_connections;
        } else {
            // get link connection (input)
            if (! empty($row['isys_catg_connector_list__isys_catg_connector_list__id'])) {
                $link_row = $connector_dao->get_data($row['isys_catg_connector_list__isys_catg_connector_list__id'])->get_row();
                if ($link_row['con_connector'] > 0) {
                    $l_conn_row = $this->_get_connector($link_row['con_connector']);
                    $type = $l_conn_row['isys_obj_type__const'];
                    $connector['link_type'] = $type == 'C__OBJTYPE__PATCH_PANEL' ? 'link' : 'dose';
                    $connected_object = $this->_get_connected_object_information($link_row, $l_conn_row);
                    $connected_object = $this->_add_cable_information($connected_object, $link_row);
                    $connector[$key] = $connected_object;
                }
            }
        }

        // get corresponding virtual connector if is set
        $connector['virtual_connection'] = $this->_get_virtual_connection($connector_id);

        return $connector;
    }

    /**
     * Get connected SM-VIEW cable pins.
     *
     * @param  int $connector_id
     * @return array|null
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _get_connected_cable_pins($connector_id)
    {
        if (! class_exists('isys_cmdb_dao_smview_cable_pin')) {
            // SM-VIEW cable is not installed
            return null;
        }
        $cable_pin_dao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        $pin_connections = $cable_pin_dao->filter_objects(['connector_id' => $connector_id]);
        if (empty($pin_connections)) {
            return null;
        }

        $lang = isys_application::instance()->container->get('language');
        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $connections = [];
        foreach ($pin_connections as $conn) {
            $obj = $global_dao->get_object($conn['obj_id'])->get_row();
            $route = $this->_get_route_by_wire_pin($conn['connected_wire_pin_id']);
            $connections[] = [
                'id' => $conn['id'],
                'title' => $conn['description'],
                'scope' => $conn['classification'],
                'obj_id' => $obj['isys_obj__id'],
                'obj_title' => $obj['isys_obj__title'],
                'obj_type_title' => $lang->get($obj['isys_obj_type__title']),
                'obj_url' => isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT   => $obj['isys_obj__id'],
                    C__CMDB__GET__CATG     => defined_or_default('C__CATG__SMVIEW_CABLE_PIN'),
                    C__CMDB__GET__CATLEVEL => $conn['id'],
                ]),
                'is_cable_pin' => true,
                'route_name' => ! empty($route) ? $route['isys_obj__title'] : null,
                'route_url' => empty($route) ? null : isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT => $route['isys_obj__id'],
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__SMVIEW_CABLE_ROUTE'),
                ]),
            ];
        }

        return $connections;
    }

    /**
     * Get route by wire pin.
     *
     * @param  int $wire_pin_id
     * @return array|null
     * @throws isys_exception_database
     */
    private function _get_route_by_wire_pin($wire_pin_id)
    {
        if (empty($wire_pin_id)) {
            return null;
        }

        $wire_pin_dao = new isys_cmdb_dao_smview_cable_wire_pin($this->m_db);
        $wp = $wire_pin_dao->get_object('id', $wire_pin_id);
        if (empty($wp)) {
            return null;
        }

        $route_element_dao = new isys_cmdb_dao_smview_cable_route_element($this->m_db);
        $re = $route_element_dao->get_object('wire_id', $wp['wire_id']);
        if (empty($re)) {
            return null;
        }

        $route_dao = new isys_cmdb_dao_smview_cable_route($this->m_db);
        $r = $route_dao->get_object('id', $re['route_id']);
        if (empty($r)) {
            return null;
        }

        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        return $global_dao->get_object($r['obj_id'])->get_row();
    }

    /**
     * Get connector by ID.
     *
     * @param  int $connector_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_connector($connector_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $l_sql = 'SELECT isys_obj__id, isys_obj__title, isys_obj_type__id, isys_obj_type__title, isys_obj_type__const,
                isys_catg_connector_list__type FROM isys_obj
                INNER JOIN isys_obj_type ON isys_obj_type__id = isys_obj__isys_obj_type__id
                INNER JOIN isys_catg_connector_list ON isys_catg_connector_list__isys_obj__id = isys_obj__id
                WHERE isys_catg_connector_list__id = ' . $connector_dao->convert_sql_id($connector_id) . ';';

        return $connector_dao->retrieve($l_sql)->get_row();
    }

    /**
     * Add cable information.
     *
     * @param  array $connector
     * @param  array $row
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _add_cable_information($connector, $row)
    {
        $cable_dao = new isys_cmdb_dao_category_g_cable($this->m_db);
        $sql = 'SELECT * FROM isys_catg_cable_list l
                    LEFT JOIN isys_depth_unit ON isys_depth_unit__id = isys_catg_cable_list__isys_depth_unit__id
                    LEFT JOIN isys_cable_colour c ON l.isys_catg_cable_list__isys_cable_colour__id = c.isys_cable_colour__id
                    WHERE isys_catg_cable_list__isys_obj__id = ' . $row['cable_id'];
        $cable_row = $cable_dao->retrieve($sql)->get_row();

        $lang = isys_application::instance()->container->get('language');
        $connector['cable_id'] = $row['cable_id'];
        $connector['cable_title'] = $row['cable_title'];
        if (! empty($row['cable_id'])) {
            $connector['cable_color_id'] = $cable_row['isys_cable_colour__id'];
            $connector['cable_color'] = $cable_row['isys_cable_colour__title'];

            $sql = "SELECT * FROM isys_cable_type WHERE isys_cable_type__id = " . $cable_dao->convert_sql_int($cable_row['isys_catg_cable_list__isys_cable_type__id']);
            $cable_type = $cable_dao->retrieve($sql)->get_row();
            if ($cable_type) {
                $connector['cable_type_id'] = $cable_type['isys_cable_type__id'];
                $connector['cable_type'] = $cable_type['isys_cable_type__title'];
            }

            $l = (float) $cable_row['isys_catg_cable_list__length'];
            $f = (int) $cable_row['isys_depth_unit__factor'];
            $length = $f > 0 ? ($l / $f) : $l;
            $unit = $lang->get($cable_row['isys_depth_unit__title']);
            $connector['cable_length'] = $length . $unit;
        }
        return $connector;
    }

    /**
     * Get information of connected object.
     *
     * @param  array $row
     * @param  array $l_conn_row
     * @return array
     * @throws Exception
     */
    private function _get_connected_object_information($row, $l_conn_row)
    {
        // !!!
        //
        // WICHTIG: Wenn das Objekt nicht in einem Schrank eingebaut ist, wird es als "Dose" behandelt
        //
        // !!!

        // check if device is in a rack
        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $location_dao = isys_cmdb_dao_category_g_location::instance($this->m_db);
        $is_socket = true;
        $chassis_title = null;
        $slot_title = null;
        $location_row = $location_dao->get_data(null, $l_conn_row['isys_obj__id'])->get_row();
        if ($location_row) {
            // check parent object type
            $parent_id = $location_row['isys_catg_location_list__parentid'];
            $item = $global_dao->get_object($parent_id)->get_row();

            if ($this->_is_chassis($item)) {
                // check if chassis is in a rack
                $location_row = $location_dao->get_data(null, $item['isys_obj__id'])->get_row();
                if ($location_row) {
                    $chassis_title = $item['isys_obj__title'];
                    $slot_title = $this->_get_slot_title_by_module($item['isys_obj__id'], $l_conn_row['isys_obj__id']);

                    // check parent object type
                    $parent_id = $location_row['isys_catg_location_list__parentid'];
                    $item = $global_dao->get_object($parent_id)->get_row();
                    $is_socket = $item['isys_obj_type__const'] != 'C__OBJTYPE__ENCLOSURE';
                }
            } else {
                $is_socket = $item['isys_obj_type__const'] != 'C__OBJTYPE__ENCLOSURE';
            }
        }

        $end_device_name = null;
        $planned_end_device_name = null;
        if ($is_socket) {
            $end_device_dao = new isys_cmdb_dao_category_g_smview_patch_end_device($this->m_db);
            $end_device = $end_device_dao->get_data(null, $l_conn_row['isys_obj__id'])->get_row();
            $end_device_name = $end_device ? $end_device['isys_catg_smview_patch_end_device_list__name'] : null;
            $planned_end_device_name = $end_device ? $end_device['isys_catg_smview_patch_end_device_list__planned_name'] : null;
        }

        // get connected devices from feed
        $feed_hosts = $this->_get_feed_hosts($row);

        $lang = isys_application::instance()->container->get('language');
        return [
            'id'                             => $row['con_connector'],
            'title'                          => $row['connector_name'],
            'obj_id'                         => $l_conn_row['isys_obj__id'],
            'obj_title'                      => $l_conn_row['isys_obj__title'],
            'obj_type_title'                 => $lang->get($l_conn_row['isys_obj_type__title']),
            'obj_url'                        => isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT => $l_conn_row['isys_obj__id'],
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__CABLING')]),
            'is_socket'                      => $is_socket, // Behandlung des Objektes als "Dose"
            'manual_end_device_name'         => $end_device_name,
            'planned_manual_end_device_name' => $planned_end_device_name,
            'feed_hosts'                     => $feed_hosts,
            'chassis_title'                  => $chassis_title,
            'slot_title'                     => $slot_title,
        ];
    }

    /**
     * Get slot title by module and its chassis.
     *
     * @param  int $chassis_obj_id
     * @param  int $module_obj_id
     * @return string|null
     * @throws isys_exception_database
     */
    private function _get_slot_title_by_module($chassis_obj_id, $module_obj_id)
    {
        $chassis_dao = isys_cmdb_dao_category_s_chassis::instance($this->m_db);
        $slot_dao = isys_cmdb_dao_category_s_chassis_slot::instance($this->m_db);
        $res = $slot_dao->get_data(null, $chassis_obj_id, '', null, defined_or_default('C__RECORD_STATUS__NORMAL'));
        while ($slot_row = $res->get_row()) {
            // get assigned devices of this slot
            $l_assigned_slots = array_keys($slot_dao->get_assigned_chassis_items_by_cat_id($slot_row['isys_cats_chassis_slot_list__id']));
            $l_chassis_res = $chassis_dao->get_data(null, $chassis_obj_id, '', null, defined_or_default('C__RECORD_STATUS__NORMAL'));
            while ($l_chassis_row = $l_chassis_res->get_row()) {
                if ($l_chassis_row['isys_connection__isys_obj__id'] &&
                    in_array($l_chassis_row['isys_cats_chassis_list__id'], $l_assigned_slots) &&
                    $l_chassis_row['isys_connection__isys_obj__id'] == $module_obj_id) {
                    return $slot_row['isys_cats_chassis_slot_list__title'];
                }
            }
        }
        return null;
    }

    /**
     * Try to get connected devices from feed if add-on is installed.
     *
     * @param  array $row
     * @return array
     */
    private function _get_feed_hosts($row)
    {
        if (! class_exists('isys_cmdb_dao_category_g_smview_feed_virtual_device')) {
            // feed add-on is not installed
            return [];
        }

        $feed_hosts = [];
        $dao = new isys_cmdb_dao_category_g_smview_feed_virtual_device($this->m_db);
        $virtual_device_data = $dao->get_data_as_array(null, $row['isys_obj__id']); // obj_id of switch
        $ports = $dao->get_virtual_device_ports_by_connector($row['isys_catg_connector_list__title'], $virtual_device_data);
        foreach ($ports as $p) {
            $feed_hosts[] = $p['isys_host_name'] . '@' . $p['isys_port'];
        }
        return $feed_hosts;
    }
    
    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     */
    public function create_data($p_data)
    {
        // no op
        return false;
    }
    
    /**
     * Updates existing entity.
     *
     * @param  integer $p_list_id Entity's identifier
     * @param  array   $p_data    Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     */
    public function save_data($p_list_id, $p_data)
    {
        // no op
        return false;
    }
}
