<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Patch Formfactor.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_patch_formfactor extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_patch_formfactor';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     */
    protected function properties()
    {
        return [
            'half_he' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                        C__PROPERTY__INFO => [
                            C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_PATCH_FORMFACTOR__HALF_HE',
                            C__PROPERTY__INFO__DESCRIPTION => 'Half HE'
                        ],
                        C__PROPERTY__DATA => [
                            C__PROPERTY__DATA__FIELD => 'isys_catg_smview_patch_formfactor_list__half_he',
                            C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                                'SELECT (CASE WHEN isys_catg_smview_patch_formfactor_list__half_he = \'1\' THEN ' .
                                $this->convert_sql_text('LC__UNIVERSAL__YES') . ' WHEN isys_catg_smview_patch_formfactor_list__half_he = \'0\' THEN ' .
                                $this->convert_sql_text('LC__UNIVERSAL__NO') . ' END) FROM isys_catg_smview_patch_formfactor_list',
                                'isys_catg_smview_patch_formfactor_list',
                                'isys_catg_smview_patch_formfactor_list__id',
                                'isys_catg_smview_patch_formfactor_list__isys_obj__id'
                            ),
                            C__PROPERTY__DATA__JOIN   => [
                                idoit\Module\Report\SqlQuery\Structure\SelectJoin::factory('isys_catg_smview_patch_formfactor_list', 'LEFT', 'isys_catg_smview_patch_formfactor_list__isys_obj__id', 'isys_obj__id')
                            ]
                        ],
                        C__PROPERTY__UI => [
                            C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_PATCH_FORMFACTOR_HALF_HE',
                            C__PROPERTY__UI__PARAMS  => [
                                'p_arData' => get_smarty_arr_YES_NO(),
                            ],
                            C__PROPERTY__UI__DEFAULT => 0
                        ]
                    ]
            ),
            'comment' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::commentary(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_PATCH__COMMENT',
                        C__PROPERTY__INFO__DESCRIPTION => 'Comment'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_patch_formfactor_list__comment'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_PATCH_COMMENT'
                    ],
                    C__PROPERTY__CHECK => [
                        // This is necessary to keep linebreaks
                        C__PROPERTY__CHECK__SANITIZATION => null
                    ]
                ]
            ),
        ];
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     * @throws isys_exception_dao
     */
    public function create_data($p_data)
    {
        if (isset($_POST['flip_connectors']) && $_POST['flip_connectors'] == '1') {
            $this->_flipConnectorTypes();
            return -1;
        }
        return parent::create_data($p_data);
    }

    /**
     * Updates existing entity.
     *
     * @param   integer $p_category_data_id Entity's identifier
     * @param   array   $p_data             Properties in a associative array with tags as keys and their corresponding values as values.
     * @return  boolean
     * @throws  isys_exception_dao
     * @throws  isys_exception_dao_cmdb
     */
    public function save_data($p_category_data_id, $p_data)
    {
        if (isset($_POST['flip_connectors']) && $_POST['flip_connectors'] == '1') {
            $this->_flipConnectorTypes();
            return true;
        }
        return parent::save_data($p_category_data_id, $p_data);
    }

    /**
     * Flip connector types.
     *
     * @throws isys_exception_dao
     */
    private function _flipConnectorTypes()
    {
        // get current in- and outputs
        $dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $sql = ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT;
        $sql .= ' AND isys_catg_connector_list.isys_catg_connector_list__assigned_category = "C__CATG__CONNECTOR"';
        $res_in = $dao->get_data(null, $this->m_object_id, $sql, null, defined_or_default('C__RECORD_STATUS__NORMAL'));
        $sql = ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__OUTPUT;
        $sql .= ' AND isys_catg_connector_list.isys_catg_connector_list__assigned_category = "C__CATG__CONNECTOR"';
        $res_out = $dao->get_data(null, $this->m_object_id, $sql, null, defined_or_default('C__RECORD_STATUS__NORMAL'));

        // flip types
        $this->_changeConnectorType($res_in, C__CONNECTOR__OUTPUT);
        $this->_changeConnectorType($res_out, C__CONNECTOR__INPUT);
    }

    /**
     * Change connector type of given result set.
     *
     * @param  isys_component_dao_result $res
     * @param  int $to
     * @throws isys_exception_dao
     */
    private function _changeConnectorType($res, $to)
    {
        $dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        while ($row = $res->get_row()) {
            $sql = "UPDATE isys_catg_connector_list SET isys_catg_connector_list__type = $to
                    WHERE isys_catg_connector_list__id = " . $row['isys_catg_connector_list__id'];
            $dao->update($sql);
        }
    }
}
