<?php

use idoit\Component\Property\Exception\UnsupportedConfigurationTypeException;
use idoit\Component\Property\Type\TextProperty;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Patch Grid.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_patch_grid extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_patch_grid';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws UnsupportedConfigurationTypeException
     */
    protected function properties()
    {
        return [
            'description' => (new TextProperty(
                'C__CATG__SMVIEW_PATCH_DEVICE_LAYOUT_DESCRIPTION',
                'LC__CMDB__CATG__SMVIEW_PATCH__DESCRIPTION',
                'isys_catg_smview_patch_grid_list__description',
                'isys_catg_smview_patch_grid_list'
            )),
            'grid_template_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                C__PROPERTY__INFO => [
                    C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_PATCH__GRID_TEMPLATE_ID',
                    C__PROPERTY__INFO__DESCRIPTION => 'Description'
                ],
                C__PROPERTY__DATA => [
                    C__PROPERTY__DATA__FIELD => 'isys_catg_smview_patch_grid_list__grid_template_id'
                ],
                C__PROPERTY__UI => [
                    C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_PATCH_DEVICE_LAYOUT_GRID_TEMPLATE_ID',
                ],
                C__PROPERTY__PROVIDES => [
                    C__PROPERTY__PROVIDES__SEARCH     => false,
                    C__PROPERTY__PROVIDES__LIST       => false,
                    C__PROPERTY__PROVIDES__REPORT     => false,
                    C__PROPERTY__PROVIDES__VALIDATION => false,
                    C__PROPERTY__PROVIDES__VIRTUAL    => true,
                ]
                    ]
            ),
        ];
    }
    
    /**
     * Retrieve custom properties.
     *
     * @return array
     */
    public function custom_properties()
    {
        return [
            'grid' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_PATCH_GRID__GRID',
                        C__PROPERTY__INFO__DESCRIPTION => 'Grid'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_patch__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_PATCH_GRID',
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
        ];
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     * @throws isys_exception_database
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     */
    public function create_data($p_data)
    {
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        $template = $grid_template_dao->get_object('id', $p_data['grid_template_id']);
        if (! $template['is_template']) {
            // create a copy if the origin grid is not a template
            $p_data['grid_template_id'] =  $this->_copyConnectorTemplate($template, $p_data['isys_obj__id']);
        }

        // add list entry
        $list_id = $this->_assign_grid_template_to_catg($p_data['grid_template_id'], $p_data);
        
        // get current connectors
        $available_connectors = $this->get_available_connectors($p_data['isys_obj__id']);
        
        // get all connector templates
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template($this->m_db);
        $connector_templates = $connector_template_dao->filter_objects(
            array('grid_template_id' => $p_data['grid_template_id'])
        );
        
        // merge with connector templates
        $this->_merge_and_assign_connectors($available_connectors, $connector_templates, $list_id);
        return $list_id;
    }

    /**
     * Copy connector template.
     *
     * @param  array $template
     * @param  int $obj_id
     * @return false|int
     * @throws isys_exception_database
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     */
    private function _copyConnectorTemplate($template, $obj_id)
    {
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template($this->m_db);
        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $originId = $template['id'];

        $obj = $dao->get_object($obj_id)->get_row();
        $template['title'] = $obj['isys_obj__title'];
        unset($template['id']);

        $id = $grid_template_dao->create_object($template);

        // copy all connector templates
        $connector_templates = $connector_template_dao->filter_objects(
            array('grid_template_id' => $originId)
        );
        foreach ($connector_templates as $c) {
            unset($c['id']);
            $c['grid_template_id'] = $id;
            $connector_template_dao->create_object($c);
        }
        return $id;
    }

    /**
     * Get available connectors of this object.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_available_connectors($obj_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $picture_dao = new isys_cmdb_dao_smview_patch_picture($this->m_db);
        $l_data = $connector_dao->get_data(
            null,
            $obj_id,
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__OUTPUT,
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL')
        );
        
        $connectors = [];
        while ($row = $l_data->get_row()) {
            $connector_type_id = $row['isys_catg_connector_list__isys_connection_type__id'];
            $pic = $picture_dao->get_object('id', $connector_type_id);
            $id = $row['isys_catg_connector_list__id'];
            $connector = [
                'connector_id'         => $id,
                'connector_title'      => $row['isys_catg_connector_list__title'],
                'connector_type'       => $row['isys_catg_connector_list__type'],
                'connector_type_id'    => $connector_type_id,
                'connector_type_title' => ! empty($pic) ? $pic['title'] : null,
                'url'                  => $picture_dao->get_file_url($pic['obj_id']),
            ];
            $connectors[$id] = $connector;
        }
        return $connectors;
    }

    /**
     * Merge and assign connectors with connector templates.
     *
     * @param  array $available_connectors
     * @param  array $connector_templates
     * @param  int $list_id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    private function _merge_and_assign_connectors($available_connectors, $connector_templates, $list_id)
    {
        $grid_connector_dao = new isys_cmdb_dao_smview_patch_grid_connector($this->m_db);
        foreach ($connector_templates as $tpl) {
            foreach ($available_connectors as $c) {
                if ($c['connector_type_id'] != $tpl['connector_type_id']) {
                    continue;
                }
                if ($c['connector_type'] != $tpl['connector_type']) {
                    continue;
                }
                if ($c['connector_title'] != $tpl['connector_title']) {
                    continue;
                }
                $params = [
                    'grid_list_id'          => $list_id,
                    'connector_template_id' => $tpl['id'],
                    'connector_id'          => $c['connector_id'],
                ];
                $grid_connector_dao->create_object($params);
            }
        }
    }

    /**
     * Updates existing entity.
     *
     * @param  integer $p_list_id Entity's identifier
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function save_data($p_list_id, $p_data)
    {
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        $layout = json_decode($p_data['grid'], true);
        $grid_data = $layout['frame'];
        $grid_template_id = $grid_data['id'];
        
        $gen_data = $this->get_general_data();
        $is_new = empty($gen_data['isys_catg_smview_patch_grid_list__grid_template_id']);
        $create_new = ! empty($grid_data['create_copy']) ? true : $is_new;
        if ($create_new && ! $grid_data['id']) {
            // new grid, which does not use a template
            $grid_template_id = $grid_template_dao->create_object($grid_data);
        }
        $grid = $grid_template_dao->get_object('id', $grid_template_id);
        
        // update assignment to catg
        $this->_assign_grid_template_to_catg($grid_template_id, $p_data, $p_list_id);
        
        if ($grid_data['id']) {
            // grid uses a template
            if (! $grid['is_template'] || ($grid_data['is_template'] && $grid_data['modify_template'])) {
                // grid is not a template or grid is a template, which should be updated
                $this->_update_grid($grid_template_id, $grid_data);
                $assigned_ports = $this->_save_connector_templates($grid_template_id, $layout['ports']);
            } else {
                // only get posted connectors, to assign it to used template
                $assigned_ports = $this->_get_posted_connectors($layout['ports']);
            }
        } else {
            // grid uses no template
            $assigned_ports = $this->_save_connector_templates($grid_template_id, $layout['ports']);
        }
        
        // save connector assignments
        $this->_assign_connector_instances_to_template($p_list_id, $assigned_ports);
        
        $this->delete_unused_grids();
        return true;
    }

    /**
     * Assign grid template to grid list.
     *
     * @param  int $grid_template_id
     * @param  array $p_data
     * @param  int $list_id
     * @return boolean
     * @throws isys_exception_dao
     */
    private function _assign_grid_template_to_catg($grid_template_id, $p_data, $list_id = null)
    {
        $l_data = array(
            'grid_template_id' => $grid_template_id,
            'description' => $p_data['description'],
            'status'      => 2,
        );

        if (isset($p_data['isys_obj__id'])) {
            $l_data['isys_obj__id'] = $p_data['isys_obj__id'];
        }
        if (isset($list_id)) {
            $l_data['id'] = $list_id;
        }
        
        $l_prepared_query = $this->prepare_query($l_data);
        if (isset($list_id)) {
            $l_query = "UPDATE " . $this->m_table . " SET " . $l_prepared_query . " WHERE " . $this->m_table . "__id = " . $this->convert_sql_id($list_id) . ";";
        } else {
            $l_query = 'INSERT INTO ' . $this->m_table . ' SET ' . $l_prepared_query . ';';
        }
        
        if ($this->update($l_query) && $this->apply_update()) {
            return isset($list_id) ? $list_id : intval($this->get_last_insert_id());
        }
        return false;
    }

    /**
     * Update grid data.
     *
     * @param  int $grid_template_id
     * @param  array $grid_data
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    private function _update_grid($grid_template_id, $grid_data)
    {
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        $grid_template = $grid_template_dao->get_object('id', $grid_template_id);
        $is_template = $grid_template['is_template'];
        $grid = [
            'title'             => $grid_data['title'],
            'column_count'      => $grid_data['column_count'],
            'row_count'         => $grid_data['row_count'],
            'column_count_back' => $grid_data['column_count_back'] ?? null,
            'row_count_back'    => $grid_data['row_count_back'] ?? null,
            'is_template'       => $is_template == 1 ?: $grid_data['is_template'], // always keep templates, if they are one
        ];
        $grid_template_dao->update_object($grid_template_id, null, $grid);
    }
    
    /**
     * Get posted assignments between connector template and connector instance.
     *
     * @param  array $connectors
     * @return array
     */
    private function _get_posted_connectors($connectors)
    {
        $assigned_connectors = [];
        foreach ($connectors as $c) {
            if (empty($c['connector_id'])) {
                continue;
            }
            
            $assigned_connectors[$c['id']] = [
                'connector_template_id' => $c['id'],
                'connector_id'          => $c['connector_id'],
            ];
        }
        return $assigned_connectors;
    }

    /**
     * Save posted connector templates.
     *
     * @param  int $grid_template_id
     * @param  array $connectors
     * @return array
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    private function _save_connector_templates($grid_template_id, $connectors)
    {
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template($this->m_db);

        // get current connectors
        $assigned_connectors_before = $connector_template_dao->filter_objects(array('grid_template_id' => $grid_template_id));
        
        // sort posted connector templates
        $connectors_by_id = [];
        $connectors_by_coords = [];
        foreach ($connectors as $c) {
            if (empty($c['id'])) {
                continue;
            }
            $connectors_by_id[$c['id']] = $c['id'];
            
            $coord = $c['angle'] . '_' . $c['pos_y'] . '_' . $c['pos_x'];
            if (isset($connectors_by_coords[$coord])) {
                throw new isys_exception_validation('there is already a connector assigned to this position', []);
            }
            $connectors_by_coords[$coord] = $c;
        }
        
        // first remove obsolete connector templates
        foreach ($assigned_connectors_before as $connector) {
            if (! isset($connectors_by_id[$connector['id']])) {
                $connector_template_dao->delete_object($connector['id']);
            }
        }
        
        // create new connectors or update existing ones
        $assigned_connectors_after = [];
        foreach ($connectors as $c) {
            // unable to save empty elements or elements without connector ID
            if (empty($c) || empty($c['connector_id'])) {
                continue;
            }
            
            if (key_exists('merged', $c) && ! $c['merged']) {
                // this connector was not merged successfully from a template
                continue;
            }
            
            $connector_id = $this->_save_connector_template($grid_template_id, $c);
            $assigned_connectors_after[$connector_id] = [
                'connector_template_id' => $connector_id,
                'connector_id'          => $c['connector_id'],
            ];
        }
        return $assigned_connectors_after;
    }

    /**
     * Save a posted connector template.
     *
     * @param  int $grid_template_id
     * @param  array $conn
     * @return int
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    private function _save_connector_template($grid_template_id, $conn)
    {
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template($this->m_db);
        /*
         * Validation of connector template position will be disabled, to
         * support a switch of the positions of two connector templates.
         *
         * Validation of positions will be handled in code below.
         */
        $connector_template_dao->disable_position_validation();
        $connector_instance = $this->_get_connector_instance($conn['connector_id']);
        if (! empty($conn['id'])) {
            // this is an configured connector template
            $ct_id = $conn['id'];
            $port = [
                'connector_title'   => $connector_instance['connector_title'],
                'connector_type'    => $connector_instance['connector_type'],
                'connector_type_id' => $connector_instance['connector_type_id'],
                'title'             => $conn['title'],
                'number'            => $conn['number'],
                'pos_x'             => $conn['pos_x'],
                'pos_y'             => $conn['pos_y'],
                'rotation'          => $conn['rotation'],
                'label_orientation' => $conn['label_orientation'],
                'status'            => $conn['status'],
            ];
            $connector_template_dao->update_object($ct_id, null, $port);
        } else {
            // this is a new connector template
            $port = [
                'grid_template_id'  => $grid_template_id,
                'connector_title'   => $connector_instance['connector_title'],
                'connector_type'    => $connector_instance['connector_type'],
                'connector_type_id' => $connector_instance['connector_type_id'],
                'title'             => $conn['title'],
                'number'            => $conn['number'],
                'pos_x'             => $conn['pos_x'],
                'pos_y'             => $conn['pos_y'],
                'rotation'          => $conn['rotation'],
                'label_orientation' => $conn['label_orientation'],
                'status'            => $conn['status'],
                'angle'             => $conn['angle'],
            ];
            $ct_id = $connector_template_dao->create_object($port);
        }
        return $ct_id;
    }
    
    /**
     * Get connector instance by ID.
     *
     * @param  int $connector_id
     * @return array
     */
    private function _get_connector_instance($connector_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $row = $connector_dao->get_data($connector_id)->get_row();
        return [
            'connector_title'   => $row['isys_catg_connector_list__title'],
            'connector_type'    => $row['isys_catg_connector_list__type'],
            'connector_type_id' => $row['isys_catg_connector_list__isys_connection_type__id'],
        ];
    }

    /**
     * Assign connector instances to connector templates.
     *
     * @param  int $list_id
     * @param  array $assignments
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    private function _assign_connector_instances_to_template($list_id, $assignments)
    {
        $grid_connector_dao = new isys_cmdb_dao_smview_patch_grid_connector($this->m_db);
        $current_items = $grid_connector_dao->filter_objects(array('grid_list_id' => $list_id));
        
        // create new assignments
        $new_items = [];
        foreach ($assignments as $a) {
            $item_id = null;
            
            // try to find existing assignment
            foreach ($current_items as $item) {
                if ($a['connector_template_id'] == $item['connector_template_id'] && $a['connector_id'] == $item['connector_id']) {
                    $item_id = $item['id'];
                    break;
                }
            }
            
            if (! $item_id) {
                // create assignment
                $item_id = $grid_connector_dao->create_object(
                    [
                    'grid_list_id'          => $list_id,
                    'connector_template_id' => $a['connector_template_id'],
                    'connector_id'          => $a['connector_id'],
                    ]
                );
            }
            $new_items[] = $item_id;
        }
        
        // remove obsolete assignments
        foreach ($current_items as $item) {
            if (! in_array($item['id'], $new_items)) {
                $grid_connector_dao->delete_object($item['id']);
            }
        }
    }

    /**
     * Get objects using this grid template.
     *
     * @param  int $id
     * @return array
     * @throws Exception
     */
    public function get_objects_using_grid_template($id)
    {
        $lang = isys_application::instance()->container->get('language');
        $list_items = $this->get_data(
            null,
            null,
            " AND isys_catg_smview_patch_grid_list__grid_template_id=$id ",
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL'));
        $using_objects = [];
        while ($row = $list_items->get_row()) {
            $using_objects[] = [
                'id'         => $row['isys_obj__id'],
                'title'      => $row['isys_obj__title'],
                'type_title' => $lang->get($row['isys_obj_type__title']),
                'url'        => isys_helper_link::create_url(
                    [
                    C__CMDB__GET__OBJECT => $row['isys_obj__id'],
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__SMVIEW_PATCH_GRID')]
                ),
            ];
        }
        return $using_objects;
    }

    /**
     * Delete unused grids.
     *
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function delete_unused_grids()
    {
        $list_items = $this->get_data();
        $used_grid_templates = [];
        while ($row = $list_items->get_row()) {
            $id = $row['isys_catg_smview_patch_grid_list__grid_template_id'];
            $used_grid_templates[$id] = $id;
        }
        
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        $templates = $grid_template_dao->get_objects();
        foreach ($templates as $tpl) {
            $id = $tpl['id'];
            if (! empty($tpl['is_template'])) {
                // do not delete templates
                continue;
            }
            
            if (isset($used_grid_templates[$id])) {
                // grid is in use
                continue;
            }
            
            // remove this unused grid
            $grid_template_dao->delete_object($id);
        }
    }
}
