<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Patch Parking Deck.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_patch_parking_deck extends isys_cmdb_dao_category_g_virtual
{
    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var  string
     */
    protected $m_category = 'smview_patch_parking_deck';

    /**
     * @var integer
     */
    protected $m_list_id = 'isys_catg_smview_patch_parking_deck_info_list__id';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * @var  string
     */
    protected $m_source_table = 'isys_catg_smview_patch_parking_deck_info';

    /**
     * Main table where properties are stored persistently.
     *
     * @var  string
     */
    protected $m_table = 'isys_catg_smview_patch_parking_deck_info_list';

    /**
     * Method for returning the properties.
     *
     * @return  array
     */
    protected function properties()
    {
        $parking_deck_info_dao = new isys_cmdb_dao_category_g_smview_patch_parking_deck_info($this->m_db);
        return $parking_deck_info_dao->get_properties();
    }

    /**
     * Retrieves the number of saved category-entries to the given object.
     *
     * @param  int $objectId
     * @return int
     * @throws isys_exception_database
     */
    public function get_count($objectId = null)
    {
        // return count of all parking deck objects
        $sql = 'SELECT COUNT(*) as count FROM isys_obj o 
            JOIN isys_obj_type t ON o.isys_obj__isys_obj_type__id = t.isys_obj_type__id
            WHERE t.isys_obj_type__const = ' . $this->convert_sql_text('C__SHD__PARKING_DECK_OBJECT');
        $row = $this->retrieve($sql)->get_row();
        return $row['count'];
    }

    /**
     * Fetches category data from database.
     *
     * @param  integer $p_category_data_id
     * @param  integer $p_obj_id
     * @param  string $p_condition
     * @param  mixed $p_filter
     * @param  integer $p_status
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_data($p_category_data_id = null, $p_obj_id = null, $p_condition = '', $p_filter = null, $p_status = null)
    {
        // return all parking deck objects
        $sql = 'SELECT * FROM isys_obj o 
            JOIN isys_obj_type t ON o.isys_obj__isys_obj_type__id = t.isys_obj_type__id
            LEFT JOIN isys_catg_smview_patch_parking_deck_info_list l ON l.isys_catg_smview_patch_parking_deck_info_list__isys_obj__id = o.isys_obj__id
            WHERE t.isys_obj_type__const = ' . $this->convert_sql_text('C__SHD__PARKING_DECK_OBJECT');
        if (! empty($p_category_data_id)) {
            $sql .= ' AND l.isys_catg_smview_patch_parking_deck_info_list__id = ' . $this->convert_sql_id($p_category_data_id);
        }
        return $this->retrieve($sql);
    }

    /**
     * Get merged connectors.
     *
     * @param  array $connectors
     * @param  boolean $onlyConnectedConnectors
     * @return array
     * @throws Exception
     */
    public function get_connectors($connectors, $onlyConnectedConnectors = false)
    {
        $mergedConnectors = [];
        foreach ($connectors as $lc) {
            if ($onlyConnectedConnectors && empty($lc['con_connector'])) {
                continue;
            }
            $mergedConnectors[] = $this->_get_connector_information($lc);
        }
        return $mergedConnectors;
    }

    /**
     * Get connector information.
     *
     * @param  array $connector
     * @return array
     * @throws Exception
     */
    private function _get_connector_information($connector)
    {
        if (empty($connector)) {
            return [
                'connector_id'    => null,
                'connector_title' => null,
                'connector_type'  => null,
                'connection'      => [],
            ];
        }
        $connectedObject = [];
        if (! empty($connector['con_connector'])) {
            $dao = new isys_cmdb_dao_category_g_connector($this->m_db);
            $connectedConnector = $dao->get_data($connector['con_connector'])->get_row();

            $lang = isys_application::instance()->container->get('language');
            $obj_type = $lang->get($connectedConnector['isys_obj_type__title']);
            $connectedObject = [
                'object_type_title' => $obj_type,
                'object_title'      => $connectedConnector['isys_obj__title'],
                'connector_id'      => $connectedConnector['isys_catg_connector_list__id'],
                'connector_title'   => $connectedConnector['isys_catg_connector_list__title'],
                'connector_url'     => isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT   => $connectedConnector['isys_obj__id'],
                    C__CMDB__GET__CATG     => defined_or_default('C__CATG__CONNECTOR'),
                    C__CMDB__GET__CATLEVEL => $connectedConnector['isys_catg_connector_list__id'],
                ]),
            ];
        }
        return [
            'connector_id'    => $connector['isys_catg_connector_list__id'],
            'connector_title' => $connector['isys_catg_connector_list__title'],
            'connector_type'  => $connector['isys_catg_connector_list__type'],
            'connection'      => $connectedObject,
        ];
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     */
    public function create_data($p_data)
    {
        // no op
        return false;
    }

    /**
     * Updates existing entity.
     *
     * @param  integer $p_list_id Entity's identifier
     * @param  array   $p_data    Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     */
    public function save_data($p_list_id, $p_data)
    {
        // no op
        return false;
    }

    /**
     * Move connection from local connector to (new) parking deck object.
     *
     * @param  int $connector_id
     * @param  int|null $parking_deck_obj_id
     * @param  string|null $title
     * @return array
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function park_connector($connector_id, $parking_deck_obj_id = null, $title = null)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $parking_deck_info_dao = new isys_cmdb_dao_category_g_smview_patch_parking_deck_info($this->m_db);

        // without title use obj_title of original object
        $local_connector = $connector_dao->get_data($connector_id)->get_row();
        $title = empty($title) ? $local_connector['isys_obj__title'] : $title;
        if (empty($parking_deck_obj_id)) {
            // create a new parking deck object
            $type = defined_or_default('C__SHD__PARKING_DECK_OBJECT');
            $parking_deck_obj_id = $connector_dao->create_object($title, $type);

            // add parking deck information
            $parking_deck_info_dao->create_data([
                'isys_obj__id'   => $parking_deck_obj_id,
                'creation_date'  => date('Y-m-d H:i:s'),
                'obj_title'      => $local_connector['isys_obj__title'],
                'obj_type_const' => $local_connector['isys_obj_type__const'],
            ]);
        }

        $parked_connectors = $connector_dao->get_data_as_array(null, $parking_deck_obj_id);
        $parked_connector = null;
        $local_key = $local_connector['isys_connection_type__id'] . '_' . $local_connector['isys_catg_connector_list__type'] . '_' . $local_connector['isys_catg_connector_list__title'];
        foreach ($parked_connectors as $pc) {
            $parked_key = $pc['isys_connection_type__id'] . '_' . $pc['isys_catg_connector_list__type'] . '_' . $pc['isys_catg_connector_list__title'];
            if ($local_key == $parked_key) {
                $parked_connector = $pc;
                break;
            }
        }

        if (! $parked_connector) {
            // create new connector on parking deck object
            $id = $connector_dao->create($parking_deck_obj_id, $local_connector['isys_catg_connector_list__type'], null, $local_connector['isys_connection_type__id'], $local_connector['isys_catg_connector_list__title']);
            $parked_connector = $connector_dao->get_data($id)->get_row();
        }

        // assign connection from original object to parking deck object
        $this->_update_connection($local_connector['con_connector'], $parked_connector['isys_catg_connector_list__id'], $local_connector['cable_id']);

        // return parking deck object with all connectors
        $local_connectors = $connector_dao->get_data_as_array(null, $local_connector['isys_obj__id']);
        $parked_connectors = $connector_dao->get_data_as_array(null, $parking_deck_obj_id);
        return [
            'obj_id'            => $parking_deck_obj_id,
            'obj_title'         => $title,
            'local_connectors'  => $this->get_connectors($local_connectors),
            'parked_connectors' => $this->get_connectors($parked_connectors),
        ];
    }

    /**
     * Move connection from parked connector to local connector.
     *
     * @param  int $parked_connector_id
     * @param  int $local_connector_id
     * @param  string $mode
     * @return array
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function connect_connector($parked_connector_id, $local_connector_id, $mode = 'assign')
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $local_connector = $connector_dao->get_data($local_connector_id)->get_row();
        $parked_connector = $connector_dao->get_data($parked_connector_id)->get_row();
        if ($mode != 'overwrite' && $local_connector['con_connector']) {
            // error, local connector is already in use
            return [];
        }

        // assign connection from parking deck object to original object
        $success = $this->_update_connection($parked_connector['con_connector'], $local_connector['isys_catg_connector_list__id'], $parked_connector['cable_id']);
        if ($success) {
            // remove connector from parking deck object
            $connector_dao->delete_entry($parked_connector['isys_catg_connector_list__id'], $connector_dao->get_table());
        }

        $parked_connectors = $connector_dao->get_data_as_array(null, $parked_connector['isys_obj__id']);
        if (count($parked_connectors) == 0) {
            // delete parking deck object
            $connector_dao->delete_object($parked_connector['isys_obj__id']);
        }

        // return parking deck object with all connectors
        $local_connectors = $connector_dao->get_data_as_array(null, $local_connector['isys_obj__id']);
        return [
            'obj_id'            => $parked_connector['isys_obj__id'],
            'obj_title'         => $parked_connector['isys_obj__title'],
            'local_connectors'  => $this->get_connectors($local_connectors),
            'parked_connectors' => $this->get_connectors($parked_connectors),
        ];
    }

    /**
     * Get connected cable pins.
     *
     * @param  int $connected_connector_id
     * @param  int $new_connector_id
     * @param  int $p_cableID
     * @return boolean
     * @throws isys_exception_database
     * @throws isys_exception_dao
     * @throws isys_exception_cmdb
     */
    private function _update_connection($connected_connector_id, $new_connector_id, $p_cableID = null)
    {
        // detach cable from connected connector of local connector
        $l_dao = new isys_cmdb_dao_cable_connection($this->m_db);
        $l_cable_connection_id = $l_dao->handle_cable_connection_detachment(
            $l_dao->get_cable_connection_id_by_connector_id($connected_connector_id),
            $connected_connector_id,
            null,
            $p_cableID
        );

        // attach parked connector to connected connector of former local connector
        return $this->handle_cable_connection_attachment(
            $connected_connector_id,
            $new_connector_id,
            $p_cableID,
            $l_cable_connection_id
        );
    }

    /**
     * This function handles the cable connection.
     *
     * COPIED FROM src/classes/modules/cmdb/dao/isys_cmdb_dao_cable_connection.class.php L339
     *
     * @param int    $p_current_connectorID
     * @param int|null    $p_connected_connectorID
     * @param int|null    $p_cableID
     * @param int|null    $p_cable_connection_id
     * @param string $p_cableName
     * @param        $p_is_master_obj
     *
     * @return bool
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @author   Van Quyen Hoang <qhoang@i-doit.com>
     */
    private function handle_cable_connection_attachment($p_current_connectorID, $p_connected_connectorID = null, $p_cableID = null, $p_cable_connection_id = null, $p_cableName = '', $p_is_master_obj = false)
    {
        $l_dao = new isys_cmdb_dao_cable_connection($this->m_db);
        $l_nRetCode = true;

        if ($p_connected_connectorID !== null && $p_connected_connectorID > 0) {
            // Add cable if no cable has been selected
            if (!$p_cableID) {
                $p_cableID = isys_cmdb_dao_cable_connection::recycle_cable($p_cableName);
            }

            if ($p_cable_connection_id !== null) {
                // check if cable connection ID really exists
                $row = $l_dao->retrieve("SELECT * FROM isys_cable_connection WHERE isys_cable_connection__id = $p_cable_connection_id;")->get_row();
                if (empty($row)) {
                    $p_cable_connection_id = null;
                }
            }

            $l_cable_connection_id = ($p_cable_connection_id === null) ? $l_dao->add_cable_connection($p_cableID) : $p_cable_connection_id;

            if (!!$p_is_master_obj) {
                $l_master_connector = $p_current_connectorID;
            } else {
                $l_master_connector = $p_connected_connectorID;
            }

            if ($l_master_connector && $p_current_connectorID && $p_connected_connectorID) {
                $l_cable_connection_id = ($l_cable_connection_id === null) ? $l_dao->add_cable_connection($p_cableID) : $l_cable_connection_id;
                $l_nRetCode = $l_dao->save_connection($p_current_connectorID, $p_connected_connectorID, $l_cable_connection_id, $l_master_connector);
            }
        } elseif ($p_current_connectorID > 0 && (int)$p_cable_connection_id > 0) {
            // ID-5885 Update used cable object in connection
            $l_dao->update_cable_connection_connector($p_current_connectorID, $p_cable_connection_id);
        }

        return $l_nRetCode;
    }
}
