<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Patch Formfactor.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_patch_pin_mapper extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_patch_pin_mapper';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * Method for returning the properties.
     *
     * @return array
     */
    protected function properties()
    {
        return [];
    }

    /**
     * Retrieves the number of saved category-entries to the given object.
     *
     * @param  int $objectId
     * @return int
     */
    public function get_count($objectId = null)
    {
        if (! class_exists('isys_cmdb_dao_smview_cable_pin')) {
            // SM-VIEW cable is not installed
            return 0;
        }

        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        return $connector_dao->get_data(
            null,
            $objectId,
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT,
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL')
        )->count();
    }

    /**
     * Get incoming connectors if an object.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_incoming_connectors($obj_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $picture_dao = new isys_cmdb_dao_smview_patch_picture($this->m_db);
        $l_data = $connector_dao->get_data(
            null,
            $obj_id,
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT,
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL')
        );

        $connectors = [];
        while ($row = $l_data->get_row()) {
            $connector_type_id = $row['isys_catg_connector_list__isys_connection_type__id'];
            $pic = $picture_dao->get_object('id', $connector_type_id);
            $id = $row['isys_catg_connector_list__id'];
            $connector = [
                'connector_id'         => $id,
                'connector_title'      => $row['isys_catg_connector_list__title'],
                'connector_type'       => $row['isys_catg_connector_list__type'],
                'connector_type_id'    => $connector_type_id,
                'connector_type_title' => ! empty($pic) ? $pic['title'] : null,
                'obj_title'            => $row['isys_obj__title'],
                'url'                  => $picture_dao->get_file_url($pic['obj_id']),
            ];
            $connectors[] = $connector;
        }
        return $connectors;
    }
}
