<?php

/**
 * i-doit
 *
 * Grid Connector DAO for SM-VIEW Patch.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_cmdb_dao_smview_patch_grid_connector extends isys_cmdb_dao_smview_patch_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_patch_grid_connector__id'                    => 'id',
        'isys_catg_smview_patch_grid_connector__grid_list_id'          => 'grid_list_id',
        'isys_catg_smview_patch_grid_connector__connector_template_id' => 'connector_template_id',
        'isys_catg_smview_patch_grid_connector__connector_id'          => 'connector_id',
        'isys_catg_smview_patch_grid_connector__connector_status'      => 'connector_status',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_patch_grid_connector';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_patch_grid_connector__id';

    /**
     * Create a new object.
     *
     * @param  array $params
     * @param  array $optionalParams
     * @return int|false
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function create_object($params, $optionalParams = [])
    {
        $this->_validate($params);
        return parent::create_object($params, $optionalParams);
    }

    /**
     * Update an existing object.
     *
     * @param  int $id
     * @param  array $params
     * @param  array $optionalParams
     * @return boolean
     * @throws isys_exception_validation
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function update_object($id, $params, $optionalParams = [])
    {
        $this->_validate($optionalParams, $id);
        return parent::update_object($id, $params, $optionalParams);
    }

    /**
     * Validate request.
     *
     * @param  array $params
     * @param  int $id
     * @throws isys_exception_validation
     * @throws isys_exception_database
     */
    private function _validate($params, $id = null)
    {
        $objs = $this->filter_objects(
            array(
            'connector_template_id' => $params['connector_template_id'],
            'connector_id'          => $params['connector_id'],
            )
        );
        
        if (empty($objs)) {
            return;
        }
        
        if (empty($id)) {
            throw new isys_exception_validation('this assignment already exists', []);
        }
        
        foreach ($objs as $obj) {
            if ($obj['id'] != $id) {
                throw new isys_exception_validation('this assignment already exists', []);
            }
        }
    }
}
