<?php

/**
 * i-doit
 *
 * Grid Template DAO for SM-VIEW Patch.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_cmdb_dao_smview_patch_grid_template extends isys_cmdb_dao_smview_patch_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_patch_grid_template__id'                => 'id',
        'isys_catg_smview_patch_grid_template__title'             => 'title',
        'isys_catg_smview_patch_grid_template__column_count'      => 'column_count',
        'isys_catg_smview_patch_grid_template__row_count'         => 'row_count',
        'isys_catg_smview_patch_grid_template__column_count_back' => 'column_count_back',
        'isys_catg_smview_patch_grid_template__row_count_back'    => 'row_count_back',
        'isys_catg_smview_patch_grid_template__version'           => 'version',
        'isys_catg_smview_patch_grid_template__is_template'       => 'is_template',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_patch_grid_template';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_patch_grid_template__id';

    /**
     * Get available icon sizes.
     *
     * @return array
     * @throws Exception
     */
    public static function get_available_sizes()
    {
        $lang = isys_application::instance()->container->get('language');
        return array(
            'xs' => ['label' => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__SIZE_XS'), 'cssClass' => 'layout-xs'],
            's'  => ['label' => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__SIZE_SM'), 'cssClass' => 'layout-sm'],
            'm'  => ['label' => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__SIZE_MD'), 'cssClass' => ''],
            'l'  => ['label' => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__SIZE_LG'), 'cssClass' => 'layout-lg'],
            'xl' => ['label' => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__SIZE_XL'), 'cssClass' => 'layout-xl'],
        );
    }

    /**
     * Create a new object.
     *
     * @param  array $params
     * @param  array $optionalParams
     * @return int|false
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     */
    public function create_object($params, $optionalParams = [])
    {
        if (! array_key_exists('column_count', $params) || ! array_key_exists('row_count', $params)) {
            throw new isys_exception_validation('column_count and row_count are required', []);
        }
        
        $column_count = (int) $params['column_count'];
        $row_count = (int) $params['row_count'];
        if ($column_count < 1) {
            throw new isys_exception_validation('column count has to be greater than 0', []);
        }
        if ($row_count < 1) {
            throw new isys_exception_validation('row count has to be greater than 0', []);
        }
        return parent::create_object($params, $optionalParams);
    }

    /**
     * Update an existing object.
     *
     * @param  int $id
     * @param  array $params
     * @param  array $optionalParams
     * @return boolean
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     */
    public function update_object($id, $params, $optionalParams = [])
    {
        $validateParams = ['column_count', 'row_count'];
        foreach ($validateParams as $validateParam) {
            if (array_key_exists($validateParam, $optionalParams)) {
                $column_count = (int) $optionalParams[$validateParam];
                if ($column_count < 1) {
                    throw new isys_exception_validation($validateParam . ' has to be greater than 0', []);
                }
            }
        }
        return parent::update_object($id, $params, $optionalParams);
    }
}
