<?php

/**
 * i-doit
 *
 * Picture DAO for SM-VIEW Patch.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_cmdb_dao_smview_patch_picture extends isys_cmdb_dao_smview_patch_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_patch_picture__isys_connection_type__id' => 'connection_type_id',
        'isys_catg_smview_patch_picture__isys_obj__id'             => 'obj_id', // this is the image object
    );
    
    /**
     * Select field mapping.
     *
     * Used to create select-fields like "$key AS $value,".
     *
     * @var array
     */
    protected $select_field_mapping = array(
        't.isys_connection_type__id'                     => 'id',
        't.isys_connection_type__title'                  => 'title',
        't.isys_connection_type__const'                  => 'const',
        'p.isys_catg_smview_patch_picture__isys_obj__id' => 'obj_id',
        'v.isys_file_version__revision'                  => 'revision',
        'physical.isys_file_physical__filename'          => 'filename',
        'physical.isys_file_physical__filename_original' => 'filename_original',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_patch_picture';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_patch_picture__isys_connection_type__id';
    
    /**
     * Get base query.
     *
     * @return string
     */
    protected function _get_base_query()
    {
        $query = "SELECT ";
        foreach ($this->select_field_mapping as $db => $name) {
            $query .= " $db AS $name,";
        }
        $base_query = trim($query, ',');
        $base_query .= " FROM isys_connection_type as t
                LEFT JOIN isys_catg_smview_patch_picture p ON t.isys_connection_type__id = p.isys_catg_smview_patch_picture__isys_connection_type__id
                LEFT JOIN isys_file_version v ON v.isys_file_version__isys_obj__id = p.isys_catg_smview_patch_picture__isys_obj__id
                LEFT JOIN isys_file_physical physical ON physical.isys_file_physical__id = isys_file_version__isys_file_physical__id
                WHERE 1=1 ";
        return $base_query;
    }

    /**
     * Get all objects.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_objects()
    {
        $query = $this->_get_base_query();
        $query .= ' ORDER BY isys_file_version__revision DESC';
        $res = $this->retrieve($query);
        
        $objects = array();
        while ($row = $res->get_row()) {
            $id = $row['id'];
            if (isset($objects[$id])) {
                continue;
            }
            $objects[$id] = $row;
        }
        return array_values($objects);
    }

    /**
     * Get an object.
     *
     * @param  string $field
     * @param  string $value
     * @return array
     * @throws isys_exception_database
     */
    public function get_object($field, $value)
    {
        $db_field = array_search($field, $this->select_field_mapping);
        $query = $this->_get_base_query();
        $query .= " AND $db_field = '$value' ";
        $query .= ' ORDER BY isys_file_version__revision DESC';
        
        $res = $this->retrieve($query);
        return $res->count() > 0 ? $res->get_row() : array();
    }
    
    /**
     * Get file URL by obj_id of the file.
     *
     * @param  int $file_obj_id
     * @return string|null
     */
    public function get_file_url($file_obj_id)
    {
        if (empty($file_obj_id)) {
            return null;
        }

        // get latest version of file
        $file_dao = new \isys_cmdb_dao_category_s_file(\isys_application::instance()->database);
        $files = $file_dao->get_file_by_obj_id($file_obj_id);
        $file = $files->get_row();

        // use CMDB module to display images directly
        return isys_helper_link::create_url([
            C__GET__MODULE_ID    => C__MODULE__CMDB,
            C__GET__FILE_MANAGER => 'get',
            C__GET__FILE__ID     => $file['isys_file_physical__id'],
        ]);

        //
        // Code to generate a link to this image in the filesystem
        //
//        // get latest version of file
//        $file_dao = new \isys_cmdb_dao_category_s_file(\isys_application::instance()->database);
//        $files = $file_dao->get_file_by_obj_id($file_obj_id);
//        $file = $files->get_row();
//
//        if (empty($file)) {
//            return;
//        }
//
//        // get physical file data
//        $physical_files = $file_dao->get_file_by_version_id($file['isys_file_version__id']);
//        $physical_file = $physical_files->get_row();
//
//        // get filesystem upload path
//        $l_comp_filemanager = new \isys_component_filemanager();
//        $m_target_dir = $l_comp_filemanager->get_upload_path();
//
//        $base = isys_application::instance()->www_path;
//        $app = isys_application::instance()->app_path;
//
//        // remove app path from upload path to get relative path
//        $upload_path = str_replace($app, '', $m_target_dir);
//        $url = $base . ltrim($upload_path, '/');
//        $url .= $physical_file['isys_file_physical__filename'];
//        $url = str_replace('\\', '/', $url);
//
//        // add server
//        $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
//        $full_url = $protocol . $_SERVER['HTTP_HOST'] . '/' . ltrim($url, '/');
//        return $full_url;
    }

    /**
     * Get the image of a device consider angle.
     *
     * @param  int    $obj_id
     * @param  string $angle
     * @return string|null
     * @throws isys_exception_database
     */
    public static function get_device_image($obj_id, $angle = 'front')
    {
        if (empty($obj_id)) {
            return null;
        }

        $file_dao = isys_cmdb_dao_category_g_file::instance(isys_application::instance()->database);
        $files = $file_dao->get_data_as_array(null, $obj_id, null, null, defined_or_default('C__RECORD_STATUS__NORMAL'));
        if (empty($files)) {
            return null;
        }

        $file = null;
        foreach ($files as $f) {
            if (empty($f['isys_file_version__id'])) {
                continue;
            }

            $query = "SELECT * FROM isys_file_version v
                LEFT JOIN isys_file_physical physical ON physical.isys_file_physical__id = v.isys_file_version__isys_file_physical__id
                WHERE v.isys_file_version__id = " . $f['isys_file_version__id'];
            $fileRow = $file_dao->retrieve($query)->get_row();

            if (! $fileRow) {
                continue;
            }

            $file_title = strtolower($fileRow['isys_file_physical__filename_original']);
            if (strpos($file_title, 'dimage') === false) {
                continue;
            }

            $isBackImage = strpos($file_title, '_back_') !== false || strpos($file_title, '_rear_') !== false;
            if (($angle == 'back' && $isBackImage) || ($angle == 'front' && ! $isBackImage)) {
                // found matching image
                $file = $fileRow;
                break;
            }
        }

        if (! $file) {
            return null;
        }

        return isys_helper_link::create_url([
            C__GET__MODULE_ID    => defined_or_default('C__MODULE__CMDB'),
            C__GET__FILE_MANAGER => 'get',
            C__GET__FILE__ID     => $file['isys_file_physical__id'],
        ]);
    }
}
