<?php

/**
 * i-doit
 *
 * DAO: list of SM-VIEW Patch Parking Deck.
 *
 * @package    i-doit
 * @subpackage ukd
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_cmdb_dao_list_catg_smview_patch_parking_deck extends isys_cmdb_dao_list
{

    /**
     * Return constant of category.
     *
     * @return integer
     */
    public function get_category()
    {
        return defined_or_default('C__CATG__SMVIEW_PATCH_PARKING_DECK');
    }

    /**
     * Return constant of category type.
     *
     * @return integer
     */
    public function get_category_type()
    {
        return C__CMDB__CATEGORY__TYPE_GLOBAL;
    }

    /**
     * Returns array with table headers.
     *
     * @return array
     */
    public function get_fields()
    {
        return [
            "isys_obj__title"                                               => 'LC__UNIVERSAL__OBJECT_TITLE',
            "isys_catg_smview_patch_parking_deck_info_list__obj_title"      => 'LC__CMDB__CATG__SMVIEW_PATCH_PARKING_DECK_INFO__OBJECT_TITLE',
            "isys_catg_smview_patch_parking_deck_info_list__obj_type_const" => 'LC__UNIVERSAL__OBJECT_TYPE',
            "isys_catg_smview_patch_parking_deck_info_list__creation_date"  => 'LC__CMDB__CATG__SMVIEW_PATCH_PARKING_DECK_INFO__CREATION_DATE',
            "isys_obj__created_by"                                          => 'LC__UNIVERSAL__CREATED_BY',
        ];
    }

    /**
     * Modify row method will be called for each row to alter its content.
     *
     * @param  array &$p_row
     * @throws isys_exception_database
     * @throws Exception
     */
    public function modify_row(&$p_row)
    {
        $const = $p_row['isys_catg_smview_patch_parking_deck_info_list__obj_type_const'];
        $sql = 'SELECT * FROM isys_obj_type t WHERE t.isys_obj_type__const = ' . $this->convert_sql_text($const);
        $object_type = $this->m_cat_dao->retrieve($sql)->get_row();

        if ($object_type) {
            $lang = isys_application::instance()->container->get('language');
            $obj_type_title = $lang->get($object_type['isys_obj_type__title']);
            $p_row['isys_catg_smview_patch_parking_deck_info_list__obj_type_const'] = $obj_type_title;
        }

        // important: set this ID to create a valid row-link in listing
        $p_row['isys_catg_virtual_list__id'] = $p_row['isys_catg_smview_patch_parking_deck_info_list__id'];
    }
}
