<?php

use idoit\Module\Patch\Analyzer;

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Patch Analyzer.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_patch_analyzer extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_patch::get_tpl_dir() . 'module/cmdb/catg__smview_patch_analyzer.tpl');
    }

    /**
     * Processes the UI for the category
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->deactivate_commentary();
        $this->disable_ajax_save();

        if (! isys_module_smview_patch::is_licensed()) {
            $lang = isys_application::instance()->container->get('language');
            $this->get_template_component()
                ->assign('license_error', $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__LICENSE_ERROR'));
            return;
        }

        $js_config = self::get_analyzer_configuration($this->m_object_id);
        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_patch::get_assets_www_dir())
            ->assign('js_config', json_encode($js_config));
    }

    /**
     * Get analyzer configuration.
     *
     * @param  int|null $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public static function get_analyzer_configuration($obj_id = null)
    {
        $analyzer = new Analyzer();
        $levels = $obj_id ? $analyzer->get_levels_by_object_id($obj_id) : $analyzer->get_levels_from_root();
        return [
            'ajax_url'         => isys_helper_link::create_url([
                C__GET__AJAX      => 1,
                C__GET__AJAX_CALL => 'patch_analyzer']),
            'connection_url'   => isys_helper_link::create_url([
                C__CMDB__GET__OBJECT => -1,
                C__CMDB__GET__CATG   => defined_or_default('C__CATG__CONNECTOR')]),
            'image_url'        => isys_helper_link::get_base() . '/images/',
            'object_url'       => isys_helper_link::create_url([C__CMDB__GET__OBJECT   => -1]),
            'session'          => $_COOKIE['PHPSESSID'],
            'language'         => self::_get_js_translations(),
            'dynamic_levels'   => $levels,
            'presets'          => $analyzer->get_presets(),
            'connector_types'  => $analyzer->get_connector_types(),
        ];
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private static function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_FILTER',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_RESOLUTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_UPDATE',
            'LC__CMDB__CATG__SMVIEW_PATCH__SHOW_RACK_VIEW',
            'LC__CMDB__CATG__SMVIEW_PATCH__SHOW_CAPACITY',
            'LC__CMDB__CATG__SMVIEW_PATCH__SHOW_ONLY_CONNECTED_NODES',
            'LC__CMDB__CATG__SMVIEW_PATCH__NODE_SYMBOLS',
            'LC__CMDB__CATG__SMVIEW_PATCH__NODE_IMAGES',
            'LC__CMDB__CATG__SMVIEW_PATCH__MARK_NEIGHBOR_EDGES',
            'LC__CMDB__CATG__SMVIEW_PATCH__FILTER_CONNECTION_TYPE',
            'LC__CMDB__CATG__SMVIEW_PATCH__FILTER_CONNECTOR_TYPE',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_PRESET',
            'LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_HIERARCHIC',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_ORTHOGONAL',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_CIRCULAR',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_ORGANIC',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_RADIAL',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_SERIES_PARALLEL',
            'LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_BALLOON',
            'LC__CMDB__CATG__SMVIEW_PATCH__EXPORT_GRAPH',
            'LC__CMDB__CATG__SMVIEW_PATCH__DISPLAY_DETAILS',
            'LC__CMDB__CATG__SMVIEW_PATCH__NODE_REPRESENTATION',
            'LC__CMDB__CATG__SMVIEW_PATCH__EDGE_REPRESENTATION',
            'LC__CMDB__CATG__SMVIEW_PATCH__EDGE_COLORING',
            'LC__UNIVERSAL__DEFAULT',
            'LC__CMDB__CATG__SMVIEW_PATCH__CAPACITY',
            'LC__CMDB__CATG__SMVIEW_PATCH__CABLE_COLOR',
            'LC_UNIVERSAL__OBJECT_TYPE',
            'LC__CMDB__CATG__SMVIEW_PATCH__PATCHED_COUNT',
            'LC__CMDB__CATG__SMVIEW_PATCH__TOTAL_COUNT',
            'LC__CMDB__CATG__SMVIEW_PATCH__CTX_OPEN_OBJECT',
            'LC__CMDB__CATG__SMVIEW_PATCH__CTX_OPEN_CONNECTIONS',
            'LC__CMDB__CATG__SMVIEW_PATCH__ROUTING',
            'LC__CMDB__CATG__SMVIEW_PATCH__ROUTING_SHORTEST_PATH',
            'LC__CMDB__CATG__SMVIEW_PATCH__ROUTING_OPTIMAL_PATH',
            'LC__CMDB__CATG__SMVIEW_PATCH__ROUTING_FREE_CONNECTION_COUNT',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $lang = isys_application::instance()->container->get('language');
        return $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_TITLE');
    }
}
