<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Patch Grid.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_patch_editor extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        
        $this->set_template(isys_module_smview_patch::get_tpl_dir() . 'module/cmdb/catg__smview_patch_editor.tpl');
    }

    /**
     * Processes the UI for the category grid
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->deactivate_commentary();
        $this->disable_ajax_save();

        if (! isys_module_smview_patch::is_licensed()) {
            $lang = isys_application::instance()->container->get('language');
            $this->get_template_component()
                ->assign('license_error', $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__LICENSE_ERROR'));
            return;
        }

        // get all racks
        $global_dao = isys_cmdb_dao_category_g_global::instance(isys_application::instance()->database);
        $obj_row = $global_dao->get_object($this->m_object_id)->get_row();
        $global_data = $global_dao->get_objects_by_type_id(defined_or_default('C__OBJTYPE__ENCLOSURE'), defined_or_default('C__RECORD_STATUS__NORMAL'));
        $racks = [];
        while ($row = $global_data->get_row()) {
            $racks[$row['isys_obj__id']] = $row['isys_obj__title'];
        }

        // create JS configuration
        $use_cable_color = isys_settings::get('smview_patch.editor.color_from_cable');
        $rtl = isys_settings::get(isys_module_smview_patch::RIGHT_ALIGN_CONNECTOR_TITLE, 0);
        $tracePortDirectly = isys_settings::get(isys_module_smview_patch::TRACE_RIGHT_PORT_DIRECTLY, 0);

        $permissions = [
            'has_edit_permission'             => isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_PATCH_EDITOR"),
            'has_planner_edit_permission'     => isys_auth_smview_patch::instance()->is_allowed_to(isys_auth::EDIT, 'planner'),
            'has_planner_activate_permission' => isys_auth_smview_patch::instance()->is_allowed_to(isys_auth::EXECUTE, 'planner'),
            'has_end_device_edit_permission'  => isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_PATCH_END_DEVICE"),
        ];

        $js_config = [
            'ajax_url'                     => isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'patch_grid']),
            'session'                      => $_COOKIE['PHPSESSID'],
            'image_url'                    => isys_helper_link::get_base() . '/images/icons/silk/',
            'spinner_url'                  => isys_helper_link::get_base() . '/images/ajax-loading.gif',
            'lang'                         => isys_application::instance()->language,
            'language'                     => $this->_get_js_translations(),
            'rack_obj_id'                  => $obj_row['isys_obj_type__const'] == 'C__OBJTYPE__ENCLOSURE' ? $this->m_object_id : null,
            'device_obj_id'                => $obj_row['isys_obj_type__const'] != 'C__OBJTYPE__ENCLOSURE' ? $this->m_object_id : null,
            'racks'                        => $racks,
            'size'                         => isys_usersettings::get('smview_patch.editor.icon_size', 'm'),
            'colors'                       => isys_module_smview_patch::get_editor_colors(),
            'cable_colors'                 => $this->_get_from_dialog('isys_cable_colour'),
            'cable_types'                  => $this->_get_from_dialog('isys_cable_type'),
            'connector_types'              => $this->_get_from_dialog('isys_connection_type'),
            'status_list'                  => $this->_get_status(),
            'use_cable_color'              => $use_cable_color == '1',
            'right_align_connector_titles' => ! empty($rtl),
            'permissions'                  => $permissions,
            'trace_right_port_directly'    => ! empty($tracePortDirectly),
        ];

        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_patch::get_assets_www_dir())
            ->assign('js_config', json_encode($js_config));

        isys_component_template_navbar::getInstance()
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT_VERTICAL',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK_VERTICAL',
            'LC__CMDB__CATG__SMVIEW_PATCH__NO_GRID_DEFINED1',
            'LC__CMDB__CATG__SMVIEW_PATCH__NO_GRID_DEFINED2',
            'LC__CMDB__CATG__SMVIEW_PATCH__OPEN_OBJECT',
            'LC__CMDB__CATG__SMVIEW_PATCH__NOT_CONNECTED',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED',
            'LC__CMDB__CATG__SMVIEW_PATCH__TRACE',
            'LC__CMDB__CATG__SMVIEW_PATCH__OPEN_CONNECTED_CABLE_PIN',
            'LC__CMDB__CATG__SMVIEW_PATCH__OPEN_SOCKET_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__OPEN_END_DEVICE_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__NAME',
            'LC__CMDB__CATG__SMVIEW_PATCH__TYPE',
            'LC__CMDB__CATG__SMVIEW_PATCH__CABLE',
            'LC__CMDB__CATG__SMVIEW_PATCH__CABLE_COLOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__CABLE_TYPE',
            'LC__CMDB__CATG__SMVIEW_PATCH__CABLE_LENGTH',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED_WITH',
            'LC__CMDB__CATG__SMVIEW_PATCH__LINK_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__PATCH_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__INTERFACE_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_PATCH',
            'LC__CMDB__CATG__SMVIEW_PATCH__RECONNECT_PATCH',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_PATCH',
            'LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_LINK',
            'LC__CMDB__CATG__SMVIEW_PATCH__RECONNECT_LINK',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_LINK',
            'LC__CMDB__CATG__SMVIEW_PATCH__SWITCH_SIDES',
            'LC__CMDB__CATG__SMVIEW_PATCH__OPEN_ON_OTHER_SIDE',
            'LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_LIST_VIEW',
            'LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_COMMENT_DIALOG',
            'LC__CMDB__CATG__SMVIEW_PATCH__PORT',
            'LC__CMDB__CATG__SMVIEW_PATCH__COMMENT',
            'LC__CMDB__CATG__SMVIEW_PATCH__PRINT_RACK_DATA',
            'LC__CMDB__CATG__SMVIEW_PATCH__PRINT_DEVICE_DATA',
            'LC__CMDB__CATG__SMVIEW_PATCH__COLLAPSE_MENU',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED_WITH',
            'LC__UNIVERSAL__STATUS',
            'LC__UNIVERSAL__BUTTON_SAVE',
            'LC__CMDB__CATG__SMVIEW_PATCH__SOCKET_SELECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__END_DEVICE_SELECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__SEARCH',
            'LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES',
            'LC__CMDB__CATG__SMVIEW_PATCH__OBJECTS',
            'LC__CMDB__CATG__SMVIEW_PATCH__PORTS',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_SOCKET',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_END_DEVICE',
            'LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_SOCKET',
            'LC__CMDB__CATG__SMVIEW_PATCH__CABLE_NAME_ALREADY_EXISTS',
            'LC__CMDB__CATG__SMVIEW_PATCH__ADD_CABLE_WITH_SAME_NAME',
            'LC__CMDB__CATG__SMVIEW_PATCH__ADD_TO_EXISTING_CABLE',
            'LC__CMDB__CATG__SMVIEW_PATCH__PLANNER_MODE',
            'LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_PLANNER_MODE',
            'LC__CMDB__CATG__SMVIEW_PATCH__VIRTUAL_PATCH_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_VIRTUAL_PATCH',
            'LC__CMDB__CATG__SMVIEW_PATCH__RECONNECT_VIRTUAL_PATCH',
            'LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_VIRTUAL_PATCH',
            'LC__CMDB__CATG__SMVIEW_PATCH__ACTIVATE_PLANNED_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__ACTIVATE_PLANNED_CONNECTIONS',
            'LC__CMDB__CATG__SMVIEW_PATCH__REMOVE_PLANNED_CONNECTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__ACTIVATE_PLANNED_CONNECTION_REMOVAL',
            'LC__CMDB__CATG__SMVIEW_PATCH__REMOVE_PLANNED_CONNECTION_REMOVAL',
            'LC__CMDB__CATG__SMVIEW_PATCH__END_DEVICE_TITLE',
            'LC__CMDB__CATG__SMVIEW_PATCH__PLANNED_END_DEVICE_TITLE',
            'LC__CMDB__CATG__SMVIEW_PATCH__VERIFY_CABLE_NAME',
        ];
        
        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Get all available cable colors.
     *
     * @param  string $table_name
     * @return array
     * @throws isys_exception_database
     */
    private function _get_from_dialog($table_name)
    {
        $dao = new isys_cmdb_dao_category_g_global($this->m_database_component);
        $sql = "SELECT * FROM $table_name";
        $res = $dao->retrieve($sql);
        $colors = [];
        while ($row = $res->get_row()) {
            $id = $table_name . '__id';
            $title = $table_name . '__title';
            $colors[$row[$id]] = $row[$title];
        }
        return $colors;
    }

    /**
     * Get all available cable colors.
     *
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _get_status()
    {
        $table_name = 'isys_cmdb_status';
        $dao = new isys_cmdb_dao_category_g_global($this->m_database_component);
        $sql = "SELECT * FROM $table_name";
        $res = $dao->retrieve($sql);
        $status = [];
        $lang = isys_application::instance()->container->get('language');
        while ($row = $res->get_row()) {
            $id = $table_name . '__id';
            $title = $table_name . '__title';
            $status[$row[$id]] = [
                'title' => $lang->get($row[$title]),
                'color' => '#' . $row[$table_name . '__color'],
            ];
        }
        return $status;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $lang = isys_application::instance()->container->get('language');
        return $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__PATCH_EDITOR_TITLE');
    }
}
