<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Patch Grid.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_patch_grid extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_patch::get_tpl_dir() . 'module/cmdb/catg__smview_patch_grid.tpl');
    }

    /**
     * Processes the UI for the category grid
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        $l_rules = [];
        $l_catdata = $p_cat->get_general_data();
        $list_id = $l_catdata['isys_catg_smview_patch_grid_list__id'];

        if ($_POST['navMode'] && $_POST['navMode'] == defined_or_default('C__NAVMODE__DELETE') && $list_id) {
            // delete this item
            $p_cat->delete_entry($list_id, 'isys_catg_smview_patch_grid_list');
        }

        parent::process($p_cat);
        $this->deactivate_commentary();
        $this->disable_ajax_save();

        if (! isys_module_smview_patch::is_licensed()) {
            $lang = isys_application::instance()->container->get('language');
            $this->get_template_component()
                ->assign('license_error', $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__LICENSE_ERROR'));
            return;
        }

        $available_connectors = $p_cat->get_available_connectors($this->m_object_id);

        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_database_component);
        $global_dao = new isys_cmdb_dao_category_g_global($this->m_database_component);
        $obj = $global_dao->get_object($this->m_object_id)->get_row();
        $grid = [
            'frame' => [
                'title' => $obj['isys_obj__title'],
            ],
            'ports' => [],
        ];

        if ($list_id) {
            // get grid data
            $grid_template_id = $l_catdata['isys_catg_smview_patch_grid_list__grid_template_id'];
            if ($grid_template_id) {
                $grid_template = $grid_template_dao->get_object('id', $grid_template_id);

                $assigned_connectors = $this->_get_assigned_connector_templates($list_id, $grid_template_id, $available_connectors);
                $grid['ports'] = array_values($assigned_connectors);
                $grid['frame'] = [
                    'id'                => $grid_template_id,
                    'title'             => $grid_template['title'],
                    'column_count'      => (int) $grid_template['column_count'],
                    'row_count'         => (int) $grid_template['row_count'],
                    'column_count_back' => (int) $grid_template['column_count_back'],
                    'row_count_back'    => (int) $grid_template['row_count_back'],
                    'is_template'       => (int) $grid_template['is_template'],
                    'used_by'           => $p_cat->get_objects_using_grid_template($grid_template_id),
                ];

                // get custom properties
                $l_custom_properties = $p_cat->get_custom_properties();
                foreach ($l_custom_properties as $l_property) {
                    $l_rules[$l_property[C__PROPERTY__UI][C__PROPERTY__UI__ID]]['p_strValue'] = $grid;
                }
            }
        }
        
        // get available grid templates
        $templates = $grid_template_dao->filter_objects(array('is_template' => 1));
        $available_grid_templates = [];
        foreach ($templates as $template) {
            $available_grid_templates[$template['id']] = $template['title'];
        }

        $isEditable = false;
        if ($_POST[C__GET__NAVMODE] && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')) {
            $isEditable = true;
        }
        if ($_POST[C__GET__NAVMODE] && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__NEW')) {
            $isEditable = true;
        }
        if ($_GET[C__CMDB__GET__EDITMODE] && $_GET[C__CMDB__GET__EDITMODE] == defined_or_default('C__EDITMODE__ON')) {
            $isEditable = true;
        }
        if ($_POST[C__GET__NAVMODE] && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            // disable edit mode after save
            $isEditable = false;
        }

//        if (! isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::EDIT, $this->m_object_id, $p_cat->get_category_const())) {
        if (! isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_PATCH_GRID")) {
            $isEditable = false;
        }

        if ($isEditable) {
            isys_component_template_navbar::getInstance()
                ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active(true, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__CANCEL'))
                ->set_active(true, defined_or_default('C__NAVBAR_BUTTON__CANCEL'))
                ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'));

        }
        
        // create JS configuration
        $rtl = isys_settings::get(isys_module_smview_patch::RIGHT_ALIGN_CONNECTOR_TITLE, 0);
        $js_config = [
            'ajax_url'    => isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'patch_grid_template']),
            'session'     => $_COOKIE['PHPSESSID'],
            'image_url'   => isys_helper_link::get_base() . '/images/icons/silk/',
            'grid'        => $grid,
            'connectors'  => array_values($available_connectors),
            'templates'   => $available_grid_templates,
            'language'    => $this->_get_js_translations(),
            'size'        => isys_usersettings::get('smview_patch.editor.icon_size', 'm'),
            'sizes'       => isys_cmdb_dao_smview_patch_grid_template::get_available_sizes(),
            'is_editable' => $isEditable,
            'right_align_connector_titles' => ! empty($rtl),
        ];

        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_patch::get_assets_www_dir())
            ->assign('js_config', json_encode($js_config))
            ->assign('C__CATG__SMVIEW_PATCH_GRID', json_encode($grid))
            ->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);

        if ($list_id) {
            $canDelete = isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::DELETE, $this->m_object_id, $p_cat->get_category_const());
            $btn = defined_or_default('C__NAVBAR_BUTTON__DELETE');
            $lang = isys_application::instance()->container->get('language');
            isys_component_template_navbar::getInstance()
                ->add_onclick_prepend(
                $btn,
                'if (! confirm(\'' . $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__TREE_DELETE_OBJECT') . '\')) {return false;}')
                ->set_active($canDelete, $btn)
                ->set_visible($canDelete, $btn);
        }
    }

    /**
     * Get configured connector templates.
     *
     * @param  int $list_id
     * @param  int $grid_template_id
     * @param  array $available_connectors
     * @return array
     * @throws isys_exception_database
     */
    private function _get_assigned_connector_templates($list_id, $grid_template_id, $available_connectors)
    {
        $grid_connector_dao = new isys_cmdb_dao_smview_patch_grid_connector($this->m_database_component);
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template($this->m_database_component);
        
        // get all connector templates
        $connector_templates = [];
        $tpls = $connector_template_dao->filter_objects(array('grid_template_id' => $grid_template_id));
        foreach ($tpls as $tpl) {
            $connector_templates[$tpl['id']] = $tpl;
        }
        
        // get all assigned connector instances
        $connections = $grid_connector_dao->filter_objects(array('grid_list_id' => $list_id));
        $connectors = [];
        foreach ($connections as $c) {
            $connector_template_id = $c['connector_template_id'];
            if (! isset($connector_templates[$connector_template_id])) {
                continue;
            }
            $ct = $connector_templates[$connector_template_id];
            
            // remove this template to leave only new ones
            unset($connector_templates[$connector_template_id]);
            
            $connector_id = $c['connector_id'];
            $connector = $this->_get_extended_connector_template($ct, $connector_id, $available_connectors);
            $connectors[$c['connector_id']] = $connector;
        }
        
        // add new connector templates, which are not assigned yet
        foreach ($connector_templates as $id => $tpl) {
            $connector = $this->_get_extended_connector_template($tpl);
            $connector['is_new'] = true;
            $connectors['new_' . $id] = $connector;
        }
        return $connectors;
    }

    /**
     * Get connector template with extended information.
     *
     * @param  array $template
     * @param  int $connector_id
     * @param  array $available_connectors
     * @return array
     * @throws isys_exception_database
     */
    private function _get_extended_connector_template($template, $connector_id = null, $available_connectors = null)
    {
        $connector = [
            'id'                   => (int) $template['id'],
            'title'                => $template['title'],
            'number'               => (int) $template['number'],
            'pos_x'                => (int) $template['pos_x'],
            'pos_y'                => (int) $template['pos_y'],
            'rotation'             => (int) $template['rotation'],
            'label_orientation'    => $template['label_orientation'],
            'status'               => $template['status'],
            'angle'                => $template['angle'],
            // leave this blank if connector is not available anymore
            'connector_id'         => null,
            'connector_title'      => $template['connector_title'],
            'connector_type'       => $template['connector_type'],
            'connector_type_id'    => (int) $template['connector_type_id'],
        ];
        
        if (isset($connector_id) && isset($available_connectors[$connector_id])) {
            $conn = $available_connectors[$connector_id];
            
            $connector['connector_id'] = $connector_id;
            $connector['title'] = $conn['connector_title'] ?: $connector['title'];
            
            // update data with current connector settings
            $connector['connector_title'] = $conn['connector_title'];
            $connector['connector_type'] = $conn['connector_type'];
            $connector['connector_type_id'] = $conn['connector_type_id'];
        }
        
        $picture_dao = new isys_cmdb_dao_smview_patch_picture($this->m_database_component);
        $pic = $picture_dao->get_object('id', $connector['connector_type_id']);
        $type_title = ! empty($pic) ? $pic['title'] : null;
        
        $connector['connector_type_title'] = $type_title;
        $connector['url'] = $picture_dao->get_file_url($pic['obj_id']);
        return $connector;
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_PATCH__DESCRIPTION',
            'LC__CMDB__CATG__SMVIEW_PATCH__IS_TEMPLATE',
            'LC__CMDB__CATG__SMVIEW_PATCH__USE_TEMPLATE',
            'LC__CMDB__CATG__SMVIEW_PATCH__EDIT_TEMPLATE',
            'LC__CMDB__CATG__SMVIEW_PATCH__ROW_COUNT',
            'LC__CMDB__CATG__SMVIEW_PATCH__COLUMN_COUNT',
            'LC__CMDB__CATG__SMVIEW_PATCH__SIZE',
            'LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT',
            'LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_NO',
            'LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_L-R',
            'LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_T-B',
            'LC__CMDB__CATG__SMVIEW_PATCH__GENERATE_ASSIGNMENT',
            'LC__CMDB__CATG__SMVIEW_PATCH__SPIN',
            'LC__CMDB__CATG__SMVIEW_PATCH__DELETE',
            'LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_CONNECTOR_TYPE',
            'LC__CMDB__CATG__SMVIEW_PATCH__ERROR_NO_MATCHING_NAME',
            'LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_HAS_NEW_CONNECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_CONNECTOR_NOT_ASSIGNED',
            'LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_USED_BY',
            'LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGN',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT',
            'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK',
            'LC__CMDB__CATG__SMVIEW_PATCH__CREATE_TEMPLATE_COPY',
            'LC__CMDB__CATG__SMVIEW_PATCH__OTHER',
            'LC__CMDB__CATG__SMVIEW_PATCH__FRAME_PROPERTIES',
            'LC__CMDB__CATG__SMVIEW_PATCH__FRAME_EDITOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__PORTS',
        ];
        
        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $lang = isys_application::instance()->container->get('language');
        return $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__GRID_EDITOR_TITLE');
    }
}
