<?php

use idoit\Module\Patch\Analyzer;

/**
 * SM-VIEW Patch Analyzer AJAX handler.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_ajax_handler_patch_analyzer extends isys_ajax_handler
{
    /**
     * Init method, which gets called from the framework.
     *
     * @throws Exception
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $analyzer = new Analyzer();
            switch ($this->m_get['func']) {
                case 'filter-elements-by-level':
                    $obj_id = ! empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                    $level = ! empty($this->m_post['level']) ? $this->m_post['level'] : $this->m_get['level'];
                    $connectorTypes = ! empty($this->m_post['connector_types']) ? $this->m_post['connector_types'] : $this->m_get['connector_types'];
                    $connectorTypes = ! empty($connectorTypes) ? explode(',', $connectorTypes) : null;
                    $l_return['data'] = $analyzer->get_dynamic_elements_by_level($obj_id, $level, C__CONNECTOR__INPUT, $connectorTypes);
                    break;
                case 'get-racks':
                    $obj_id = ! empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                    $level = ! empty($this->m_post['level']) ? $this->m_post['level'] : $this->m_get['level'];
                    $l_return['data'] = $this->_get_racks($obj_id, $level);
                    break;
                case 'get-elements-by-type':
                    $type_id = ! empty($this->m_post['type_id']) ? $this->m_post['type_id'] : $this->m_get['type_id'];
                    $obj_id = ! empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                    $l_return['data'] = $analyzer->get_elements_by_type($type_id, $obj_id);
                    break;
                case 'get-elements-underneath_object':
                    $obj_id = ! empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                    $l_return['data'] = $analyzer->get_elements_underneath_object($obj_id);
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }

    /**
     * Get racks assigned to a location.
     *
     * @param  int $obj_id
     * @param  int $level
     * @return array
     * @throws isys_exception_database
     */
    private function _get_racks($obj_id, $level)
    {
        $analyzer = new Analyzer();
        $elements = $analyzer->get_dynamic_elements_by_level($obj_id, $level);

        $racks = [];
        $patch_dao = isys_cmdb_dao_category_g_smview_patch_editor::instance(isys_application::instance()->database);
        foreach ($elements as $element) {
            $rack = $patch_dao->get_rack($element['id']);
            $rack['connections'] = $element['connections'];
            $racks[] = $rack;
        }
        return $racks;
    }
}
