<?php

/**
 * SM-VIEW Patch Grid AJAX handler.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_ajax_handler_patch_grid extends isys_ajax_handler
{

    private $_cached_locations = [];

    /**
     * Init method, which gets called from the framework.
     *
     * @throws Exception
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $canView = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, "CATEGORY/C__CATG__SMVIEW_PATCH_EDITOR");
            $canEdit = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_PATCH_EDITOR");
            $canEditPlanner = isys_auth_smview_patch::instance()->is_allowed_to(isys_auth::EDIT, 'planner');
            $canActivatePlanner = isys_auth_smview_patch::instance()->is_allowed_to(isys_auth::EXECUTE, 'planner');
            switch ($this->m_get['func']) {
                case 'get-rack':
                    if ($canView) {
                        $rack_id = ! empty($this->m_post['rack_id']) ? $this->m_post['rack_id'] : $this->m_get['rack_id'];
                        $device_id = ! empty($this->m_post['device_id']) ? $this->m_post['device_id'] : $this->m_get['device_id'];
                        $patch_dao = isys_cmdb_dao_category_g_smview_patch_editor::instance(isys_application::instance()->database);
                        $l_return['data'] = $patch_dao->get_rack($rack_id, $device_id);
                    }
                    break;
                case 'get-grid':
                    if ($canView) {
                        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $patch_dao = isys_cmdb_dao_category_g_smview_patch_editor::instance(isys_application::instance()->database);
                        $l_return['data'] = $patch_dao->get_grid($obj_id);
                    }
                    break;
                case 'get-port':
                    if ($canView) {
                        $connector_id = !empty($this->m_post['connector_id']) ? $this->m_post['connector_id'] : $this->m_get['connector_id'];
                        $patch_dao = isys_cmdb_dao_category_g_smview_patch_editor::instance(isys_application::instance()->database);
                        $l_return['data'] = $patch_dao->get_connector_by_id($connector_id);
                    }
                    break;
                case 'get-cables-by-name':
                    if ($canView) {
                        $cable_name = !empty($this->m_post['cable_name']) ? $this->m_post['cable_name'] : $this->m_get['cable_name'];
                        $l_return['data'] = $this->_get_cables_by_name($cable_name);
                    }
                    break;
                case 'connect-ports':
                    if ($canEdit) {
                        $connector_a = !empty($this->m_post['connector_a']) ? $this->m_post['connector_a'] : $this->m_get['connector_a'];
                        $connector_b = !empty($this->m_post['connector_b']) ? $this->m_post['connector_b'] : $this->m_get['connector_b'];
                        $cable_name = !empty($this->m_post['cable_name']) ? $this->m_post['cable_name'] : $this->m_get['cable_name'];
                        $add_to_exiting_cable = !empty($this->m_post['add_to_exiting_cable']) ? $this->m_post['add_to_exiting_cable'] : $this->m_get['add_to_exiting_cable'];
                        $add_to_exiting_cable = $add_to_exiting_cable == 'true';
                        $gridDao = new isys_cmdb_dao_smview_patch_grid($this->m_database_component);
                        $l_return['data'] = $gridDao->assign_connectors($connector_a, $connector_b, $cable_name, $add_to_exiting_cable);
                    }
                    break;
                case 'disconnect-ports':
                    if ($canEdit) {
                        $connector_id = !empty($this->m_post['connector_id']) ? $this->m_post['connector_id'] : $this->m_get['connector_id'];
                        $gridDao = new isys_cmdb_dao_smview_patch_grid($this->m_database_component);
                        $l_return['data'] = $gridDao->detach_cable($connector_id, true);
                    }
                    break;
                case 'connect-virtual-ports':
                    // create a new virtual connection
                    if ($canView && $canEditPlanner) {
                        $connector_a = !empty($this->m_post['connector_a']) ? $this->m_post['connector_a'] : $this->m_get['connector_a'];
                        $connector_b = !empty($this->m_post['connector_b']) ? $this->m_post['connector_b'] : $this->m_get['connector_b'];
                        $cable_name = !empty($this->m_post['cable_name']) ? $this->m_post['cable_name'] : $this->m_get['cable_name'];
                        $add_to_exiting_cable = !empty($this->m_post['add_to_exiting_cable']) ? $this->m_post['add_to_exiting_cable'] : $this->m_get['add_to_exiting_cable'];
                        $add_to_exiting_cable = $add_to_exiting_cable == 'true';
                        $gridDao = new isys_cmdb_dao_smview_patch_grid($this->m_database_component);
                        $l_return['data'] = $gridDao->assign_virtual_connectors($connector_a, $connector_b, $cable_name, $add_to_exiting_cable);
                    }
                    break;
                case 'disconnect-virtual-ports':
                    // disconnect two connectors virtually
                    if ($canView && $canEditPlanner) {
                        $connector_id = !empty($this->m_post['connector_id']) ? $this->m_post['connector_id'] : $this->m_get['connector_id'];
                        $gridDao = new isys_cmdb_dao_smview_patch_grid($this->m_database_component);
                        $l_return['data'] = $gridDao->detach_virtual_connector($connector_id );
                    }
                    break;
                case 'remove-virtual-connection':
                    // remove a virtual connection
                    if ($canView && $canEditPlanner) {
                        $connector_a = !empty($this->m_post['connector_a']) ? $this->m_post['connector_a'] : $this->m_get['connector_a'];
                        $connector_b = !empty($this->m_post['connector_b']) ? $this->m_post['connector_b'] : $this->m_get['connector_b'];
                        $gridDao = new isys_cmdb_dao_smview_patch_grid($this->m_database_component);
                        $l_return['data'] = $gridDao->remove_virtual_connection($connector_a, $connector_b);
                    }
                    break;
                case 'activate-virtual-connection':
                    // activate a planned virtual connection
                    if ($canView && $canActivatePlanner) {
                        $connector_a = !empty($this->m_post['connector_a']) ? $this->m_post['connector_a'] : $this->m_get['connector_a'];
                        $connector_b = !empty($this->m_post['connector_b']) ? $this->m_post['connector_b'] : $this->m_get['connector_b'];
                        $gridDao = new isys_cmdb_dao_smview_patch_grid($this->m_database_component);
                        $l_return['data'] = $gridDao->activate_virtual_connection($connector_a, $connector_b);
                    }
                    break;
                case 'activate-virtual-connections':
                    // activate all planned virtual connection of a device
                    if ($canView && $canActivatePlanner) {
                        $device_id = !empty($this->m_post['device_id']) ? $this->m_post['device_id'] : $this->m_get['device_id'];
                        $gridDao = new isys_cmdb_dao_smview_patch_grid($this->m_database_component);
                        $l_return['data'] = $gridDao->activate_virtual_connections($device_id);
                    }
                    break;
                case 'change-cable':
                    if ($canEdit) {
                        $cable_id = !empty($this->m_post['cable_id']) ? $this->m_post['cable_id'] : $this->m_get['cable_id'];
                        $length = !empty($this->m_post['length']) ? $this->m_post['length'] : $this->m_get['length'];
                        $color_id = !empty($this->m_post['color_id']) ? $this->m_post['color_id'] : $this->m_get['color_id'];
                        $type_id = !empty($this->m_post['type_id']) ? $this->m_post['type_id'] : $this->m_get['type_id'];
                        $this->_change_cable_data($cable_id, $length, $color_id, $type_id);

                        $title = !empty($this->m_post['title']) ? $this->m_post['title'] : $this->m_get['title'];
                        if (! empty($title)) {
                            $dao = new isys_cmdb_dao_object(isys_application::instance()->database);
                            $objectRow = $dao->get_object($cable_id)->get_row();
                            if ($objectRow['isys_obj__title'] != $title) {
                                $dao->update_object($cable_id, null, $title);

                                $changes = [
                                    'LC__UNIVERSAL__TITLE' => [
                                        'from' => $objectRow['isys_obj__title'],
                                        'to'   => $title,
                                    ],
                                ];
                                $this->_logCategoryChanged(
                                    $cable_id,
                                    'LC__CMDB__CATG__GLOBAL',
                                    $changes);
                                }
                        }

                        $end_device_name = !empty($this->m_post['end_device']) ? $this->m_post['end_device'] : $this->m_get['end_device'];
                        $planned_end_device_name = !empty($this->m_post['planned_end_device']) ? $this->m_post['planned_end_device'] : $this->m_get['planned_end_device'];
                        $this->_save_end_device($end_device_name, $planned_end_device_name);
                    }
                    break;
                case 'change-connector-type':
                    if ($canEdit) {
                        $connector_id = !empty($this->m_post['connector_id']) ? $this->m_post['connector_id'] : $this->m_get['connector_id'];
                        $type = !empty($this->m_post['type']) ? $this->m_post['type'] : $this->m_get['type'];
                        $cable_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
                        $l_return['data'] = $cable_dao->save_data($connector_id, ['connection_type' => $type]);
                    }
                    break;
                case 'change-connector-status':
                    if ($canEdit) {
                        $connector_id = !empty($this->m_post['connector_id']) ? $this->m_post['connector_id'] : $this->m_get['connector_id'];
                        $status = !empty($this->m_post['status']) ? $this->m_post['status'] : $this->m_get['status'];
                        $this->_set_connector_status($connector_id, $status);
                    }
                    break;
                case 'add-device-comment':
                    if ($canEdit) {
                        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $comment = !empty($this->m_post['comment']) ? $this->m_post['comment'] : $this->m_get['comment'];
                        $this->_add_object_comment($obj_id, $comment);
                    }
                    break;
                case 'get-object-types':
                    if ($canView) {
                        $object_type_id = !empty($this->m_post['object_type_id']) ? $this->m_post['object_type_id'] : $this->m_get['object_type_id'];
                        $connection_type = !empty($this->m_post['connection_type']) ? $this->m_post['connection_type'] : $this->m_get['connection_type'];
                        $default_socket_type = isys_settings::get(isys_module_smview_patch::DEFAULT_SOCKET_TYPE, null);
                        $l_return['data'] = [
                            'object_types'        => $this->_get_object_types($object_type_id, $connection_type),
                            'default_socket_type' => $default_socket_type,
                        ];
                    }
                    break;
                case 'get-objects':
                    if ($canView) {
                        $object_type_id = !empty($this->m_post['object_type_id']) ? $this->m_post['object_type_id'] : $this->m_get['object_type_id'];
                        $object_id = !empty($this->m_post['object_id']) ? $this->m_post['object_id'] : $this->m_get['object_id'];
                        $connection_type = !empty($this->m_post['connection_type']) ? $this->m_post['connection_type'] : $this->m_get['connection_type'];
                        $l_return['data'] = $this->_get_objects_by_object_type_id($object_type_id, $object_id, $connection_type);
                    }
                    break;
                case 'get-ports':
                    if ($canView) {
                        $object_id = !empty($this->m_post['object_id']) ? $this->m_post['object_id'] : $this->m_get['object_id'];
                        $port_id = !empty($this->m_post['port_id']) ? $this->m_post['port_id'] : $this->m_get['port_id'];
                        $connection_type = !empty($this->m_post['connection_type']) ? $this->m_post['connection_type'] : $this->m_get['connection_type'];
                        $l_return['data'] = $this->_get_ports($object_id, null, $port_id, $connection_type);
                    }
                    break;
                case 'get-port-data':
                    if ($canView) {
                        $port_id = !empty($this->m_post['port_id']) ? $this->m_post['port_id'] : $this->m_get['port_id'];
                        $connection_type = !empty($this->m_post['connection_type']) ? $this->m_post['connection_type'] : $this->m_get['connection_type'];
                        $ports = $this->_get_ports(null, $port_id, null, $connection_type);
                        $l_return['data'] = count($ports) > 0 ? array_shift($ports) : [];
                    }
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate() 
    {
        return true;
    }

    /**
     * Get all active cables by its name.
     *
     * @param  string $cable_name
     * @return array
     * @throws isys_exception_database
     */
    private function _get_cables_by_name($cable_name)
    {
        $cables = [];
        $l_dao = isys_cmdb_dao_cable_connection::instance(isys_application::instance()->database);
        if (!empty($cable_name)) {
            $p_obj_type = defined_or_default('C__OBJTYPE__CABLE');
            $p_status = defined_or_default('C__RECORD_STATUS__NORMAL');
            $l_sql = 'SELECT isys_obj__id FROM isys_obj WHERE isys_obj__title = ' . $l_dao->convert_sql_text($cable_name)
                   . ' AND isys_obj__isys_obj_type__id = ' . $l_dao->convert_sql_id($p_obj_type)
                   . ' AND isys_obj__status = ' . $l_dao->convert_sql_int($p_status);

            $rows = $l_dao->retrieve($l_sql . ';');
            while ($row = $rows->get_row()) {
                $cables[] = [
                    'id'    => $row['isys_obj__id'],
                    'title' => $row['isys_obj__title'],
                ];
            }
        }
        return $cables;
    }

    /**
     * Change cable length.
     *
     * @param  int $cable_id
     * @param  string $length
     * @throws isys_exception_database
     */
    private function _change_cable_data($cable_id, $length, $color_id, $type_id)
    {
        $parts = preg_split('#(?<=[\d,.])(?=[a-z])#i', $length);
        $length = $parts[0];
        $unit = $parts[1];

        $length = str_replace(',', '.', $length);
        $length = floatval($length);

        $cable_dao = new isys_cmdb_dao_category_g_cable(isys_application::instance()->database);
        $cableRow = $cable_dao->get_data(null, $cable_id)->get_row();

        $changes = [];
        $unit_const = $this->_get_cable_length_unit_const($unit);
        $unit_factor = null;
        if ($unit_const) {
            $sql = "SELECT * FROM isys_depth_unit WHERE isys_depth_unit__const = '$unit_const'";
            $row = $cable_dao->retrieve($sql)->get_row();
            $cable_dao->save_single_value($cable_id, ['length_unit' => $row['isys_depth_unit__id']]);
            $unit_factor = $row['isys_depth_unit__factor'];

            $changes['LC__CMDB__CATS__CABLE__LENGTH_UNIT'] = [
                'from' => $cableRow ? $cableRow['isys_catg_cable_list__isys_depth_unit__id'] : '',
                'to'   => $row['isys_depth_unit__id'],
            ];
        }

        if (empty($unit_factor)) {
            // try to get existing unit factor
            $sql = 'SELECT u.isys_depth_unit__factor FROM isys_catg_cable_list l
                    LEFT JOIN isys_depth_unit u ON isys_depth_unit__id = isys_catg_cable_list__isys_depth_unit__id
                    WHERE isys_catg_cable_list__isys_obj__id = ' . $cable_id;
            $unit_row = $cable_dao->retrieve($sql)->get_row();
            $unit_factor = $unit_row ? $unit_row['isys_depth_unit__factor'] : null;
        }

        if ($unit_factor) {
            // handle length factor
            $f = (int) $unit_factor;
            $length = $f > 0 ? ($length * $f) : $length;
        }

        $cable_dao->save_single_value($cable_id, ['length' => $length]);
        $changes['LC__CMDB__CATS__CABLE__LENGTH'] = [
            'from' => $cableRow['isys_catg_cable_list__length'],
            'to'   => $length,
        ];

        $cable_dao->save_single_value($cable_id, ['cable_colour' => $color_id]);
        $changes['LC__CMDB__CATS__CABLE__COLOUR'] = [
            'from' => $cableRow['isys_catg_cable_list__isys_cable_colour__id'],
            'to'   => $color_id,
        ];

        $cable_dao->save_single_value($cable_id, ['cable_type' => $type_id]);
        $changes['LC__CMDB__CATS__CABLE__TYPE'] = [
            'from' => $cableRow['isys_catg_cable_list__isys_cable_type__id'],
            'to'   => $type_id,
        ];

        $this->_logCategoryChanged(
            $cable_id,
            'LC__CATG__CABLE',
            $changes);
    }

    /**
     * Update cable length unit.
     *
     * @param string $unit
     * @return int|null
     * @throws Exception
     */
    private function _get_cable_length_unit_const($unit)
    {
        $lang = isys_application::instance()->container->get('language');
        $unit_const = null;
        switch ($unit) {
            case $lang->get('LC__DEPTH_UNIT__MM'):
                $unit_const = 'C__DEPTH_UNIT__MM';
                break;
            case $lang->get('LC__DEPTH_UNIT__CM'):
                $unit_const = 'C__DEPTH_UNIT__CM';
                break;
            case $lang->get('LC__DEPTH_UNIT__INCH'):
                $unit_const = 'C__DEPTH_UNIT__INCH';
                break;
            case $lang->get('LC__DEPTH_UNIT__FOOT'):
                $unit_const = 'C__DEPTH_UNIT__FOOT';
                break;
            case $lang->get('LC__DEPTH_UNIT__METER'):
                $unit_const = 'C__DEPTH_UNIT__METER';
                break;
            case $lang->get('LC__DEPTH_UNIT__KILOMETER'):
                $unit_const = 'C__DEPTH_UNIT__KILOMETER';
                break;
        }
        return $unit_const;
    }

    /**
     * Add object comment.
     *
     * @param int $obj_id
     * @param string $comment
     */
    private function _add_object_comment($obj_id, $comment)
    {
        $form_factor_dao = isys_cmdb_dao_smview_patch_formfactor::instance(isys_application::instance()->database);
        $form_factor_row = $form_factor_dao->get_object('obj_id', $obj_id);
        if (! $form_factor_row) {
            $form_factor_row = array(
                'obj_id'  => $obj_id,
                'comment' => $comment,
            );
            $form_factor_dao->create_object($form_factor_row);
        } else {
            $form_factor_row['comment'] = $comment;
            $form_factor_dao->update_object($form_factor_row['id'], $form_factor_row);

            $changes = [
                'LC__CMDB__CATG__SMVIEW_PATCH__COMMENT' => [
                    'from' => $form_factor_row['isys_catg_smview_patch_formfactor_list__comment'] ?: '',
                    'to'   => $comment,
                ],
            ];

            $this->_logCategoryChanged(
                $obj_id,
                'LC__SHD__FORMFACTOR',
                $changes);
        }
    }

    /**
     * Set connector status.
     *
     * @param  int $connector_id
     * @param  int $status
     * @throws isys_exception_validation
     */
    private function _set_connector_status($connector_id, $status)
    {
        $dao = isys_cmdb_dao_smview_patch_grid_connector::instance(isys_application::instance()->database);
        $row = $dao->get_object('connector_id', $connector_id);
        if (empty($row)) {
            return;
        }
        $row['connector_status'] = $status;
        $dao->update_object($row['id'], $row);
    }

    /**
     * Get all object types having some configured connectors.
     *
     * @param  int|null $object_type_id
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _get_object_types($object_type_id = null, $connection_type = 'patch')
    {
        $lang = isys_application::instance()->container->get('language');
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $dao = new isys_cmdb_dao_object_type(isys_application::instance()->database);
        $res = $dao->retrieve("SELECT t.* FROM isys_obj_type t
                            JOIN isys_obj o ON o.isys_obj__isys_obj_type__id = t.isys_obj_type__id
                            JOIN isys_catg_connector_list l ON o.isys_obj__id = l.isys_catg_connector_list__isys_obj__id
                            GROUP BY t.isys_obj_type__id;");
        $object_types = [];
        while ($row = $res->get_row()) {
            $condition = $this->_get_condition_by_connection_type($connection_type);
            $condition .= ' AND isys_obj_type__id = ' .  $row['isys_obj_type__id'];

            $l_sql = "SELECT COUNT(isys_catg_connector_list.isys_catg_connector_list__id) AS count FROM isys_catg_connector_list
                LEFT JOIN isys_connection_type ON isys_catg_connector_list.isys_catg_connector_list__isys_connection_type__id = isys_connection_type__id
                INNER JOIN isys_obj ON isys_catg_connector_list.isys_catg_connector_list__isys_obj__id = isys_obj.isys_obj__id
                INNER JOIN isys_obj_type ON isys_obj.isys_obj__isys_obj_type__id = isys_obj_type__id
                WHERE TRUE " . $condition;
            $count_row = $connector_dao->retrieve($l_sql)->get_row();
            $count = (int) $count_row['count'];
            if ($count > 0 || $object_type_id == $row['isys_obj_type__id']) {
                $object_types[$row['isys_obj_type__id']] = $lang->get($row['isys_obj_type__title']);
            }
        }

        if ($connection_type == 'link') {
            // todo: handle FM-Cable
        }
        return $object_types;
    }

    /**
     * Get objects by object type having some configured connectors.
     *
     * @param  int $object_type_id
     * @param  int|null $object_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_objects_by_object_type_id($object_type_id, $object_id = null, $connection_type = 'patch')
    {
        $condition = $this->_get_condition_by_connection_type($connection_type);
        $condition .= ' AND isys_obj_type__id = ' .  $object_type_id;
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $res = $connector_dao->get_data(null, null, $condition);
        $objects = [];
        while ($row = $res->get_row()) {
            if (empty($row['con_connector']) || $object_id == $row['isys_obj__id']) {
                $path = $this->get_location_path($row['isys_obj__id']);
                $objects[$row['isys_obj__id']] = $row['isys_obj__title'] . " ($path)";
            }
        }

        if ($connection_type == 'link') {
            // todo: handle FM-Cable
        }
        return $objects;
    }

    /**
     * Get location path by object ID.
     *
     * @param  int $obj_id
     * @return string
     * @throws isys_exception_database
     */
    public function get_location_path($obj_id)
    {
        $location_dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $location_path_ids = $location_dao->get_location_path($obj_id);
        $path = [];
        $strLen = 0;
        foreach ($location_path_ids as $i) {
            if (isset($this->_cached_locations[$i])) {
                $loc = $this->_cached_locations[$i];
            } else {
                $loc = $location_dao->get_object_by_id($i)->get_row();
                $this->_cached_locations[$i] = $loc;
            }

            $path[] = $loc['isys_obj__title'];
            $strLen += strlen($loc['isys_obj__title']);
            if ($strLen > 50) {
                break;
            }
        }

        $path = array_reverse($path);
        return implode(' > ', $path);
    }

    /**
     * Get connectors by object ID or port ID.
     *
     * @param  int|null $object_id
     * @param  int|null $port_id
     * @param  int|null $selected_port_id
     * @return array
     */
    private function _get_ports($object_id = null, $port_id = null, $selected_port_id = null, $connection_type = 'patch')
    {
        $condition = $this->_get_condition_by_connection_type($connection_type);
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $res = $connector_dao->get_data($port_id, $object_id, $condition); //'AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT);
        $ports = [];
        while ($row = $res->get_row()) {
            if (! empty($row['con_connector']) && empty($port_id) && ($row['isys_catg_connector_list__id'] != $selected_port_id)) {
                continue;
            }

            $ports[$row['isys_catg_connector_list__id']] = [
                'object_type_id' => $row['isys_obj_type__id'],
                'object_id'      => $row['isys_obj__id'],
                'id'             => $row['isys_catg_connector_list__id'],
                'title'          => $row['isys_catg_connector_list__title'],
                'category'       => $row['isys_catg_connector_list__assigned_category'],
                'type'           => $row['isys_connection_type__title'],
                'status'         => $row['isys_obj__isys_cmdb_status__id'],
            ];
        }

        if ($connection_type == 'link') {
            // todo: handle FM-Cable
        }
        return $ports;
    }

    private function _get_condition_by_connection_type($connection_type)
    {
        if ($connection_type == 'patch') {
            // handle patch
            $condition  = ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__OUTPUT;
            $condition .= ' AND (isys_catg_connector_list.isys_catg_connector_list__assigned_category = "C__CATG__CONNECTOR"';
            $condition .= ' OR isys_catg_connector_list.isys_catg_connector_list__assigned_category = "C__CATG__NETWORK_PORT")';
        } else {
            // handle link
            $condition  = ' AND isys_catg_connector_list.isys_catg_connector_list__assigned_category = "C__CATG__CONNECTOR"';
            $condition .= ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT;
        }
        return $condition;
    }

    /**
     * Save end device data.
     *
     * @param string|null $name
     * @param string|null $plannedName
     * @throws Exception
     */
    private function _save_end_device($name = null, $plannedName = null)
    {
        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
        if (empty($obj_id)) {
            return;
        }

        $end_device_dao = new isys_cmdb_dao_category_g_smview_patch_end_device(isys_application::instance()->database);
        $end_device = $end_device_dao->get_data(null, $obj_id)->get_row();
        if (empty($end_device)) {
            $end_device_dao->create_data([
                'isys_obj__id' => $obj_id,
                'name'         => $name,
                'planned_name' => $plannedName,
            ]);
        } else {
            $changes = [];
            if (isset($name)) {
                $end_device_dao->save_single_value($obj_id, ['name' => $name]);
                $changes['LC__CMDB__CATG__SMVIEW_PATCH__NAME'] = [
                    'from' => $end_device["isys_catg_smview_patch_end_device_list__name"] ?: '',
                    'to'   => $name,
                ];
            }
            if (isset($plannedName)) {
                $end_device_dao->save_single_value($obj_id, ['planned_name' => $plannedName]);
                $changes['LC__CMDB__CATG__SMVIEW_PATCH__END_DEVICE_PLANNED_NAME'] = [
                    'from' => $end_device["isys_catg_smview_patch_end_device_list__planned_name"] ?: '',
                    'to'   => $plannedName,
                ];
            }
            $this->_logCategoryChanged(
                $obj_id,
                'LC__CMDB__CATG__SMVIEW_PATCH__END_DEVICE_TITLE',
                $changes);
        }
    }

    /**
     * Log category changed event.
     *
     * @param int $objId
     * @param string $catConstLang
     * @param $changes
     * @throws Exception
     */
    protected function _logCategoryChanged($objId, $catConstLang, $changes)
    {
        $language = isys_application::instance()->container->get('language');
        $filteredChanges = [];
        foreach ($changes as $property => $change) {
            if ($change['from'] != $change['to']) {
                $property = $language->get($property);
                $filteredChanges[$property] = $change;
            }
        }

        if (empty($filteredChanges)) {
            return;
        }

        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $objTypeId = $dao->get_objTypeID($objId);
        isys_event_manager::getInstance()
            ->triggerCMDBEvent(
                'C__LOGBOOK_EVENT__CATEGORY_CHANGED',
                '',
                $objId,
                $objTypeId,
                $catConstLang ? $language->get($catConstLang) : null,
                serialize($filteredChanges),
            );

        // update last change date
        $dao->update_object($objId, null, null, null, null, null, null, null, null, null, time());
    }
}
