<?php

/**
 * SM-VIEW Patch Grid Template AJAX handler.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_ajax_handler_patch_grid_template extends isys_ajax_handler
{
    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');
        $lang = isys_application::instance()->container->get('language');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $grid_template_id = ! empty($this->m_post['grid_template_id']) ? $this->m_post['grid_template_id'] : $this->m_get['grid_template_id'];
            switch ($this->m_get['func']) {
            case 'get-template':
                if (isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, "CATEGORY/C__CATG__SMVIEW_PATCH_GRID")) {
                    $l_return['data'] = $this->get_grid_template($grid_template_id);
                }
                break;
            case 'delete-template':
                if (isys_auth_cmdb::instance()->is_allowed_to(isys_auth::DELETE, "CATEGORY/C__CATG__SMVIEW_PATCH_GRID")) {
                    $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template(isys_application::instance()->database);
                    $success = $grid_template_dao->delete_object($grid_template_id);
                    $l_return['data'] = $success;
                    if ($success) {
                        $data = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__GRID_TEMPLATE_DELETED');
                        isys_notify::success($data);
                    }
                }
                break;
            case 'set-size':
                $sizes = isys_cmdb_dao_smview_patch_grid_template::get_available_sizes();
                $size = ! empty($this->m_post['size']) ? $this->m_post['size'] : $this->m_get['size'];
                $size = isset($sizes[$size]) ? $size : 'm';
                isys_usersettings::set('smview_patch.editor.icon_size', $size);
                break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate() 
    {
        return true;
    }

    /**
     * Get grid template by ID.
     *
     * @param  int $grid_template_id
     * @return array
     * @throws isys_exception_database
     */
    private function get_grid_template($grid_template_id) 
    {
        if (empty($grid_template_id)) {
            return [];
        }
        
        $dao = isys_cmdb_dao_category_g_smview_patch_grid::instance(isys_application::instance()->database);
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template(isys_application::instance()->database);
        $grid_template = $grid_template_dao->get_object('id', $grid_template_id);

        $grid['connectors'] = $this->_get_connectors_by_grid_template_id($grid_template_id);
        $grid['grid'] = [
            'title'             => $grid_template['title'],
            'column_count'      => (int) $grid_template['column_count'],
            'row_count'         => (int) $grid_template['row_count'],
            'column_count_back' => (int) $grid_template['column_count_back'],
            'row_count_back'    => (int) $grid_template['row_count_back'],
            'is_template'       => $grid_template['is_template'],
            'used_by'           => $dao->get_objects_using_grid_template($grid_template_id),
        ];
        return $grid;
    }

    /**
     * Get configured connectors by grid template ID.
     *
     * @param  int $grid_template_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_connectors_by_grid_template_id($grid_template_id) {
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template(isys_application::instance()->database);
        $picture_dao = new isys_cmdb_dao_smview_patch_picture(isys_application::instance()->database);
        $connector_templates = $connector_template_dao->filter_objects(array('grid_template_id' => $grid_template_id));
        $connectors = [];
        foreach ($connector_templates as $p) {
            $pic = $picture_dao->get_object('id', $p['connector_type_id']);
            $connector = $p;
            $connector['connector_id'] = null;
            $connector['connector_type_title'] = ! empty($pic) ? $pic['title'] : null;
            
            $url = ! empty($pic) ? $picture_dao->get_file_url($pic['obj_id']) : null;
            $connector['url'] = $url;
            $connectors[] = $connector;
        }
        return $connectors;
    }
}
