<?php

/**
 * SM-VIEW Patch Parking Deck AJAX handler.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_ajax_handler_patch_parking_deck extends isys_ajax_handler
{
    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $parking_deck_dao = new isys_cmdb_dao_category_g_smview_patch_parking_deck($this->m_database_component);
            $canEdit = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_PATCH_PARKING_DECK");
            switch ($this->m_get['func']) {
                case 'park':
                    if ($canEdit) {
                        $obj_id = ! empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $title = ! empty($this->m_post['title']) ? $this->m_post['title'] : $this->m_get['title'];
                        $connector_id = ! empty($this->m_post['connector_id']) ? $this->m_post['connector_id'] : $this->m_get['connector_id'];
                        $l_return['data'] = $parking_deck_dao->park_connector($connector_id, $obj_id, $title);
                    }
                    break;
                case 'park_all':
                    if ($canEdit) {
                        $obj_id = ! empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $title = ! empty($this->m_post['title']) ? $this->m_post['title'] : $this->m_get['title'];
                        $connector_ids = ! empty($this->m_post['connector_ids']) ? $this->m_post['connector_ids'] : $this->m_get['connector_ids'];
                        $l_return['data'] = $this->_parkAllConnectors($connector_ids, $obj_id, $title);
                    }
                    break;
                case 'connect':
                    if ($canEdit) {
                        $parked_connector_id = !empty($this->m_post['parked_connector_id']) ? $this->m_post['parked_connector_id'] : $this->m_get['parked_connector_id'];
                        $local_connector_id = !empty($this->m_post['local_connector_id']) ? $this->m_post['local_connector_id'] : $this->m_get['local_connector_id'];
                        $mode = !empty($this->m_post['mode']) ? $this->m_post['mode'] : $this->m_get['mode'];
                        $l_return['data'] = $parking_deck_dao->connect_connector($parked_connector_id, $local_connector_id, $mode);
                    }
                    break;
                case 'change-title':
                    if ($canEdit) {
                        $global_dao = new isys_cmdb_dao_category_g_global($this->m_database_component);
                        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $title = !empty($this->m_post['title']) ? $this->m_post['title'] : $this->m_get['title'];
                        $l_return['data'] = $global_dao->save_title($obj_id, $title);
                    }
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);
        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }

    /**
     * Park all given connectors.
     *
     * @param  string $connector_ids
     * @param  int|null $obj_id
     * @param  string|null $title
     * @return array
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _parkAllConnectors($connector_ids, $obj_id = null, $title = null)
    {
        $parking_deck_dao = new isys_cmdb_dao_category_g_smview_patch_parking_deck($this->m_database_component);
        $connectors = explode(',', $connector_ids);
        $ret = null;
        foreach ($connectors as $connector_id) {
            $ret = $parking_deck_dao->park_connector($connector_id, $obj_id, $title);
            if ($obj_id == null || $title == null) {
                $obj_id = $ret['obj_id'];
                $title = $ret['obj_title'];
            }
        }
        return $ret;
    }
}
