<?php

/**
 * SM-VIEW Patch Grid AJAX handler.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_ajax_handler_patch_pin_mapper extends isys_ajax_handler
{
    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $obj_id = ! empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
            $canView = isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::VIEW, $obj_id, 'C__CATG__SMVIEW_PATCH_PIN_MAPPER');
            $canEdit = isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::EDIT, $obj_id, 'C__CATG__SMVIEW_PATCH_PIN_MAPPER');
            switch ($this->m_get['func']) {
                case 'get-connectors':
                    if ($canView) {
                        $l_return['data'] = $this->_get_connected_cable_pins($obj_id);
                    }
                    break;
                case 'get-objects':
                    if ($canView) {
                        $obj_type_id = !empty($this->m_post['obj_type_id']) ? $this->m_post['obj_type_id'] : $this->m_get['obj_type_id'];
                        $l_return['data'] = $this->_get_objects_by_type($obj_type_id);
                    }
                    break;
                case 'get-object':
                    if ($canView) {
                        $ref_obj_id = !empty($this->m_post['ref_obj_id']) ? $this->m_post['ref_obj_id'] : $this->m_get['ref_obj_id'];
                        $l_return['data'] = $this->_get_cable_pin_mapping($obj_id, $ref_obj_id);
                    }
                    break;
                case 'assign-connector':
                    if ($canEdit) {
                        $pin_id = !empty($this->m_post['pin_id']) ? $this->m_post['pin_id'] : $this->m_get['pin_id'];
                        $connector_id = !empty($this->m_post['connector_id']) ? $this->m_post['connector_id'] : $this->m_get['connector_id'];
                        $this->_assign_connector($pin_id, $connector_id);
                    }
                    $l_return['data'] = null;
                    break;
                case 'add-connector':
                    if ($canEdit) {
                        $pin_id = !empty($this->m_post['pin_id']) ? $this->m_post['pin_id'] : $this->m_get['pin_id'];
                        $connector_type_id = !empty($this->m_post['connector_type_id']) ? $this->m_post['connector_type_id'] : $this->m_get['connector_type_id'];
                        $this->_add_connector($obj_id, $pin_id, $connector_type_id);
                    }
                    $l_return['data'] = null;
                    break;
                case 'add-all-connectors':
                    if ($canEdit) {
                        $ref_obj_id = !empty($this->m_post['ref_obj_id']) ? $this->m_post['ref_obj_id'] : $this->m_get['ref_obj_id'];
                        $connector_type_id = !empty($this->m_post['connector_type_id']) ? $this->m_post['connector_type_id'] : $this->m_get['connector_type_id'];
                        $this->_add_all_connectors($obj_id, $ref_obj_id, $connector_type_id);
                    }
                    $l_return['data'] = null;
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }

    /**
     * Get connected cable pins.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_connected_cable_pins($obj_id)
    {
        $connections = [];
        $dao = new isys_cmdb_dao_smview_cable_pin(isys_application::instance()->database);
        $global_dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $pin_dao = new isys_cmdb_dao_category_g_smview_patch_pin_mapper(isys_application::instance()->database);
        $connectors = $pin_dao->get_incoming_connectors($obj_id);
        foreach ($connectors as $connector) {
            $cable_connectors = $dao->filter_objects(array('connector_id' => $connector['connector_id']));

            $pins = [];
            foreach ($cable_connectors as $c) {
                $res = $global_dao->get_object($c['obj_id']);
                $obj = $res->get_row();
                $url = isys_helper_link::get_base() . isys_helper_link::create_url(
                    [
                        C__CMDB__GET__OBJECT => $c['obj_id'],
                        C__CMDB__GET__CATG   => defined_or_default('C__CATG__SMVIEW_CABLE_PIN')
                    ]
                );
                $pin = [
                    'obj_id'      => $c['obj_id'],
                    'obj_title'   => $obj['isys_obj__title'],
                    'description' => $c['description'],
                    'scope'       => $c['classification'],
                    'url'         => $url,
                ];
                $pins[] = $pin;
            }
            $connector['pins'] = $pins;
            $connections[] = $connector;
        }
        return $connections;
    }

    /**
     * Get objects by object type, having cable pins.
     *
     * @param  int $obj_type_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_objects_by_type($obj_type_id)
    {
        $global_dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $type = $global_dao->convert_sql_id($obj_type_id);
        $sql = "SELECT isys_obj__id, isys_obj__title FROM isys_obj o
                JOIN isys_obj_type t ON o.isys_obj__isys_obj_type__id = t.isys_obj_type__id
                JOIN isys_catg_smview_cable_pin_list p ON isys_obj__id = isys_catg_smview_cable_pin_list__isys_obj__id
                WHERE isys_obj_type__id = $type
                GROUP BY isys_obj__id;";
        $res = $global_dao->retrieve($sql);
        $matches = [];

        $handler = new isys_ajax_handler_patch_grid(null, null);
        while ($row = $res->get_row()) {
            try {
                $location = '  @' . $handler->get_location_path($row['isys_obj__id']);
            } catch (isys_exception_database $e) {
                $location = '';
            }
            $matches[$row['isys_obj__id']] = $row['isys_obj__title'] . $location;
        }
        return $matches;
    }

    /**
     * Get cable pin to connector mapping.
     *
     * @param  int $obj_id
     * @param  int $ref_obj_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_cable_pin_mapping($obj_id, $ref_obj_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $dao = isys_cmdb_dao_category_g_smview_patch_pin_mapper::instance(isys_application::instance()->database);
        $connectors = $dao->get_incoming_connectors($obj_id);

        $pin_dao = new isys_cmdb_dao_smview_cable_pin(isys_application::instance()->database);
        $cable_pins = $pin_dao->filter_objects(array('obj_id' => $ref_obj_id, 'type' => isys_cmdb_dao_smview_cable_pin::PIN_TYPE_INPUT));

        $mapping = [];
        foreach ($cable_pins as $pin) {
            $connector = null;
            foreach ($connectors as $con) {
                if ($con['connector_title'] == $pin['description']) {
                    $connector = $con;
                    $connector['type'] = 'matching';
                }
            }

            if ($pin['connector_id']) {
                $l_data = $connector_dao->get_data($pin['connector_id'])->get_row();
                $connector = [
                    'connector_id'         => $pin['connector_id'],
                    'connector_title'      => $l_data['isys_catg_connector_list__title'],
                    'connector_type_id'    => $l_data['isys_connection_type__id'],
                    'connector_type_title' => $l_data['isys_connection_type__title'],
                    'obj_title'            => $l_data['isys_obj__title'],
                    'type'                 => 'assigned',
                ];
            }

            $mapping[] = array(
                'pin'       => $pin,
                'connector' => $connector,
            );
        }
        return $mapping;
    }

    /**
     * Assign existing connector.
     *
     * @param  int $pin_id
     * @param  int $connector_id
     * @throws isys_exception_database
     */
    private function _assign_connector($pin_id, $connector_id)
    {
        $pin_dao = new isys_cmdb_dao_smview_cable_pin(isys_application::instance()->database);
        $pin = $pin_dao->get_object('id', $pin_id);
        if (! empty($pin)) {
            $pin_dao->update_pin(
                $pin_id,
                $pin['description'],
                $pin['number'],
                $pin['type'],
                $pin['internal_pin_id'],
                $pin['patch_pin_id'],
                $connector_id,
                $pin['classification']
            );
        }
    }

    /**
     * Add a new connector.
     *
     * @param  int $obj_id
     * @param  int $pin_id
     * @param  int $connector_type_id
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _add_connector($obj_id, $pin_id, $connector_type_id)
    {
        $pin_dao = new isys_cmdb_dao_smview_cable_pin(isys_application::instance()->database);
        $pin = $pin_dao->get_object('id', $pin_id);
        if (empty($pin)) {
            return;
        }
        $this->_create_connector($obj_id, $pin, $connector_type_id);
    }

    /**
     * Add all new connectors.
     *
     * @param  int $obj_id
     * @param  int $ref_obj_id
     * @param  int $connector_type_id
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _add_all_connectors($obj_id, $ref_obj_id, $connector_type_id)
    {
        $connectors = $this->_get_cable_pin_mapping($obj_id, $ref_obj_id);
        foreach ($connectors as $m) {
            $c = $m['connector'];
            if ($c['type'] && ($c['type'] == 'assigned' || $c['type'] == 'matching')) {
                continue;
            }
            $this->_create_connector($obj_id, $m['pin'], $connector_type_id);
        }
    }

    /**
     * Create connector.
     *
     * @param  int $obj_id
     * @param  array $pin
     * @param  int|null $connector_type_id
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _create_connector($obj_id, $pin, $connector_type_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $connector_in_id = $connector_dao->create(
            $obj_id,
            C__CONNECTOR__INPUT,
            null,
            $connector_type_id,
            $pin['description']);
        if ($connector_in_id) {
            $this->_assign_connector($pin['id'], $connector_in_id);
            $connector_dao->create(
                $obj_id,
                C__CONNECTOR__OUTPUT,
                null,
                $connector_type_id,
                $pin['description'],
                null,
                $connector_in_id);
        }
    }
}
