<?php

/**
 * i-doit
 *
 * Class autoloader.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_module_smview_patch_autoload extends isys_module_manager_autoload
{
    /**
     * Autoloader
     *
     * @param  string $className
     * @return boolean
     */
    public static function init($className)
    {
        $addOnPath = '/src/classes/modules/smview_patch/';
        $class_loader_paths = array(
            '/api/model/',
            '/api/model/patch/',
            '/auth/',
            '/cmdb/dao/',
            '/cmdb/dao/global/',
            '/cmdb/dao/list/',
            '/cmdb/ui/global/',
            '/handler/controller/',
        );

        //dynamic loading of classes
        foreach ($class_loader_paths as $clp) {
            $l_path = $clp . $className . '.class.php';
            if (! file_exists(__DIR__ . $l_path)) {
                continue;
            }

            if (parent::include_file($addOnPath . $l_path)) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_path);

                return true;
            }
        }
        
        $l_classes = [
            'isys_ajax_handler_patch_analyzer'                                => '/handler/ajax/isys_ajax_handler_patch_analyzer.class.php',
            'isys_ajax_handler_patch_grid'                                    => '/handler/ajax/isys_ajax_handler_patch_grid.class.php',
            'isys_ajax_handler_patch_grid_template'                           => '/handler/ajax/isys_ajax_handler_patch_grid_template.class.php',
            'isys_ajax_handler_patch_parking_deck'                            => '/handler/ajax/isys_ajax_handler_patch_parking_deck.class.php',
            'isys_ajax_handler_patch_pin_mapper'                              => '/handler/ajax/isys_ajax_handler_patch_pin_mapper.class.php',
            'idoit\Module\Patch\DependentObjectManager\AbstractManager'       => '/src/DependentObjectManager/AbstractManager.php',
            'idoit\Module\Patch\DependentObjectManager\AnalyzerPresetManager' => '/src/DependentObjectManager/AnalyzerPresetManager.php',
            'idoit\Module\Patch\DependentObjectManager\ImageMappingManager'   => '/src/DependentObjectManager/ImageMappingManager.php',
            'idoit\Module\Patch\Exchange\GridExporter'                        => '/src/Exchange/GridExporter.php',
            'idoit\Module\Patch\Exchange\GridImporter'                        => '/src/Exchange/GridImporter.php',
            'idoit\Module\Patch\Analyzer'                                     => '/src/Analyzer.php',
            'idoit\Module\Patch\Importer\Duplicator'                          => '/src/Importer/Duplicator.php',
            'idoit\Module\Patch\Importer\Excel'                               => '/src/Importer/Excel.php',
            'idoit\Module\Patch\Importer\ImporterException'                   => '/src/Importer/ImporterException.php',
            'idoit\Module\Smview_Patch\Console\Command\TestDataGeneratorCommand'     => '/src/Console/Command/TestDataGeneratorCommand.php',
        ];

        // include composer's autoloader
        $vendorDir = dirname(__FILE__) . '/vendor/';
        if (file_exists($vendorDir . 'autoload.php')) {
            include_once($vendorDir . 'autoload.php');
        }
        unset($vendorDir);
        
        if (isset($l_classes[$className])) {
            if (parent::include_file($addOnPath . $l_classes[$className])) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_classes[$className]);

                return true;
            }
        }
        return false;
    }
}
