import * as React from 'react'
import { SelectableTable } from '../SelectableTable'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_PATCH__ALTERNATIVE_TITLE: 'Name of Parking Deck Object',
  LC__CMDB__CATG__SMVIEW_PATCH__LOCAL_OBJECT: 'Local Object',
  LC__CMDB__CATG__SMVIEW_PATCH__PARKING_DECK_OBJECT: 'Parking Deck Object',
  LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED_WITH: 'connected with',
  LC__CMDB__CATG__SMVIEW_PATCH__ASSIGN_CONNECTOR: 'Assign',
  LC__CMDB__CATG__SMVIEW_PATCH__OVERWRITE_CONNECTOR: 'Overwrite',
  LC__CMDB__CATG__SMVIEW_PATCH__PARK_CONNECTOR: 'Park'
}

const localObject = {
  obj_id: 11,
  obj_title: 'Switch01',
  obj_type_title: 'Switch'
}

const local_connectors = [
  {
    connector_id: 111,
    connector_title: '01',
    connector_type: 1,
    connection: {
      object_type_title: 'Patch Panel',
      object_title: 'Patch01',
      connector_title: '03',
      connector_url: 'http://url'
    }
  },
  {
    connector_id: 133,
    connector_title: '02',
    connector_type: 2,
    connection: {
      object_type_title: 'Patch Panel',
      object_title: 'Patch08',
      connector_title: '01',
      connector_url: 'http://url'
    }
  },
  {
    connector_id: 155,
    connector_title: '03',
    connector_type: 1,
    connection: {
      object_type_title: null,
      object_title: null,
      connector_title: null,
      connector_url: null
    }
  },
  {
    connector_id: 177,
    connector_title: null,
    connector_type: 1,
    connection: {
      object_type_title: null,
      object_title: null,
      connector_title: null,
      connector_url: null
    }
  }
]

it('renders selectable table read-only mode correctly', () => {
  const selectionMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
    <SelectableTable language={lang}
      title="Test"
      object={localObject}
      isEditable={false}
      side="local"
      rows={local_connectors}
      selected_connector={111}
      onSelectionChange={selectionMockFunction}
      onTitleChange={null} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const tds = instance.findAllByType('td')
  const td = tds[1]
  td.props.onClick()
  expect(selectionMockFunction).toHaveBeenCalledWith('local', 133)
})

it('renders selectable table edit mode correctly', () => {
  const titleMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
    <SelectableTable language={lang}
      title="Test"
      object={localObject}
      isEditable={true}
      side="local"
      rows={local_connectors}
      selected_connector={111}
      onSelectionChange={null}
      onTitleChange={titleMockFunction} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const input = instance.findByProps({ type: 'text' })
  input.props.onChange({
    currentTarget: { value: 'Title01' }
  })
  expect(titleMockFunction).toHaveBeenCalledWith('Title01')
})

it('renders selectable table without connectors correctly', () => {
  const component = ReactTestRenderer.create(
    <SelectableTable language={lang}
      title="Test"
      object={localObject}
      isEditable={true}
      side="local"
      rows={[]}
      selected_connector={111}
      onSelectionChange={null} />
  )
  expect(component).toMatchSnapshot()
})
