<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Patch Parking Deck.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_patch_parking_deck extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_patch::get_tpl_dir() . 'module/cmdb/catg__smview_patch_parking_deck.tpl');
    }

    /**
     * Processes category data list for multi-valued categories.
     *
     * @param  isys_cmdb_dao_category $p_cat Category's DAO
     * @param  array $p_get_param_override
     * @param  string $p_strVarName
     * @param  string $p_strTemplateName
     * @param  boolean $p_bCheckbox
     * @param  boolean $p_bOrderLink
     * @param  string $p_db_field_name
     * @return null
     * @throws isys_exception_general
     * @throws isys_exception_cmdb
     */
    public function process_list(
        isys_cmdb_dao_category &$p_cat,
        $p_get_param_override = null,
        $p_strVarName = null,
        $p_strTemplateName = null,
        $p_bCheckbox = true,
        $p_bOrderLink = true,
        $p_db_field_name = null
    ) {
        $lang = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_category_g_connector($this->m_database_component);
        $connector_count = $dao->get_data(null, $p_cat->get_object_id())->count();
        if ($connector_count > 0) {
            // if object has connectors display park- and merge-button
            isys_component_template_navbar::getInstance()
                ->set_active(isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_PATCH_PARKING_DECK"),
                    defined_or_default('C__NAVBAR_BUTTON__NEW'))
                ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__NEW'))
                ->set_title($lang->get('LC__CMDB__CATG__SMVIEW_PATCH__PARKING_DECK_BUTTON_NEW_PARK_OBJECT'), defined_or_default('C__NAVBAR_BUTTON__NEW'));

            isys_component_template_navbar::getInstance()
                ->set_title($lang->get('LC__CMDB__CATG__SMVIEW_PATCH__PARKING_DECK_BUTTON_MERGE_PARK_OBJECT'), defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        } else {
            isys_component_template_navbar::getInstance()
                ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__NEW'))
                ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__NEW'));

            isys_component_template_navbar::getInstance()
                ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        }
        return parent::process_list($p_cat, $p_get_param_override, $p_strVarName, $p_strTemplateName, $p_bCheckbox, $p_bOrderLink, $p_db_field_name);
    }

    /**
     * Processes the UI for the category
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $dao = new isys_cmdb_dao_category_g_connector($this->m_database_component);
        $l_cat_data = $p_cat->get_general_data();
        $list_id = $l_cat_data['isys_catg_smview_patch_parking_deck_info_list__id'];

        $isMergeView = false;
        $localObject = $this->_get_object_information($p_cat->get_object_id());
        $parkedObject = [];
        $parkedConnectors = [];
        $l_rules = [];
        if ($list_id) {
            // merge current park object
            $this->get_template_component()->assign('park_deck_object', $l_cat_data);
            $isMergeView = true;

            // add connectors of parking deck object
            $parking_obj_id = $l_cat_data['isys_catg_smview_patch_parking_deck_info_list__isys_obj__id'];
            $parkedObject = $this->_get_object_information($parking_obj_id);
            $parkedConnectors = $dao->get_data_as_array(null, $parking_obj_id);
        } else {
            // add default data
            $l_rules['C__CATG__SMVIEW_PATCH_PARKING_DECK_INFO_OBJECT_TITLE']['p_strValue'] = $localObject['obj_title'];
            $l_rules['C__CATG__SMVIEW_PATCH_PARKING_DECK_INFO_OBJECT_TYPE_CONST']['p_strValue'] = $localObject['obj_type_const'];
            $l_rules['C__CATG__SMVIEW_PATCH_PARKING_DECK_INFO_CREATION_DATE']['p_strValue'] = date('Y-m-d H:i:s');
        }

        $isEditable = false;
        if ($_POST[C__GET__NAVMODE] && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')) {
            $isEditable = true;
        }
        if ($_POST[C__GET__NAVMODE] && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__NEW')) {
            $isEditable = true;
        }
        if ($_GET[C__CMDB__GET__EDITMODE] && $_GET[C__CMDB__GET__EDITMODE] == defined_or_default('C__EDITMODE__ON')) {
            $isEditable = true;
        }
        if ($_POST[C__GET__NAVMODE] && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            // disable edit mode after save
            $isEditable = false;
        }
        if (! isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_PATCH_PARKING_DECK")) {
            $isEditable = false;
            $isMergeView = false;
        }

        $parking_deck_dao = new isys_cmdb_dao_category_g_smview_patch_parking_deck($this->m_database_component);
        $connectors = $dao->get_data_as_array(null, $p_cat->get_object_id());

        // create JS configuration
        $js_config = [
            'ajax_url'          => isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'patch_parking_deck']),
            'session'           => $_COOKIE['PHPSESSID'],
            'spinner_url'       => isys_helper_link::get_base() . '/images/icons/silk/',
            'local_object'      => $localObject,
            'parked_object'     => $parkedObject,
            'local_connectors'  => $parking_deck_dao->get_connectors($connectors),
            'parked_connectors' => $parking_deck_dao->get_connectors($parkedConnectors, true),
            'language'          => $this->_get_js_translations(),
            'is_editable'       => $isEditable,
            'is_merge_view'     => $isMergeView,
        ];

        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_patch::get_assets_www_dir())
            ->assign('js_config', json_encode($js_config))
            ->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);

        $this->deactivate_commentary();
        $this->disable_ajax_save();
    }

    /**
     * Get object information.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_object_information($obj_id)
    {
        $lang = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_category_g_global($this->m_database_component);
        $obj = $dao->get_object($obj_id)->get_row();
        return [
            'obj_id'         => $obj['isys_obj__id'],
            'obj_title'      => $obj['isys_obj__title'],
            'obj_type_title' => $lang->get($obj['isys_obj_type__title']),
            'obj_type_const' => $obj['isys_obj_type__const'],
        ];
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_PATCH__LOCAL_OBJECT',
            'LC__CMDB__CATG__SMVIEW_PATCH__PARKING_DECK_OBJECT',
            'LC__CMDB__CATG__SMVIEW_PATCH__ALTERNATIVE_TITLE',
            'LC__CMDB__CATG__SMVIEW_PATCH__CONNECTED_WITH',
            'LC__CMDB__CATG__SMVIEW_PATCH__PARK_CONNECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__PARK_ALL_LINK_CONNECTORS',
            'LC__CMDB__CATG__SMVIEW_PATCH__PARK_ALL_PATCH_CONNECTORS',
            'LC__CMDB__CATG__SMVIEW_PATCH__ASSIGN_CONNECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__OVERWRITE_CONNECTOR',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $lang = isys_application::instance()->container->get('language');
        return $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__PARKING_DECK_TITLE');
    }
}
